@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -29,21 +27,19 @@
  *
  *	This include file defines C calling sequence defines
  *	for ppc port.
- *
- * HISTORY
- * 20-May-97  Umesh Vaishampayan  (umeshv@apple.com)
- *	Added C_RED_ZONE.
- * 29-Dec-96  Umesh Vaishampayan  (umeshv@NeXT.com)
- *	Ported from m98k.
- * 11-June-91  Mike DeMoney (mike@next.com)
- *	Created.
  */
 
 #ifndef	_ARCH_PPC_CFRAME_H_
 #define	_ARCH_PPC_CFRAME_H_
 
+#if defined (__ppc64__)
+#define	C_ARGSAVE_LEN	64	/* at least 64 bytes of arg save */
+#define	C_STACK_ALIGN	32	/* stack must be 32 byte aligned */
+#define	C_RED_ZONE	320	/* 320 bytes to skip over saved registers */
+#else
 #define	C_ARGSAVE_LEN	32	/* at least 32 bytes of arg save */
 #define	C_STACK_ALIGN	16	/* stack must be 16 byte aligned */
-#define	C_RED_ZONE	244	/* 224 bytes to skip over saved registers */
+#define	C_RED_ZONE	224	/* 224 bytes to skip over saved registers */
+#endif
 
 #endif	/* _ARCH_PPC_CFRAME_H_ */
