@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -47,80 +45,111 @@
 struct tty	cons;
 struct tty	*constty;		/* current console device */
 
+int cnopen(__unused dev_t dev, int flag, int devtype, struct proc *pp);
+int cnclose(__unused dev_t dev, int flag, int mode, struct proc *pp);
+int cnread(__unused dev_t dev, struct uio *uio, int ioflag);
+int cnwrite(__unused dev_t dev, struct uio *uio, int ioflag);
+int cnioctl(__unused dev_t dev, int cmd, caddr_t addr, int flg, struct proc *p);
+int cnselect(__unused dev_t dev, int flag, void * wql, struct proc *p);
+
+void slave_cnenable(void);
+
+int alert(
+	__unused int width, 
+	__unused int height, 
+	__unused const char *title, 
+	const char *msg, 
+	int p1, int p2, int p3, int p4, int p5, int p6, int p7, int p8);
+int alert_done(void);
+
 /*ARGSUSED*/
 int
-cnopen(dev, flag, devtype, pp)
-	dev_t dev;
-	int flag, devtype;
-	struct proc *pp;
+cnopen(__unused dev_t dev, int flag, int devtype, struct proc *pp)
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
+	
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_open)(device, flag, devtype, pp));
+	error =  (*cdevsw[major(device)].d_open)(device, flag, devtype, pp);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
 int
-cnclose(dev, flag, mode, pp)
-	dev_t dev;
-	int flag, mode;
-	struct proc *pp;
+cnclose(__unused dev_t dev, int flag, int mode, struct proc *pp)
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_close)(device, flag, mode, pp));
+	error =  (*cdevsw[major(device)].d_close)(device, flag, mode, pp);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
+
+
 }
 
 /*ARGSUSED*/
 int
-cnread(dev, uio, ioflag)
-	dev_t dev;
-	struct uio *uio;
-	int ioflag;
+cnread(__unused dev_t dev, struct uio *uio, int ioflag)
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_read)(device, uio, ioflag));
+	error = (*cdevsw[major(device)].d_read)(device, uio, ioflag);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
 int
-cnwrite(dev, uio, ioflag)
-	dev_t dev;
-	struct uio *uio;
-	int ioflag;
+cnwrite(__unused dev_t dev, struct uio *uio, int ioflag)
 {
     dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-    return ((*cdevsw[major(device)].d_write)(device, uio, ioflag));
+    error =  (*cdevsw[major(device)].d_write)(device, uio, ioflag);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
 int
-cnioctl(dev, cmd, addr, flag, p)
-	dev_t dev;
-	int cmd;
-	caddr_t addr;
-	int flag;
-	struct proc *p;
+cnioctl(__unused dev_t dev, int cmd, caddr_t addr, int flag, struct proc *p)
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
+
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 
 	if (constty)
 	    device = constty->t_dev;
@@ -130,23 +159,26 @@ cnioctl(dev, cmd, addr, flag, p)
 	 * Superuser can always use this to wrest control of console
 	 * output from the "virtual" console.
 	 */
-	if (cmd == TIOCCONS && constty) {
-		int error = suser(p->p_ucred, (u_short *) NULL);
-		if (error)
-			return (error);
+	if ((unsigned) cmd == TIOCCONS && constty) {
+		error = proc_suser(p);
+		if (error) {
+			goto out;
+		}
 		constty = NULL;
-		return (0);
+		error = 0;
+		goto out;
 	}
-	return ((*cdevsw[major(device)].d_ioctl)(device, cmd, addr, flag, p));
+	error =  (*cdevsw[major(device)].d_ioctl)(device, cmd, addr, flag, p);
+out:
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
+/* called with funnel held */
 int
-cnselect(dev, flag, wql, p)
-	dev_t dev;
-	int flag;
-	void * wql;
-	struct proc *p;
+cnselect(__unused dev_t dev, int flag, void * wql, struct proc *p)
 {
 	dev_t device;
 
@@ -162,12 +194,18 @@ int
 cngetc()
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_getc)(device));
+	error =  (*cdevsw[major(device)].d_getc)(device);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
@@ -176,21 +214,26 @@ cnputc(c)
 	char c;
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_putc)(device, c));
+	error =  (*cdevsw[major(device)].d_putc)(device, c);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 #endif
 
-#if	NCPUS > 1
-slave_cnenable()
+void
+slave_cnenable(void)
 {
 	/* FIXME: what to do here? */
 }
-#endif	NCPUS > 1
 
 #if 0
 void
@@ -210,9 +253,9 @@ kprintf( const char *format, ...)
  */
 int 
 alert(
-	int width, 
-	int height, 
-	const char *title, 
+	__unused int width, 
+	__unused int height, 
+	__unused const char *title, 
 	const char *msg, 
 	int p1, 
 	int p2, 
@@ -236,7 +279,7 @@ alert(
 }
 
 int 
-alert_done()
+alert_done(void)
 {
 	/* DoRestore(); */
 	return 0;
