@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -41,10 +42,25 @@
 #include <dev/kmreg_com.h>
 #include <pexpert/pexpert.h>
 
+extern int hz;
+
+extern void cnputcusr(char);
+extern int  cngetc(void);
+
+void	kminit(void);
+int	kmopen(dev_t dev, int flag, int devtype, struct proc *pp);
+int	kmclose(dev_t dev, int flag, int mode, struct proc *p);
+int	kmread(dev_t dev, struct uio *uio, int ioflag);
+int	kmwrite(dev_t dev, struct uio *uio, int ioflag);
+int	kmioctl(dev_t dev, int cmd, caddr_t data, int flag, struct proc *p);
+int	kmputc(int c);
+int	kmgetc(dev_t dev);
+int	kmgetc_silent(dev_t dev);
+void	cons_cinput(char ch);
+
 /*
  * 'Global' variables, shared only by this file and conf.c.
  */
-extern struct tty	cons;
 struct tty *km_tty[1] = { &cons };
 
 /*
@@ -63,9 +79,10 @@ static void kmstart(struct tty *tp);
 
 extern void KeyboardOpen(void);
 
-int kminit()
+void
+kminit(void)
 {
-   	 cons.t_dev = makedev(12, 0);
+   	cons.t_dev = makedev(12, 0);
 	initialized = 1;
 }
 /*
@@ -75,10 +92,9 @@ int
 kmopen(
 	dev_t dev, 
 	int flag,
-	int devtype, 
+	__unused int devtype, 
 	struct proc *pp)
 {
-	int rtn;
 	int unit;
 	struct tty *tp;
 	struct winsize *wp;
@@ -101,7 +117,7 @@ kmopen(
 		tp->t_ispeed = tp->t_ospeed = TTYDEF_SPEED;
 		termioschars(&tp->t_termios);
 		ttsetwater(tp);
-	} else if ((tp->t_state & TS_XCLUDE) && pp->p_ucred->cr_uid != 0)
+	} else if ((tp->t_state & TS_XCLUDE) && proc_suser(pp))
 		return EBUSY;
 
 	tp->t_state |= TS_CARR_ON; /* lie and say carrier exists and is on. */
@@ -133,10 +149,10 @@ kmopen(
 
 int 
 kmclose(
-	dev_t dev, 
+	__unused dev_t dev, 
 	int flag,
-	int mode,
-	struct proc *p)
+	__unused int mode,
+	__unused struct proc *p)
 {
 	 
 	struct tty *tp;
@@ -149,7 +165,7 @@ kmclose(
 
 int 
 kmread(
-	dev_t dev, 
+	__unused dev_t dev, 
 	struct uio *uio,
 	int ioflag)
 {
@@ -161,7 +177,7 @@ kmread(
 
 int 
 kmwrite(
-	dev_t dev, 
+	__unused dev_t dev, 
 	struct uio *uio,
 	int ioflag)
 {
@@ -173,7 +189,7 @@ kmwrite(
 
 int 
 kmioctl(
-	dev_t dev, 
+	__unused dev_t dev, 
 	int cmd, 
 	caddr_t data, 
 	int flag,
@@ -209,16 +225,9 @@ kmioctl(
 	    }
 	    default:		
 		error = (*linesw[tp->t_line].l_ioctl)(tp, cmd, data, flag, p);
-		if (error >= 0) {
+		if (ENOTTY != error)
 			return error;
-		}
-		error = ttioctl (tp, cmd, data, flag, p);
-		if (error >= 0) {
-			return error;
-		}
-		else {
-			return ENOTTY;
-		}
+		return ttioctl (tp, cmd, data, flag, p);
 	}
 }
 
@@ -234,16 +243,16 @@ kmputc(
 		return( 0);
 
 	if(c == '\n')
-		cnputc('\r');
+		cnputcusr('\r');
 
-	cnputc(c);
+	cnputcusr(c);
 
 	return 0;
 }
 
 int 
 kmgetc(
-	dev_t dev)
+	__unused dev_t dev)
 {
 	int c;
 	
@@ -252,13 +261,13 @@ kmgetc(
 	if (c == '\r') {
 		c = '\n';
 	}
-	cnputc(c);
+	cnputcusr(c);
 	return c;
 }
 
 int 
 kmgetc_silent(
-	dev_t dev)
+	__unused dev_t dev)
 {
 	int c;
 	
@@ -279,31 +288,17 @@ static void
 kmstart(
 	struct tty *tp)
 {
-	extern int hz;
 	if (tp->t_state & (TS_TIMEOUT | TS_BUSY | TS_TTSTOP))
 		goto out;
 	if (tp->t_outq.c_cc == 0)
 		goto out;
 	tp->t_state |= TS_BUSY;
-	if (tp->t_outq.c_cc > tp->t_lowat) {
-		/*
-		 * Start immediately.
-		 */
-		kmoutput(tp);
-	}
-	else {
-		/*
-		 * Wait a bit...
-		 */
-#if 0
-		/* FIXME */
-		timeout(kmtimeout, tp, hz);
-#else
-		kmoutput(tp);
-#endif
-	}
+	kmoutput(tp);
+	return;
+
 out:
-	ttwwakeup(tp);
+	(*linesw[tp->t_line].l_start)(tp);
+	return;
 }
 
 static void
@@ -328,7 +323,6 @@ kmoutput(
 	char 		buf[80];
 	char 		*cp;
 	int 		cc = -1;
-	extern int hz;
 
 
 	while (tp->t_outq.c_cc > 0) {
@@ -345,17 +339,16 @@ kmoutput(
 		timeout(kmtimeout, tp, hz);
 	}
 	tp->t_state &= ~TS_BUSY;
-	ttwwakeup(tp);
+	(*linesw[tp->t_line].l_start)(tp);
 
 	return 0;
 }
+
+void
 cons_cinput(char ch)
 {
 	struct tty *tp = &cons;
-	boolean_t 	funnel_state;
-
 	
 	(*linesw[tp->t_line].l_rint) (ch, tp);
-
 }
 
