@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -37,7 +35,7 @@
  *
  */
 
-
+#include <string.h>
 
 #if 0
 void *memcpy(void *dst, const void *src, unsigned int ulen)
@@ -46,7 +44,9 @@ void *memcpy(void *dst, const void *src, unsigned int ulen)
 	return dst;
 }
 #endif /* 0 */
-void *memmove(void *dst, const void *src, unsigned int ulen)
+
+void *
+memmove(void *dst, const void *src, size_t ulen)
 {
 	bcopy(src, dst, ulen);
 	return dst;
