@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -31,36 +32,31 @@
 #include <mach/exception.h>
 
 #include <kern/thread.h>
-#include <kern/thread_act.h>
 
+#include <sys/systm.h>
 #include <sys/param.h>
-#include <sys/proc.h>
+#include <sys/proc_internal.h>
 #include <sys/user.h>
+#include <sys/sysproto.h>
+#include <sys/sysent.h>
+#include <mach/thread_act.h>	/* for thread_abort_safely */
 
 #include <i386/psl.h>
+#include <i386/seg.h>
 
 #include <mach/i386/thread_status.h>
-#include <dev/i386/sel_inline.h>
 
+extern struct i386_saved_state *get_user_regs(thread_t);
 
-/*
- * FIXME.. should be included from mach_kernel/i386/seg.h
- */
+extern boolean_t valid_user_segment_selectors(uint16_t cs,
+					      uint16_t ss,
+					      uint16_t ds,
+					      uint16_t es,
+					      uint16_t fs,
+					      uint16_t gs);
 
-#define	USER_CS	0x17
-#define	USER_DS	0x1f
-#define USER_CTHREAD 0x27
-
-#define	UDATA_SEL	USER_DS
-#define	UCODE_SEL	USER_CS
-#define UCTHREAD_SEL    USER_CTHREAD
-
-#define	valid_user_code_selector(x)	(TRUE)
-#define	valid_user_data_selector(x)	(TRUE)
-#define	valid_user_stack_selector(x)	(TRUE)
-
-
-#define	NULL_SEG	0
+/* Forward: */
+extern boolean_t machine_exception(int, int, int, int *, int *);
 
 /* Signal handler flavors supported */
 /* These defns should match the Libc implmn */
@@ -76,11 +72,10 @@
  * pointer, and the argument pointer, it returns
  * to the user specified pc, psl.
  */
-
 void
 sendsig(p, catcher, sig, mask, code)
  	struct proc *p;
-	sig_t catcher;
+	user_addr_t catcher;	/* sig_t */
 	int sig, mask;
 	u_long code;
 {
@@ -96,13 +91,11 @@ sendsig(p, catcher, sig, mask, code)
 	struct sigacts *ps = p->p_sigacts;
 	int oonstack;
 	thread_t thread = current_thread();
-	thread_act_t th_act = current_act();
 	struct uthread * ut;
-	struct i386_saved_state * saved_state = (struct i386_saved_state *)
-							get_user_regs(th_act);
+	struct i386_saved_state * saved_state = get_user_regs(thread);
 	sig_t trampact;
     
-	ut = get_bsdthread_info(th_act);
+	ut = get_bsdthread_info(thread);
 	oonstack = ps->ps_sigstk.ss_flags & SA_ONSTACK;
 	if ((ps->ps_flags & SAS_ALTSTACK) && !oonstack &&
 		(ps->ps_sigonstack & sigmask(sig))) {
@@ -116,10 +109,10 @@ sendsig(p, catcher, sig, mask, code)
 	 * Build the argument list for the signal handler.
 	 */
 
-	trampact = ps->ps_trampact[sig];
+	trampact = (sig_t)ps->ps_trampact[sig];
 	/* Handler should call sigreturn to get out of it */
 	frame.retaddr = 0xffffffff;	
-	frame.catcher = catcher;
+	frame.catcher = CAST_DOWN(sig_t,catcher);	/* XXX LP64 */
 	frame.sigstyle = UC_TRAD;
 	frame.sig = sig;
 
@@ -128,19 +121,9 @@ sendsig(p, catcher, sig, mask, code)
 	} else
 		frame.code = 0;
 	frame.scp = scp;
-	if (copyout((caddr_t)&frame, (caddr_t)fp, sizeof (frame))) 
+	if (copyout((caddr_t)&frame, (user_addr_t)fp, sizeof (frame))) 
 		goto bad;
 
-#if	PC_SUPPORT
-	{
-	PCcontext_t	context = threadPCContext(thread);
-	
-	if (context && context->running) {
-		oonstack |= 02;
-		context->running = FALSE;
-	}
-	}
-#endif
 	/*
 	 * Build the signal context to be used by sigreturn.
 	 */
@@ -171,17 +154,17 @@ sendsig(p, catcher, sig, mask, code)
 		context.sc_fs = saved_state->fs;
 		context.sc_gs = saved_state->gs;
 	}
-	if (copyout((caddr_t)&context, (caddr_t)scp, sizeof (context))) 
+	if (copyout((caddr_t)&context, (user_addr_t)scp, sizeof (context))) 
 		goto bad;
 
 	saved_state->eip = (unsigned int)trampact;
-	saved_state->cs = UCODE_SEL;
+	saved_state->cs = USER_CS;
 
 	saved_state->uesp = (unsigned int)fp;
-	saved_state->ss = UDATA_SEL;
+	saved_state->ss = USER_DS;
 
-	saved_state->ds = UDATA_SEL;
-	saved_state->es = UDATA_SEL;
+	saved_state->ds = USER_DS;
+	saved_state->es = USER_DS;
 	saved_state->fs = NULL_SEG;
 	saved_state->gs = USER_CTHREAD;
 	return;
@@ -207,83 +190,88 @@ bad:
  * psl to gain improper priviledges or to cause
  * a machine fault.
  */
-struct sigreturn_args {
-	struct sigcontext *sigcntxp;
-};
 /* ARGSUSED */
 int
-sigreturn(p, uap, retval)
-	struct proc *p;
-	struct sigreturn_args *uap;
-	int *retval;
+sigreturn(
+	struct proc *p,
+	struct sigreturn_args *uap,
+	__unused int *retval)
 {
     struct sigcontext		context;
     thread_t			thread = current_thread();
-    thread_act_t		th_act = current_act();
 	int error;
     struct i386_saved_state*	saved_state = (struct i386_saved_state*)
-							get_user_regs(th_act);	
+							get_user_regs(thread);	
 	struct uthread * ut;
 
 
     
-    if (saved_state == NULL) 
-	return EINVAL;
+	if (saved_state == NULL) 
+		return EINVAL;
 
-    if (error = copyin((caddr_t)uap->sigcntxp, (caddr_t)&context, 
-						sizeof (context))) 
-					return(error);
-	ut = (struct uthread *)get_bsdthread_info(th_act);
+    if ((error = copyin(CAST_USER_ADDR_T(uap->sigcntxp), (void *)&context, 
+				sizeof (context)))) 
+		return(error);
 
-    if (context.sc_onstack & 01)
-			p->p_sigacts->ps_sigstk.ss_flags |= SA_ONSTACK;
+	/*
+	 * Validate segment selectors.
+	 * Bad values would result in kernel exception at context switch
+	 * back to user mode. If other state is invalid an exception will
+	 * occur in user context.
+	 */
+	if (!valid_user_segment_selectors(context.sc_cs,
+					  context.sc_ss,
+					  context.sc_ds,
+					  context.sc_es,
+					  context.sc_fs,
+					  context.sc_gs)) {
+		return EINVAL;
+	}
+
+	ut = (struct uthread *)get_bsdthread_info(thread);
+
+	if (context.sc_onstack & 01)
+		p->p_sigacts->ps_sigstk.ss_flags |= SA_ONSTACK;
 	else
 		p->p_sigacts->ps_sigstk.ss_flags &= ~SA_ONSTACK;
+
 	ut->uu_sigmask = context.sc_mask &~ sigcantmask;
 	if(ut->uu_siglist & ~ut->uu_sigmask)
-		signal_setast(current_act());
-    saved_state->eax = context.sc_eax;
-    saved_state->ebx = context.sc_ebx;
-    saved_state->ecx = context.sc_ecx;
-    saved_state->edx = context.sc_edx;
-    saved_state->edi = context.sc_edi;
-    saved_state->esi = context.sc_esi;
-    saved_state->ebp = context.sc_ebp;
-    saved_state->uesp = context.sc_esp;
-    saved_state->ss = context.sc_ss;
-    saved_state->efl = context.sc_eflags;
-    saved_state->efl &= ~EFL_USERCLR;
-    saved_state->efl |= EFL_USERSET;
-    saved_state->eip = context.sc_eip;
-    saved_state->cs = context.sc_cs;
+		signal_setast(thread);
 
-    if (context.sc_eflags & EFL_VM) {
-    	saved_state->ds = NULL_SEG;
-    	saved_state->es = NULL_SEG;
-    	saved_state->fs = NULL_SEG;
-    	saved_state->gs = NULL_SEG;
-	saved_state->v86_segs.v86_ds = context.sc_ds;
-	saved_state->v86_segs.v86_es = context.sc_es;
-	saved_state->v86_segs.v86_fs = context.sc_fs;
-	saved_state->v86_segs.v86_gs = context.sc_gs;
+	saved_state->eax = context.sc_eax;
+	saved_state->ebx = context.sc_ebx;
+	saved_state->ecx = context.sc_ecx;
+	saved_state->edx = context.sc_edx;
+	saved_state->edi = context.sc_edi;
+	saved_state->esi = context.sc_esi;
+	saved_state->ebp = context.sc_ebp;
+	saved_state->uesp = context.sc_esp;
+	saved_state->ss = context.sc_ss;
+	saved_state->efl = context.sc_eflags;
+	saved_state->efl &= ~EFL_USERCLR;
+	saved_state->efl |= EFL_USERSET;
+	saved_state->eip = context.sc_eip;
+	saved_state->cs = context.sc_cs;
 
-	saved_state->efl |= EFL_VM;
-    }
-    else {
-	saved_state->ds = context.sc_ds;
-	saved_state->es = context.sc_es;
-	saved_state->fs = context.sc_fs;
-	saved_state->gs = context.sc_gs;
-    }
+	if (context.sc_eflags & EFL_VM) {
+		saved_state->ds = NULL_SEG;
+    		saved_state->es = NULL_SEG;
+    		saved_state->fs = NULL_SEG;
+    		saved_state->gs = NULL_SEG;
+		saved_state->v86_segs.v86_ds = context.sc_ds;
+		saved_state->v86_segs.v86_es = context.sc_es;
+		saved_state->v86_segs.v86_fs = context.sc_fs;
+		saved_state->v86_segs.v86_gs = context.sc_gs;
 
-#if	PC_SUPPORT
-    if (context.sc_onstack & 02) {
-	PCcontext_t	context = threadPCContext(thread);
-	
-	if (context)
-	    context->running = TRUE;
-    }
-#endif
+		saved_state->efl |= EFL_VM;
+	}
+	else {
+		saved_state->ds = context.sc_ds;
+		saved_state->es = context.sc_es;
+		saved_state->fs = context.sc_fs;
+		saved_state->gs = context.sc_gs;
+	}
 
 	return (EJUSTRETURN);
 }
@@ -295,11 +283,11 @@ sigreturn(p, uap, retval)
 
 boolean_t
 machine_exception(
-    int		exception,
-    int		code,
-    int		subcode,
-    int		*unix_signal,
-    int		*unix_code
+    int			exception,
+    int			code,
+    __unused int	subcode,
+    int			*unix_signal,
+    int			*unix_code
 )
 {
 
@@ -321,3 +309,52 @@ machine_exception(
    
     return(TRUE);
 }
+
+#include <sys/systm.h>
+#include <sys/sysent.h>
+
+int __pthread_cset(struct sysent *);
+void __pthread_creset(struct sysent *);
+
+int
+__pthread_cset(struct sysent *callp)
+{
+	unsigned int cancel_enable;
+	thread_t thread;
+	struct uthread * uthread; 
+
+	thread = current_thread();
+	uthread = get_bsdthread_info(thread);
+
+	cancel_enable = callp->sy_cancel;
+	if (cancel_enable == _SYSCALL_CANCEL_NONE) {
+		uthread->uu_flag |= UT_NOTCANCELPT;
+	} else {
+		if((uthread->uu_flag & (UT_CANCELDISABLE | UT_CANCEL | UT_CANCELED)) == UT_CANCEL) {
+			if (cancel_enable == _SYSCALL_CANCEL_PRE)
+				return(EINTR);
+			else
+				thread_abort_safely(thread);
+		}
+	}   
+	return(0);
+}
+
+
+void
+__pthread_creset(struct sysent *callp)
+{
+
+	unsigned int cancel_enable;
+	thread_t thread;
+	struct uthread * uthread; 
+
+	thread = current_thread();
+	uthread = get_bsdthread_info(thread);
+	
+	cancel_enable = callp->sy_cancel;
+	if (!cancel_enable) 
+		uthread->uu_flag &= ~UT_NOTCANCELPT;
+
+}
+
