@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -56,12 +54,19 @@ consopen(dev, flag, devtype, pp)
 	struct proc *pp;
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
+	
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_open)(device, flag, devtype, pp));
+	error =  (*cdevsw[major(device)].d_open)(device, flag, devtype, pp);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
@@ -72,12 +77,20 @@ consclose(dev, flag, mode, pp)
 	struct proc *pp;
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_close)(device, flag, mode, pp));
+	error =  (*cdevsw[major(device)].d_close)(device, flag, mode, pp);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
+
+
 }
 
 /*ARGSUSED*/
@@ -88,12 +101,18 @@ consread(dev, uio, ioflag)
 	int ioflag;
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_read)(device, uio, ioflag));
+	error = (*cdevsw[major(device)].d_read)(device, uio, ioflag);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
@@ -104,12 +123,18 @@ conswrite(dev, uio, ioflag)
 	int ioflag;
 {
     dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-    return ((*cdevsw[major(device)].d_write)(device, uio, ioflag));
+    error =  (*cdevsw[major(device)].d_write)(device, uio, ioflag);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
@@ -122,6 +147,10 @@ consioctl(dev, cmd, addr, flag, p)
 	struct proc *p;
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
+
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 
 	if (constty)
 	    device = constty->t_dev;
@@ -132,16 +161,23 @@ consioctl(dev, cmd, addr, flag, p)
 	 * output from the "virtual" console.
 	 */
 	if (cmd == TIOCCONS && constty) {
-		int error = suser(p->p_ucred, (u_short *) NULL);
-		if (error)
-			return (error);
+		error = proc_suser(p);
+		if (error) {
+			goto out;
+		}
 		constty = NULL;
-		return (0);
+		error = 0;
+		goto out;
 	}
-	return ((*cdevsw[major(device)].d_ioctl)(device, cmd, addr, flag, p));
+	error =  (*cdevsw[major(device)].d_ioctl)(device, cmd, addr, flag, p);
+out:
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
+/* called with funnel held */
 int
 consselect(dev, flag, wql, p)
 	dev_t dev;
@@ -162,12 +198,18 @@ int
 cons_getc()
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_getc)(device));
+	error =  (*cdevsw[major(device)].d_getc)(device);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*ARGSUSED*/
@@ -176,12 +218,18 @@ cons_putc(c)
 	char c;
 {
 	dev_t device;
+	boolean_t funnel_state;
+	int error;
 
+	funnel_state = thread_funnel_set(kernel_flock, TRUE);
 	if (constty)
 	    device = constty->t_dev;
 	else
 	    device = cons.t_dev;
-	return ((*cdevsw[major(device)].d_putc)(device, c));
+	error =  (*cdevsw[major(device)].d_putc)(device, c);
+	thread_funnel_set(kernel_flock, funnel_state);
+
+	return(error);
 }
 
 /*
