@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -71,7 +69,7 @@ nvread(dev, uio, ioflag)
     int error = 0;
 
     offset = uio->uio_offset;
-    size = uio->uio_resid;
+    size = uio_resid(uio);
 
     for (read = 0; read < size; read++, offset++)  {
         error = PEnvread(offset, 1, &cc);
@@ -91,7 +89,6 @@ nvread(dev, uio, ioflag)
 
 nvwrite(dev_t dev,  struct uio *uio, int ioflag)
 {
-        register struct iovec *iov;
         long offset;
         long size;
         int c;
@@ -100,7 +97,7 @@ nvwrite(dev_t dev,  struct uio *uio, int ioflag)
         int error = 0;
 
         offset = uio->uio_offset;
-        size = uio->uio_resid;
+        size = uio_resid(uio);
 
         for (wrote = 0; wrote < size; wrote++, offset++) {
             c = uwritec(uio);
