@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -32,24 +33,26 @@
 #ifdef	YARROW_KERNEL
 
 /* null compression */
-comp_error_status comp_init(COMP_CTX* ctx)
+comp_error_status comp_init(__unused COMP_CTX* ctx)
 {
 	return COMP_SUCCESS;
 }
 
 
-comp_error_status comp_add_data(COMP_CTX* ctx,Bytef* inp,uInt inplen)
+comp_error_status comp_add_data( __unused COMP_CTX* ctx, 
+								 __unused Bytef* inp, 
+								 __unused uInt inplen )
 {
 	return COMP_SUCCESS;
 }
 
-comp_error_status comp_get_ratio(COMP_CTX* ctx,float* out)
+comp_error_status comp_get_ratio( __unused COMP_CTX* ctx,float* out )
 {
 	*out = 1.0;
 	return COMP_SUCCESS;
 }
 
-comp_error_status comp_end(COMP_CTX* ctx)
+comp_error_status comp_end( __unused COMP_CTX* ctx )
 {
 	return COMP_SUCCESS;
 }
