@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -82,7 +80,7 @@ extern "C" {
 #define MM_NULL	0
 
 /* Function forward declarations */
-SMFAPI void mmInit();
+SMFAPI void mmInit( void );
 SMFAPI MMPTR mmMalloc(DWORD request);
 SMFAPI void mmFree(MMPTR ptrnum);
 SMFAPI LPVOID mmGetPtr(MMPTR ptrnum);
