@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -85,9 +86,11 @@ hfs_vnop_inactive(struct vnop_inactive_args *ap)
 	v_type = vnode_vtype(vp);
 	cp = VTOC(vp);
 
-	if ((hfsmp->hfs_flags & HFS_READ_ONLY) || vnode_issystem(vp)) {
+	if ((hfsmp->hfs_flags & HFS_READ_ONLY) || vnode_issystem(vp) ||
+	    (hfsmp->hfs_freezing_proc == p)) {
 		return (0);
 	}
+
 	/*
 	 * Ignore nodes related to stale file handles.
 	 */
@@ -142,11 +145,11 @@ hfs_vnop_inactive(struct vnop_inactive_args *ap)
 		    // them in the catalog entry and then double
 		    // free them later.
 		    //
-		    if (hfs_start_transaction(hfsmp) != 0) {
-			error = EINVAL;
-			goto out;
-		    }
-		    started_tr = 1;
+//		    if (hfs_start_transaction(hfsmp) != 0) {
+//			error = EINVAL;
+//			goto out;
+//		    }
+//		    started_tr = 1;
 		    
 			/*
 			 * Since we're already inside a transaction,
