@@ -3,25 +3,27 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
 #include <sys/param.h>
 #include <hfs/hfs_macos_defs.h>
+#include <hfs/hfs.h>
 
 
 /* CJK Mac Encoding Bits */
@@ -42,6 +44,9 @@ u_int8_t cjk_lastunique = 0;
 u_int32_t hfs_encodingbias = 0;
 int hfs_islatinbias = 0;
 
+extern lck_mtx_t  encodinglst_mutex;
+
+
 /* Map CJK bits to Mac encoding */
 u_int8_t cjk_encoding[] = {
 	/* 0000 */  kTextEncodingMacUnicode,
@@ -889,7 +894,7 @@ hfs_pickencoding(const u_int16_t *src, int len)
 
 __private_extern__
 u_int32_t
-hfs_getencodingbias()
+hfs_getencodingbias(void)
 {
 	return (hfs_encodingbias);
 }
@@ -899,6 +904,8 @@ __private_extern__
 void
 hfs_setencodingbias(u_int32_t bias)
 {
+	lck_mtx_lock(&encodinglst_mutex);
+
 	hfs_encodingbias = bias;
 
 	switch (bias) {
@@ -914,5 +921,7 @@ hfs_setencodingbias(u_int32_t bias)
 		hfs_islatinbias = 0;
 		break;					
 	}
+
+	lck_mtx_unlock(&encodinglst_mutex);
 }
 
