@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -315,15 +316,12 @@ OSStatus BTOpenPath(FCB *filePtr, KeyCompareProcPtr keyCompareProc)
 		}
 	}
 
-	// if nodeSize Matches then we don't need to release, just CheckNode
-	if ( btreePtr->nodeSize == nodeRec.blockSize )
-	{
-		err = CheckNode (btreePtr, nodeRec.buffer);
-		if (err)
-			VTOVCB(btreePtr->fileRefNum)->vcbFlags |= kHFS_DamagedVolume;
-		M_ExitOnError (err);
-	}
-	else
+	/*
+	 * If the actual node size is different than the amount we read,
+	 * then release and trash this block, and re-read with the correct
+	 * node size.
+	 */
+	if ( btreePtr->nodeSize != nodeRec.blockSize )
 	{
 		err = SetBTreeBlockSize (btreePtr->fileRefNum, btreePtr->nodeSize, 32);
 		M_ExitOnError (err);
@@ -336,7 +334,7 @@ OSStatus BTOpenPath(FCB *filePtr, KeyCompareProcPtr keyCompareProc)
 		++btreePtr->numReleaseNodes;
 		M_ExitOnError (err);
 
-		err = GetNode (btreePtr, kHeaderNodeNum, &nodeRec );		// calls CheckNode...
+		err = GetNode (btreePtr, kHeaderNodeNum, &nodeRec );
 		M_ExitOnError (err);
 	}
 
@@ -1286,15 +1284,19 @@ OSStatus	BTInsertRecord		(FCB						*filePtr,
 									goto ErrorExit;
 								}
 
-								err = UpdateNode (btreePtr, &nodeRec, 0, kLockTransaction);
-								M_ExitOnError (err);
-
-								// update BTreeControlBlock
+								/*
+								 * Update the B-tree control block.  Do this before
+								 * calling UpdateNode since it will compare the node's
+								 * height with treeDepth.
+								 */
 								btreePtr->treeDepth	 		= 1;
 								btreePtr->rootNode	 		= insertNodeNum;
 								btreePtr->firstLeafNode		= insertNodeNum;
 								btreePtr->lastLeafNode		= insertNodeNum;
 
+								err = UpdateNode (btreePtr, &nodeRec, 0, kLockTransaction);
+								M_ExitOnError (err);
+
 								M_BTreeHeaderDirty (btreePtr);
 
 								goto Success;
