@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  *
@@ -140,7 +141,9 @@ int BTScanNextRecord(	BTScanState *	scanState,
 
 static int FindNextLeafNode(	BTScanState *scanState, Boolean avoidIO )
 {
-	int		err;
+	int err;
+	BlockDescriptor block;
+	FileReference fref;
 	
 	err = noErr;		// Assume everything will be OK
 	
@@ -180,29 +183,23 @@ static int FindNextLeafNode(	BTScanState *scanState, Boolean avoidIO )
 			(u_int8_t *) scanState->currentNodePtr += scanState->btcb->nodeSize;
 		}
 		
-#if BYTE_ORDER == LITTLE_ENDIAN
-		{
-		BlockDescriptor block;
-		FileReference fref;
-
 		/* Fake a BlockDescriptor */
+		block.blockHeader = NULL;	/* No buffer cache buffer */
 		block.buffer = scanState->currentNodePtr;
+		block.blockNum = scanState->nodeNum;
 		block.blockSize = scanState->btcb->nodeSize;
 		block.blockReadFromDisk = 1;
 		block.isModified = 0;
 		
 		fref = scanState->btcb->fileRefNum;
 		
-		SWAP_BT_NODE(&block, ISHFSPLUS(VTOVCB(fref)), VTOC(fref)->c_fileid, 0);
-		}
-#endif
-
-		// Make sure this is a valid node
-		if ( CheckNode( scanState->btcb, scanState->currentNodePtr ) != noErr )
-		{
+		/* This node was read from disk, so it must be swapped/checked. */
+		err = hfs_swap_BTNode(&block, fref, kSwapBTNodeBigToHost);
+		if ( err != noErr ) {
+			printf("FindNextLeafNode: Error from hfs_swap_BTNode (node %u)\n", scanState->nodeNum);
 			continue;
 		}
-		
+
 		if ( scanState->currentNodePtr->kind == kBTLeafNode )
 			break;
 	}
