@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -27,10 +28,14 @@
 #ifndef	_i386_SIGNAL_
 #define	_i386_SIGNAL_ 1
 
+#ifndef _ANSI_SOURCE
+typedef int sig_atomic_t; 
+
+#ifndef _POSIX_C_SOURCE
+
 #include <sys/appleapiopts.h>
 
 #ifdef __APPLE_API_OBSOLETE
-typedef int sig_atomic_t; 
 
 /*
  * Information pushed on stack when a signal is delivered.
@@ -61,6 +66,8 @@ struct	sigcontext {
 };
 
 #endif /* __APPLE_API_OBSOLETE */
+#endif /* ! _POSIX_C_SOURCE */
+#endif /* ! _ANSI_SOURCE */
 
 #endif	/* _i386_SIGNAL_ */
 
