@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -221,6 +222,71 @@ proc_self()
 	return(current_proc());
 }
 
+proc_t
+proc_findref(int pid)
+{
+	boolean_t funnel_state;
+	proc_t p;
+
+	funnel_state = thread_funnel_set(kernel_flock,TRUE);
+	p = pfind(pid);
+	
+	if (p != proc_refinternal(p, 1))
+		p = PROC_NULL;
+		
+	thread_funnel_set(kernel_flock, funnel_state);
+	return(p);
+}
+
+void
+proc_dropref(proc_t p)
+{
+
+	proc_dropinternal(p, 0);
+}
+
+
+proc_t
+proc_refinternal(proc_t p, int funneled)
+{
+
+	proc_t p1 = p;
+	boolean_t funnel_state = TRUE; 	/* need to init just to avoid warnings and build failure */
+
+	if (funneled == 0)
+		funnel_state = thread_funnel_set(kernel_flock,TRUE);
+	
+	if ((p != PROC_NULL) &&(p->p_stat != SZOMB) && ((p->p_lflag & (P_LREFDRAINWAIT | P_LREFDRAIN | P_LREFDEAD)) == 0))
+		p->p_internalref++;
+	else 
+		p1 = PROC_NULL;
+
+	if (funneled == 0)
+		thread_funnel_set(kernel_flock,funnel_state);
+	return(p1);
+}
+
+void
+proc_dropinternal(proc_t p, int funneled)
+{
+	boolean_t funnel_state = TRUE; /* need to init just to avoid warnings and build failure */
+
+	if (funneled == 0)
+		funnel_state = thread_funnel_set(kernel_flock,TRUE);
+
+	if (p->p_internalref > 0) {
+		p->p_internalref--;
+		if ((p->p_internalref == 0) && ((p->p_lflag & P_LREFDRAINWAIT) == P_LREFDRAINWAIT)) {
+			p->p_lflag &= ~P_LREFDRAINWAIT;
+			wakeup(&p->p_internalref);
+		}
+	} else
+		printf("proc_dropreg -ve ref\n");
+
+	if (funneled == 0)
+		thread_funnel_set(kernel_flock,funnel_state);
+}
+
 
 int
 proc_pid(proc_t p)
