@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -60,7 +61,8 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/kernel.h>
-#include <sys/proc.h>
+#include <sys/proc_internal.h>
+#include <sys/kauth.h>
 #include <sys/reboot.h>
 #include <sys/vm.h>
 #include <sys/sysctl.h>
@@ -68,142 +70,25 @@
 
 #include <bsm/audit_kernel.h>
 
-#include <sys/mount.h>
+#include <sys/mount_internal.h>
+#include <sys/sysproto.h>
 
-#if COMPAT_43
-/* ARGSUSED */
 int
-ogethostid(p, uap, retval)
-struct proc *p;
-void *uap;
-register_t *retval;
-{
-
-	*retval = hostid;
-	return 0;
-}
-
-struct osethostid_args {
-	long hostid;
-};
-/* ARGSUSED */
-int
-osethostid(p, uap, retval)
-struct proc *p;
-register struct osethostid_args *uap;
-register_t *retval;
-{
-	int error;
-
-	if (error = suser(p->p_ucred, &p->p_acflag))
-		return (error);
-	hostid = uap->hostid;
-	return (0);
-
-}
-
-struct ogethostname_args {
-		char	*hostname;
-		u_int	len;
-};
-/* ARGSUSED */
-int
-ogethostname(p, uap, retval)
-struct proc *p;
-register struct ogethostname_args *uap;
-register_t *retval;
-{
-	int name;
-
-	name = KERN_HOSTNAME;
-
-	return (kern_sysctl(&name, 1, uap->hostname, &uap->len, 0, 0));
-}
-
-struct osethostname_args {
-		char	*hostname;
-		u_int	len;
-};
-/* ARGSUSED */
-int
-osethostname(p, uap, retval)
-struct proc *p;
-register struct osethostname_args *uap;
-register_t *retval;
-{
-	int name;
-	int error;
-
-	if (error = suser(p->p_ucred, &p->p_acflag))
-		return (error);
-		
-	name = KERN_HOSTNAME;
-	return (kern_sysctl(&name, 1, 0, 0, uap->hostname,
-	    uap->len));
-}
-
-struct ogetdomainname_args {
-		char	*domainname;
-		int	len;
-};
-/* ARGSUSED */
-int
-ogetdomainname(p, uap, retval)
-struct proc *p;
-register struct ogetdomainname_args *uap;
-register_t *retval;
-{
-	int name;
-	
-	name = KERN_DOMAINNAME;
-	return (kern_sysctl(&name, 1, uap->domainname,
-	    &uap->len, 0, 0));
-}
-
-struct osetdomainname_args {
-		char	*domainname;
-		u_int	len;
-};
-/* ARGSUSED */
-int
-osetdomainname(p, uap, retval)
-struct proc *p;
-register struct osetdomainname_args *uap;
-register_t *retval;
-{
-	int name;
-	int error;
-
-	if (error = suser(p->p_ucred, &p->p_acflag))
-		return (error);
-	name = KERN_DOMAINNAME;
-	return (kern_sysctl(&name, 1, 0, 0, uap->domainname,
-	    uap->len));
-}
-#endif /* COMPAT_43 */
-
-struct reboot_args {
-		int	opt;
-		char	*command;
-};
-
-reboot(p, uap, retval)
-struct proc *p;
-register struct reboot_args *uap;
-register_t *retval;
+reboot(struct proc *p, register struct reboot_args *uap, __unused register_t *retval)
 {
 	char command[64];
 	int error;
 	int dummy=0;
 
 	AUDIT_ARG(cmd, uap->opt);
+
 	command[0] = '\0';
 
-	if (error = suser(p->p_cred->pc_ucred, &p->p_acflag))
+	if ((error = suser(kauth_cred_get(), &p->p_acflag)))
 		return(error);	
 	
 	if (uap->opt & RB_COMMAND)
-		error = copyinstr((void *)uap->command,
+		error = copyinstr(uap->command,
 					(void *)command, sizeof(command), (size_t *)&dummy);
 	if (!error) {
 		SET(p->p_flag, P_REBOOT);	/* No more signals for this proc */
