@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -39,11 +40,15 @@
 
 typedef int load_return_t;
 
+/*
+ * Structure describing the result from calling load_machfile(), if that
+ * function returns LOAD_SUCCESS.
+ */
 typedef struct _load_result {
-	vm_offset_t		mach_header;
-	vm_offset_t		entry_point;
-	vm_offset_t		user_stack;
-	int			thread_count;
+	user_addr_t	mach_header;
+	user_addr_t	entry_point;
+	user_addr_t	user_stack;
+	int		thread_count;
 	unsigned int
 	/* boolean_t */	unixproc	:1,
 			dynlinker	:1,
@@ -51,15 +56,14 @@ typedef struct _load_result {
 					:0;
 } load_result_t;
 
+struct image_params;
 load_return_t load_machfile(
-	struct vnode		*vp,
+	struct image_params	*imgp,
 	struct mach_header	*header,
-	unsigned long		file_offset,
-	unsigned long		macho_size,
-	load_result_t		*result,
-	thread_act_t		thr_act,
+	thread_t		thr_act,
 	vm_map_t		map,
-	boolean_t		clean_regions);
+	boolean_t		clean_regions,
+	load_result_t		*result);
 
 #define LOAD_SUCCESS		0
 #define LOAD_BADARCH		1	/* CPU type/subtype not found */
