@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -61,8 +59,7 @@
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/conf.h>
-#include <sys/buf.h>
-#include <sys/proc.h>
+#include <sys/proc_internal.h>
 #include <sys/vnode.h>
 #include <sys/uio.h>
 
@@ -117,11 +114,13 @@ enosys()
 /*
  * Return error for operation not supported
  * on a specific object or file type.
+ *
+ * XXX Name of this routine is wrong.
  */
 int
 eopnotsupp()
 {
-	return (EOPNOTSUPP);
+	return (ENOTSUP);
 }
 
 /*
