@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -1434,7 +1435,8 @@ poll_callback(__unused struct kqueue *kq, struct kevent *kevp, void *data)
 {
 	struct poll_continue_args *cont = (struct poll_continue_args *)data;
 	struct pollfd *fds = CAST_DOWN(struct pollfd *, kevp->udata);
-	
+	short mask;
+
 	/* convert the results back into revents */
 	if (kevp->flags & EV_EOF)
 		fds->revents |= POLLHUP;
@@ -1444,10 +1446,16 @@ poll_callback(__unused struct kqueue *kq, struct kevent *kevp, void *data)
 
 	switch (kevp->filter) {
 	case EVFILT_READ:
-		if (kevp->data != 0)
-			fds->revents |= (fds->events & ( POLLIN | POLLRDNORM ));
-		if (kevp->flags & EV_OOBAND)
-			fds->revents |= (fds->events & ( POLLPRI | POLLRDBAND ));
+		if (fds->revents & POLLHUP)
+			mask = (POLLIN | POLLRDNORM | POLLPRI | POLLRDBAND );
+		else {
+			mask = 0;
+			if (kevp->data != 0)
+				mask |= (POLLIN | POLLRDNORM );
+			if (kevp->flags & EV_OOBAND)
+				mask |= ( POLLPRI | POLLRDBAND );
+		}
+		fds->revents |= (fds->events & mask);
 		break;
 
 	case EVFILT_WRITE:
