@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -78,22 +76,29 @@ extern	struct domain localdomain;		/* or at least forward */
 
 static struct protosw localsw[] = {
 { SOCK_STREAM,	&localdomain,	0,	PR_CONNREQUIRED|PR_WANTRCVD|PR_RIGHTS,
-  0,		0,		0,		0,
+  0,		0,		0,		uipc_ctloutput,
   0,
   0,		0,		0,		0,
-  0,		&uipc_usrreqs
+  0,	
+  &uipc_usrreqs,
+  0,		0,		0
+
 },
 { SOCK_DGRAM,	&localdomain,	0,		PR_ATOMIC|PR_ADDR|PR_RIGHTS,
-  0,		0,		0,		0,
+  0,		0,		0,		uipc_ctloutput,
   0,
   0,		0,		0,		0,
-  0,		&uipc_usrreqs
+  0,	
+  &uipc_usrreqs,
+  0,		0,		0
 },
 { 0,		0,		0,		0,
   0,		0,		raw_ctlinput,	0,
   0,
-  raw_init,	0,		0,		0,
-  0, &raw_usrreqs
+  0,		0,		0,		0,
+  0,
+  &raw_usrreqs,
+  0,			0,		0
 }
 };
 
