@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -142,18 +143,22 @@ soisconnected(so)
 	sflt_notify(so, sock_evt_connected, NULL);
 	
 	if (head && (so->so_state & SS_INCOMP)) {
-		if (head->so_proto->pr_getlock != NULL) 
+		so->so_state &= ~SS_INCOMP;
+		so->so_state |= SS_COMP;
+		if (head->so_proto->pr_getlock != NULL) {
+			socket_unlock(so, 0);
 			socket_lock(head, 1);
+		}
 		postevent(head, 0, EV_RCONN);
 		TAILQ_REMOVE(&head->so_incomp, so, so_list);
 		head->so_incqlen--;
-		so->so_state &= ~SS_INCOMP;
 		TAILQ_INSERT_TAIL(&head->so_comp, so, so_list);
-		so->so_state |= SS_COMP;
 		sorwakeup(head);
 		wakeup_one((caddr_t)&head->so_timeo);
-		if (head->so_proto->pr_getlock != NULL) 
+		if (head->so_proto->pr_getlock != NULL) {
 			socket_unlock(head, 1);
+			socket_lock(so, 0);
+		}
 	} else {
 		postevent(so, 0, EV_WCONN);
 		wakeup((caddr_t)&so->so_timeo);
