@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -29,7 +27,7 @@
 #define _BSD_MACHINE_PTRACE_H_
 
 
-#if defined (__ppc__)
+#if defined (__ppc__) || defined(__ppc64__)
 #include "ppc/ptrace.h"
 #elif defined (__i386__)
 #include "i386/ptrace.h"
