@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -25,14 +23,35 @@
 #ifndef _BSD_MACHINE_SPL_H_
 #define _BSD_MACHINE_SPL_H_
 
+#ifdef KERNEL
+#ifndef __ASSEMBLER__
+/*
+ *	Machine-dependent SPL definitions.
+ *
+ */
+typedef unsigned	spl_t;
 
-#if defined (__ppc__)
-#include "ppc/spl.h"
-#elif defined (__i386__)
-#include "i386/spl.h"
-#else
-#error architecture not supported
-#endif
+extern unsigned	int sploff(void);
+extern unsigned	int splhigh(void);
+extern unsigned	int splsched(void);
+extern unsigned	int splclock(void);
+extern unsigned	int splpower(void);
+extern unsigned	int splvm(void);
+extern unsigned	int splbio(void);
+extern unsigned	int splimp(void);
+extern unsigned	int spltty(void);
+extern unsigned	int splnet(void);
+extern unsigned	int splsoftclock(void);
+
+extern void	spllo(void);
+extern void	splon(unsigned int level);
+extern void	splx(unsigned int level);
+extern void	spln(unsigned int level);
+#define splstatclock()	splhigh()
+
+#endif /* __ASSEMBLER__ */
+
+#endif /* KERNEL */
 
 
 #endif /* _BSD_MACHINE_SPL_H_ */
