@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -88,18 +89,33 @@ typedef enum {
 struct fdescnode {
 	LIST_ENTRY(fdescnode) fd_hash;	/* Hash list */
 	struct vnode	*fd_vnode;	/* Back ptr to vnode */
-	fdntype		fd_type;	/* Type of this node */
-	unsigned	fd_fd;		/* Fd to be dup'ed */
-	char		*fd_link;	/* Link to fd/n */
-	int		fd_ix;		/* filesystem index */
+	fdntype			fd_type;	/* Type of this node */
+	unsigned		fd_fd;		/* Fd to be dup'ed */
+	char			*fd_link;	/* Link to fd/n */
+	int				fd_ix;		/* filesystem index */
 };
 
 #define VFSTOFDESC(mp)	((struct fdescmount *)((mp)->mnt_data))
 #define	VTOFDESC(vp) ((struct fdescnode *)(vp)->v_data)
 
-extern int fdesc_init __P((struct vfsconf *));
-extern int fdesc_root __P((struct mount *, struct vnode **));
-extern int fdesc_allocvp __P((fdntype, int, struct mount *, struct vnode **));
+extern int fdesc_allocvp(fdntype, int, struct mount *, struct vnode **, enum vtype);
+extern int fdesc_badop(void);
+extern int fdesc_getattr(struct vnop_getattr_args *ap);
+extern int fdesc_inactive(struct vnop_inactive_args *ap);
+extern int fdesc_init(struct vfsconf *);
+extern int fdesc_ioctl(struct vnop_ioctl_args *ap);
+extern int fdesc_lookup(struct vnop_lookup_args *ap);
+extern int fdesc_open(struct vnop_open_args *ap);
+extern int fdesc_pathconf(struct vnop_pathconf_args *ap);
+extern int fdesc_read(struct vnop_read_args *ap);
+extern int fdesc_readdir(struct vnop_readdir_args *ap);
+extern int fdesc_readlink(struct vnop_readlink_args *ap);
+extern int fdesc_reclaim(struct vnop_reclaim_args *ap);
+extern int fdesc_root(struct mount *, struct vnode **, vfs_context_t);
+extern int fdesc_select(struct vnop_select_args *ap);
+extern int fdesc_setattr(struct vnop_setattr_args *ap);
+extern int fdesc_write(struct vnop_write_args *ap);
+
 extern int (**fdesc_vnodeop_p)(void *);
 extern struct vfsops fdesc_vfsops;
 #endif /* KERNEL */
