@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -78,8 +79,6 @@
 #define DLIL_PRINTF	kprintf
 #endif
 
-//#define DLIL_ALWAYS_DELAY_DETACH 1
-
 enum {
 	kProtoKPI_DLIL	= 0,
 	kProtoKPI_v1	= 1
@@ -632,34 +631,59 @@ dlil_detach_filter_internal(interface_filter_t filter, int detached)
 {
 	int retval = 0;
 	
-	
-	/* Take the write lock */
-#if DLIL_ALWAYS_DELAY_DETACH
-	retval = EDEADLK;
-#else
-	if (detached == 0 && (retval = dlil_write_begin()) != 0)
-#endif
-	 {
+	if (detached == 0) {
+		ifnet_t				ifp = NULL;
+		interface_filter_t	entry = NULL;
+
+		/* Take the write lock */
+	 	retval = dlil_write_begin();
+	 	if (retval != 0 && retval != EDEADLK)
+	 		return retval;
+	 	
+	 	/*
+	 	 * At this point either we have the write lock (retval == 0)
+	 	 * or we couldn't get it (retval == EDEADLK) because someone
+	 	 * else up the stack is holding the read lock. It is safe to
+	 	 * read, either the read or write is held. Verify the filter
+	 	 * parameter before proceeding.
+	 	 */
+		ifnet_head_lock_shared();
+		TAILQ_FOREACH(ifp, &ifnet_head, if_link) {
+			TAILQ_FOREACH(entry, &ifp->if_flt_head, filt_next) {
+				if (entry == filter)
+					break;
+			}
+			if (entry == filter)
+				break;
+		}
+		ifnet_head_done();
+		
+		if (entry != filter) {
+			/* filter parameter is not a valid filter ref */
+			if (retval == 0) {
+				dlil_write_end();
+			}
+			return EINVAL;
+		}
+		
 		if (retval == EDEADLK) {
 			/* Perform a delayed detach */
 			filter->filt_detaching = 1;
 			dlil_detach_waiting = 1;
 			wakeup(&dlil_detach_waiting);
-			retval = 0;
+			return 0;
 		}
-		return retval;
+		
+		/* Remove the filter from the list */
+		TAILQ_REMOVE(&ifp->if_flt_head, filter, filt_next);
+		dlil_write_end();
 	}
 	
-	if (detached == 0)
-		TAILQ_REMOVE(&filter->filt_ifp->if_flt_head, filter, filt_next);
-	
-	/* release the write lock */
-	if (detached == 0)
-		dlil_write_end();
-	
+	/* Call the detached funciton if there is one */
 	if (filter->filt_detached)
 		filter->filt_detached(filter->filt_cookie, filter->filt_ifp);
 
+	/* Free the filter */
 	FREE(filter, M_NKE);
 	
 	return retval;
@@ -668,6 +692,8 @@ dlil_detach_filter_internal(interface_filter_t filter, int detached)
 void
 dlil_detach_filter(interface_filter_t filter)
 {
+	if (filter == NULL)
+		return;
 	dlil_detach_filter_internal(filter, 0);
 }
 
@@ -972,6 +998,7 @@ dlil_event(struct ifnet *ifp, struct kern_event_msg *event)
 	return result;
 }
 
+int
 dlil_output_list(
 	struct ifnet* ifp,
 	u_long proto_family,
@@ -1964,12 +1991,7 @@ dlil_detach_protocol(struct ifnet *ifp, u_long proto_family)
 	int use_reached_zero = 0;
 	
 
-#if DLIL_ALWAYS_DELAY_DETACH
-	{
-		retval = EDEADLK;
-#else
 	if ((retval = dlil_write_begin()) != 0) {
-#endif
 		if (retval == EDEADLK) {
 			retval = 0;
 			dlil_read_begin();
