@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -136,150 +134,151 @@ adspWriteHandler(gref, mp)
 	void *sp;
 
     switch(gbuf_type(mp)) {
-    case MSG_DATA:
-	if (gref->info == 0) {
-	    gbuf_freem(mp);
-	    return(STR_IGNORE);
-        }
-	/*
-	 * Fill in the global stuff
-	 */
-	ap = (struct adspcmd *)gbuf_rptr(mp);
-	ap->gref = gref;
-	ap->ioc = 0;
-	ap->mp = mp;
-	sp = (void *)gbuf_rptr(((gbuf_t *)gref->info));
-	switch(ap->csCode) {
-	case dspWrite:
-	    if ((error = adspWrite(sp, ap)))
-		gbuf_freem(mp);
-	    return(STR_IGNORE);
-	case dspAttention:
-	    if ((error = adspAttention(sp, ap)))
-		gbuf_freem(mp);
-	    return(STR_IGNORE);
-	}
-    case MSG_IOCTL:
-	if (gref->info == 0) {
-	    adspioc_ack(EPROTO, mp, gref);
-	    return(STR_IGNORE);
-        }
-	iocbp = (ioc_t *) gbuf_rptr(mp);
-	if (ADSP_IOCTL(iocbp->ioc_cmd)) {
-	    iocbp->ioc_count = sizeof(*ap) - 1;
-	    if (gbuf_cont(mp) == 0) {
-		adspioc_ack(EINVAL, mp, gref);
-		return(STR_IGNORE);
-	    }
-	    ap = (struct adspcmd *) gbuf_rptr(gbuf_cont(mp));
-	    ap->gref = gref;
-	    ap->ioc = (caddr_t) mp;
-	    ap->mp = gbuf_cont(mp); /* request head */
-	    ap->ioResult = 0;
-
-	    if ((gref->info == 0) && ((iocbp->ioc_cmd != ADSPOPEN) &&
-			            (iocbp->ioc_cmd != ADSPCLLISTEN))) {
-	        ap->ioResult = errState;
-
-		adspioc_ack(EINVAL, mp, gref);
-		return(STR_IGNORE);
-	    }
-	}	
-	sp = (void *)gbuf_rptr(((gbuf_t *)gref->info));
-	switch(iocbp->ioc_cmd) {
-	case ADSPOPEN:
-	case ADSPCLLISTEN:
-		ap->socket = ((CCBPtr)sp)->localSocket;
-		flag = (adspMode(ap) == ocAccept) ? 1 : 0;
-		if (flag && ap->socket) {
-			if (adspDeassignSocket((CCBPtr)sp) >= 0)
-				ap->socket = 0;
-		}
-		if ((ap->socket == 0) &&
-		    ((ap->socket = 
-		      (at_socket)adspAssignSocket(gref, flag)) == 0)) {
-		    adspioc_ack(EADDRNOTAVAIL, mp, gref);
-	        return(STR_IGNORE);
-		}
-	    ap->csCode = iocbp->ioc_cmd == ADSPOPEN ? dspInit : dspCLInit;
-	    if ((error = adspInit(sp, ap)) == 0) {
+		case MSG_DATA:
+		if (gref->info == 0) {
+			gbuf_freem(mp);
+			return(STR_IGNORE);
+			}
+		/*
+		 * Fill in the global stuff
+		 */
+		ap = (struct adspcmd *)gbuf_rptr(mp);
+		ap->gref = gref;
+		ap->ioc = 0;
+		ap->mp = mp;
+		sp = (void *)gbuf_rptr(((gbuf_t *)gref->info));
 		switch(ap->csCode) {
-		case dspInit:
-		    /* and open the connection */
-		    ap->csCode = dspOpen;
-		    error = adspOpen(sp, ap);
-		    break;
-		case dspCLInit:
-		    /* ADSPCLLISTEN */
-		    ap->csCode = dspCLListen;
-		    error = adspCLListen(sp, ap);
-		    break;
+			case dspWrite:
+				if ((error = adspWrite(sp, ap)))
+				gbuf_freem(mp);
+				return(STR_IGNORE);
+			case dspAttention:
+				if ((error = adspAttention(sp, ap)))
+				gbuf_freem(mp);
+				return(STR_IGNORE);
 		}
-	    }
-	    if (error) 
-		adspioc_ack(error, mp, gref); /* if this failed req complete */
-	    return(STR_IGNORE);
-	case ADSPCLOSE:
-	    ap->csCode = dspClose;
-	    if ((error = adspClose(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-		break;
-	    }
-	    break;
-	case ADSPCLREMOVE:
-	    ap->csCode = dspCLRemove;
-	    error = adspClose(sp, ap);
-	    adspioc_ack(error, mp, gref);
-	    return(STR_IGNORE);
-	case ADSPCLDENY:
-	    ap->csCode = dspCLDeny;
-	    if ((error = adspCLDeny(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	case ADSPSTATUS:
-	    ap->csCode = dspStatus;
-	    if ((error = adspStatus(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	case ADSPREAD:
-	    ap->csCode = dspRead;
-	    if ((error = adspRead(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	case ADSPATTENTION:
-	    ap->csCode = dspAttention;
-	    if ((error = adspReadAttention(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	case ADSPOPTIONS:
-	    ap->csCode = dspOptions;
-	    if ((error = adspOptions(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	case ADSPRESET:
-	    ap->csCode = dspReset;
-	    if ((error = adspReset(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	case ADSPNEWCID:
-	    ap->csCode = dspNewCID;
-	    if ((error = adspNewCID(sp, ap))) {
-		adspioc_ack(error, mp, gref);
-	    }
-	    return(STR_IGNORE);
-	default:
-	    return(STR_PUTNEXT);	/* pass it on down */
-	}
-	return(STR_IGNORE);
-    case MSG_PROTO:
-    default:
-	gbuf_freem(mp);
+		case MSG_IOCTL:
+		if (gref->info == 0) {
+			adspioc_ack(EPROTOTYPE, mp, gref);
+			return(STR_IGNORE);
+			}
+		iocbp = (ioc_t *) gbuf_rptr(mp);
+		if (ADSP_IOCTL(iocbp->ioc_cmd)) {
+			iocbp->ioc_count = sizeof(*ap) - 1;
+			if (gbuf_cont(mp) == 0) {
+			adspioc_ack(EINVAL, mp, gref);
+			return(STR_IGNORE);
+			}
+			ap = (struct adspcmd *) gbuf_rptr(gbuf_cont(mp));
+			ap->gref = gref;
+			ap->ioc = (caddr_t) mp;
+			ap->mp = gbuf_cont(mp); /* request head */
+			ap->ioResult = 0;
+	
+			if ((gref->info == 0) && ((iocbp->ioc_cmd != ADSPOPEN) &&
+							(iocbp->ioc_cmd != ADSPCLLISTEN))) {
+				ap->ioResult = errState;
+	
+			adspioc_ack(EINVAL, mp, gref);
+			return(STR_IGNORE);
+			}
+		} else
+			return(STR_PUTNEXT);	/* pass it on down */	
+		sp = (void *)gbuf_rptr(((gbuf_t *)gref->info));
+		switch(iocbp->ioc_cmd) {
+			case ADSPOPEN:
+			case ADSPCLLISTEN:
+				ap->socket = ((CCBPtr)sp)->localSocket;
+				flag = (adspMode(ap) == ocAccept) ? 1 : 0;
+				if (flag && ap->socket) {
+					if (adspDeassignSocket((CCBPtr)sp) >= 0)
+						ap->socket = 0;
+				}
+				if ((ap->socket == 0) &&
+					((ap->socket = 
+					  (at_socket)adspAssignSocket(gref, flag)) == 0)) {
+					adspioc_ack(EADDRNOTAVAIL, mp, gref);
+					return(STR_IGNORE);
+				}
+				ap->csCode = iocbp->ioc_cmd == ADSPOPEN ? dspInit : dspCLInit;
+				if ((error = adspInit(sp, ap)) == 0) {
+				switch(ap->csCode) {
+				case dspInit:
+					/* and open the connection */
+					ap->csCode = dspOpen;
+					error = adspOpen(sp, ap);
+					break;
+				case dspCLInit:
+					/* ADSPCLLISTEN */
+					ap->csCode = dspCLListen;
+					error = adspCLListen(sp, ap);
+					break;
+				}
+				}
+				if (error) 
+				adspioc_ack(error, mp, gref); /* if this failed req complete */
+				return(STR_IGNORE);
+			case ADSPCLOSE:
+				ap->csCode = dspClose;
+				if ((error = adspClose(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				break;
+				}
+				break;
+			case ADSPCLREMOVE:
+				ap->csCode = dspCLRemove;
+				error = adspClose(sp, ap);
+				adspioc_ack(error, mp, gref);
+				return(STR_IGNORE);
+			case ADSPCLDENY:
+				ap->csCode = dspCLDeny;
+				if ((error = adspCLDeny(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			case ADSPSTATUS:
+				ap->csCode = dspStatus;
+				if ((error = adspStatus(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			case ADSPREAD:
+				ap->csCode = dspRead;
+				if ((error = adspRead(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			case ADSPATTENTION:
+				ap->csCode = dspAttention;
+				if ((error = adspReadAttention(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			case ADSPOPTIONS:
+				ap->csCode = dspOptions;
+				if ((error = adspOptions(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			case ADSPRESET:
+				ap->csCode = dspReset;
+				if ((error = adspReset(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			case ADSPNEWCID:
+				ap->csCode = dspNewCID;
+				if ((error = adspNewCID(sp, ap))) {
+				adspioc_ack(error, mp, gref);
+				}
+				return(STR_IGNORE);
+			default:
+				return(STR_PUTNEXT);	/* pass it on down */
+		}
+		return(STR_IGNORE);
+		case MSG_PROTO:
+		default:
+		gbuf_freem(mp);
     }
 }
 
