@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -30,7 +28,8 @@
 #ifndef _NETAT_AT_DDP_BRT_H_
 #define _NETAT_AT_DDP_BRT_H_
 #include <sys/appleapiopts.h>
-#ifdef __APPLE_API_PRIVATE
+#ifdef KERNEL_PRIVATE
+#ifdef __APPLE_API_OBSOLETE
 
 typedef struct {
 	int 				age_flag;
@@ -83,6 +82,7 @@ typedef struct {
 /* Best Router Cache */
 extern	ddp_brt_t	at_ddp_brt[BRTSIZE];
 
-#endif /* __APPLE_API_PRIVATE */
+#endif /* __APPLE_API_OBSOLETE */
+#endif /* KERNEL_PRIVATE */
 #endif /* _NETAT_AT_DDP_BRT_H_ */
 
