@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -130,9 +131,11 @@ int at_pcballoc(so, head)
 
 	pcb->atpcb_head = head;
 	pcb->atpcb_socket = so;
+	atalk_lock();	/* makes sure the list is locked while inserting atpcb */
 	if (head)
 	    insque((queue_t)pcb, (queue_t)head);
 	so->so_pcb = (caddr_t)pcb;
+	atalk_unlock();
 
 	return (0);
 }
@@ -149,9 +152,10 @@ int at_pcbdetach(pcb)
 	}
 
 	so->so_pcb = 0;
+	so->so_flags |= SOF_PCBCLEARING;
 	if ((pcb->atpcb_next) && (pcb->atpcb_prev))
 	    remque((queue_t)pcb);
-	zfree(atpcb_zone, (vm_offset_t)pcb);
+	zfree(atpcb_zone, pcb);
 	sofree(so);
 	return(0);
 }
