@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -39,6 +37,7 @@
 #include <sys/protosw.h>
 #include <sys/domain.h>
 #include <sys/mbuf.h>
+#include <kern/locks.h>
 
 #include <sys/sysctl.h>
 
@@ -48,7 +47,6 @@
 #include <netat/at_var.h>
 #include <netat/ddp.h>
 
-struct domain atalkdomain;
 
 extern int	ddp_pru_abort(struct socket *so);
 
@@ -74,6 +72,7 @@ extern int	ddp_pru_send(struct socket *so, int flags, struct mbuf *m,
 extern int	ddp_pru_shutdown(struct socket *so);
 extern int	ddp_pru_sockaddr(struct socket *so, 
 				 struct sockaddr **nam);
+void atalk_dominit();
 
 /*
  * Dummy usrreqs struct created by Ted for FreeBSD 3.x integration. 
@@ -84,24 +83,64 @@ struct pr_usrreqs ddp_usrreqs = {
 	ddp_pru_connect, pru_connect2_notsupp, ddp_pru_control, ddp_pru_detach,
 	ddp_pru_disconnect, pru_listen_notsupp, ddp_pru_peeraddr, pru_rcvd_notsupp,
 	pru_rcvoob_notsupp, ddp_pru_send, pru_sense_null, ddp_pru_shutdown,
-	ddp_pru_sockaddr, sosend, soreceive, sopoll
+	ddp_pru_sockaddr, sosend, soreceive, pru_sopoll_notsupp
 };
 
+struct domain atalkdomain;
 struct protosw atalksw[] = {
   { SOCK_RAW,	&atalkdomain,	/*protocol*/ 0,	PR_ATOMIC|PR_ADDR,
     /*input*/ 0, /*output*/ 0, /*clinput*/ 0, ddp_ctloutput,
     /*ousrreq*/ 0, 
     ddp_init, /*fastto*/ 0, /*slowto*/ 0, /*drain*/ 0, 
-    /*sysctl*/ 0, &ddp_usrreqs
+    /*sysctl*/ 0, &ddp_usrreqs,
+	0, 0, 0
   }
 };
 
 struct domain atalkdomain =
-{ AF_APPLETALK, "appletalk", 0, 0, 0, 
+{ AF_APPLETALK, "appletalk", atalk_dominit, 0, 0, 
   atalksw, 0,
   0, 0, 0,
   DDP_X_HDR_SIZE, 0
 };
 
+struct domain * atalkdom = &atalkdomain;
+lck_mtx_t  *atalk_mutex = NULL;
+
 SYSCTL_NODE(_net, PF_APPLETALK, appletalk, CTLFLAG_RW, 0, "AppleTalk Family");
 
+void
+atalk_dominit()
+{
+	atalk_mutex = atalkdom->dom_mtx;
+}
+
+void
+atalk_lock()
+{
+	int error = 0, lr, lr_saved;
+#ifdef __ppc__
+	__asm__ volatile("mflr %0" : "=r" (lr));
+	lr_saved = lr;
+#endif 
+	lck_mtx_assert(atalkdom->dom_mtx, LCK_MTX_ASSERT_NOTOWNED);
+	lck_mtx_lock(atalkdom->dom_mtx);
+}
+	
+void
+atalk_unlock()
+{
+	int error = 0, lr, lr_saved;
+#ifdef __ppc__
+	__asm__ volatile("mflr %0" : "=r" (lr));
+	lr_saved = lr;
+#endif 
+	lck_mtx_assert(atalkdom->dom_mtx, LCK_MTX_ASSERT_OWNED);
+	lck_mtx_unlock(atalkdom->dom_mtx);
+
+}
+	
+
+
+
+
