@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -143,7 +141,7 @@ int atp_open(gref, flag)
 	if (atp_rcb_data == NULL) {
 		if (kmem_alloc(kernel_map, &temp, sizeof(struct atp_rcb) * NATP_RCB) != KERN_SUCCESS) 
 			return(ENOMEM);
-		if (atp_rcb_data == NULL) {						/* in case we lost funnel while allocating */
+		if (atp_rcb_data == NULL) {						
 		        bzero((caddr_t)temp, sizeof(struct atp_rcb) * NATP_RCB);
 			atp_rcb_data = (struct atp_rcb*)temp;					
 			for (i = 0; i < NATP_RCB; i++) {
