@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -80,7 +78,7 @@ int aurp_open(gref)
 		if (ddp_AURPfuncx(AURPCODE_REG, AURPcmdx, 0)) {
 			aurp_gref = 0;
 			aurp_minor_no[i] = 0;
-			return EPROTO;
+			return EPROTOTYPE;
 		}
 	}
 
