@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -59,12 +57,11 @@ void AURPsndTickle(state)
 	int msize;
 	gbuf_t *m;
 	aurp_hdr_t *hdrp;
-	boolean_t 	funnel_state;
 
-	funnel_state = thread_funnel_set(network_flock, TRUE);
+	atalk_lock();
 
 	if (state->rcv_state == AURPSTATE_Unconnected) {
-                (void) thread_funnel_set(network_flock, FALSE);
+		atalk_unlock();
 		return;
         }
 	/* stop trying if the retry count exceeds the maximum retry value */
@@ -81,7 +78,7 @@ void AURPsndTickle(state)
 
 		/* purge all routes associated with the tunnel peer */
 		AURPpurgeri(state->rem_node);
-                (void) thread_funnel_set(network_flock, FALSE);
+		atalk_unlock();
 		return;
 	}
 
@@ -105,7 +102,7 @@ void AURPsndTickle(state)
 	/* start the retry timer */
 	timeout(AURPsndTickle, state, AURP_TickleRetryInterval*HZ);
 
-	(void) thread_funnel_set(network_flock, FALSE);
+	atalk_unlock();
 }
 
 /* */
