@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -130,7 +131,7 @@ void ddp_brt_init()
 	bzero(at_ddp_brt, sizeof(at_ddp_brt));
 	ddp_brt_sweep_timer = 1;
 #ifdef NOT_USED
-	timeout(ddp_brt_sweep_funnel, (long)0, BRT_SWEEP_INT * SYS_HZ);
+	timeout(ddp_brt_sweep_locked, (long)0, BRT_SWEEP_INT * SYS_HZ);
 #endif
 }
 
@@ -139,17 +140,17 @@ void ddp_brt_shutdown()
 #ifdef NOT_USED
 	bzero(at_ddp_brt, sizeof(at_ddp_brt));
 	if (ddp_brt_sweep_timer)
-		untimeout(ddp_brt_sweep_funnel, 0);
+		untimeout(ddp_brt_sweep_locked, 0);
 #endif
 	ddp_brt_sweep_timer = 0;
 }
 
-/* funneled version */
-void ddp_brt_sweep_funnel()
+/* locked version */
+void ddp_brt_sweep_locked()
 {
-        thread_funnel_set(network_flock, TRUE);
+	atalk_lock();
 	ddp_brt_sweep();
-        thread_funnel_set(network_flock, FALSE);
+	atalk_unlock();
 }
 
 void ddp_brt_sweep()
@@ -182,7 +183,7 @@ void ddp_brt_sweep()
 	  }
 #ifdef NOT_USED
 	/* set up the next sweep... */
-	timeout(ddp_brt_sweep_funnel, (long)0, BRT_SWEEP_INT * SYS_HZ);
+	timeout(ddp_brt_sweep_locked, (long)0, BRT_SWEEP_INT * SYS_HZ);
 #endif
 
 }
