@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -229,7 +227,7 @@ void nbp_input(m, ifID)
 				/* true if home zone == destination zone */
  		register int	zno, i;
  		register  gbuf_t	*m2;
-		register error_found =0;
+		register int error_found =0;
 		register at_ifaddr_t *ifIDorig;
 
 		if (!ROUTING_MODE)	/* for routers only! */
@@ -1426,7 +1424,7 @@ int nbp_new_nve_entry(nve_entry, ifID)
 		new_entry->zone_hash = nbp_strhash(&new_entry->zone);
 	}
 	new_entry->tag = tag;
-	new_entry->pid =  current_proc()->p_pid;
+	new_entry->pid =  proc_selfpid();
 
 	ATDISABLE(nve_lock_pri,NVE_LOCK);
 	TAILQ_INSERT_TAIL(&name_registry, new_entry, nve_link);
