@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -31,6 +29,8 @@
 #define _NETAT_RTMP_H_
 #include <sys/appleapiopts.h>
 
+#ifdef __APPLE_API_OBSOLETE
+
 /* Changed 03-22-94 for router support  LD */
 
 /* RTMP function codes */
@@ -65,4 +65,5 @@ typedef struct {
 	unsigned char	at_rtmp_data;
 } at_rtmp_tuple;
 
+#endif /* __APPLE_API_OBSOLETE */
 #endif /* _NETAT_RTMP_H_ */
