@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -61,7 +59,7 @@
 #ifndef _NETINET_TCP_DEBUG_H_
 #define	_NETINET_TCP_DEBUG_H_
 #include <sys/appleapiopts.h>
-#ifdef __APPLE_API_PRIVATE
+#ifdef PRIVATE
 
 struct	tcp_debug {
 	n_time	td_time;
@@ -100,11 +98,6 @@ static char	*tanames[] =
 
 #define	TCP_NDEBUG 100
 
-#ifndef KERNEL
-/* XXX common variables for broken applications. */
-struct	tcp_debug tcp_debug[TCP_NDEBUG];
-int	tcp_debx;
-#endif
-#endif /* __APPLE_API_PRIVATE */
+#endif /* PRIVATE */
 
 #endif /* !_NETINET_TCP_DEBUG_H_ */
