@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -102,7 +100,7 @@
 #define	TCPS_HAVEESTABLISHED(s)	((s) >= TCPS_ESTABLISHED)
 #define	TCPS_HAVERCVDFIN(s)	((s) >= TCPS_TIME_WAIT)
 
-#ifdef __APPLE_API_UNSTABLE
+#ifdef KERNEL_PRIVATE
 #ifdef	TCPOUTFLAGS
 /*
  * Flags used when sending segments in tcp_output.
@@ -124,19 +122,19 @@ static u_char	tcp_outflags[TCP_NSTATES] = {
 	TH_ACK,			/* 10, TIME_WAIT */
 };	
 #endif
-#endif /* __APPLE_API_UNSTABLE */
+#endif KERNEL_PRIVATE
 
 #if KPROF
-#ifdef __APPLE_API_PRIVATE
+#ifdef KERNEL_PRIVATE
 int	tcp_acounts[TCP_NSTATES][PRU_NREQ];
-#endif /* __APPLE_API_PRIVATE */
+#endif KERNEL_PRIVATE
 #endif
 
 #ifdef	TCPSTATES
 char *tcpstates[] = {
 	"CLOSED",	"LISTEN",	"SYN_SENT",	"SYN_RCVD",
 	"ESTABLISHED",	"CLOSE_WAIT",	"FIN_WAIT_1",	"CLOSING",
-	"LAST_ACK",	"FIN_WAIT_2",	"TIME_WAIT",
+	"LAST_ACK",	"FIN_WAIT_2",	"TIME_WAIT"
 };
 #endif
 
