@@ -62,8 +62,8 @@ struct pfkeystat {
 #define KEY_SENDUP_ALL		1
 #define KEY_SENDUP_REGISTERED	2
 
+#ifdef KERNEL_PRIVATE
 #ifdef KERNEL
-#ifdef __APPLE_API_PRIVATE
 struct keycb {
 	struct rawcb kp_raw;	/* rawcb */
 	int kp_promisc;		/* promiscuous mode */
@@ -73,16 +73,16 @@ struct keycb {
 extern struct pfkeystat pfkeystat;
 
 #ifdef __APPLE__
-extern int key_output __P((struct mbuf *, struct socket* so));
+extern int key_output(struct mbuf *, struct socket* so);
 #else
-extern int key_output __P((struct mbuf *, ...));
+extern int key_output(struct mbuf *, ...);
 #endif
-extern int key_usrreq __P((struct socket *,
-	int, struct mbuf *, struct mbuf *, struct mbuf *));
+extern int key_usrreq(struct socket *,
+	int, struct mbuf *, struct mbuf *, struct mbuf *);
 
-extern int key_sendup __P((struct socket *, struct sadb_msg *, u_int, int));
-extern int key_sendup_mbuf __P((struct socket *, struct mbuf *, int));
-#endif /* __APPLE_API_PRIVATE */
+extern int key_sendup(struct socket *, struct sadb_msg *, u_int, int);
+extern int key_sendup_mbuf(struct socket *, struct mbuf *, int);
 #endif /* KERNEL */
+#endif /* KERNEL_PRIVATE */
 
 #endif /*_NETKEY_KEYSOCK_H_*/
