@@ -1,3 +1,25 @@
+/*
+ * Copyright (c) 2002-2005 Apple Computer, Inc. All rights reserved.
+ *
+ * @APPLE_LICENSE_HEADER_START@
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
+ * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
+ * @APPLE_LICENSE_HEADER_END@
+ */
 /*-
  * Copyright (c) 1998 Berkeley Software Design, Inc. All rights reserved.
  * Redistribution and use in source and binary forms, with or without
@@ -60,7 +82,7 @@ typedef struct nfs_lock_msg {
 	u_int64_t		lm_xid;			/* unique message transaction ID */
 	struct flock		lm_fl;			/* The lock request. */
 	struct sockaddr_storage lm_addr;		/* The address. */
-	size_t			lm_fh_len;		/* The file handle length. */
+	int			lm_fh_len;		/* The file handle length. */
 	struct xucred		lm_cred;		/* user cred for lock req */
 	u_int8_t		lm_fh[NFS_SMALLFH];	/* The file handle. */
 } LOCKD_MSG;
@@ -96,7 +118,7 @@ struct lockd_ans {
 	pid_t		la_pid;			/* pid of lock requester/owner */
 	off_t		la_start;		/* lock starting offset */
 	off_t		la_len;			/* lock length */
-	size_t		la_fh_len;		/* The file handle length. */
+	int 		la_fh_len;		/* The file handle length. */
 	u_int8_t	la_fh[NFS_SMALLFH];	/* The file handle. */
 };
 
@@ -108,9 +130,12 @@ struct lockd_ans {
 
 #ifdef KERNEL
 void	nfs_lockinit(void);
-int	nfs_dolock(struct vop_advlock_args *ap);
-int	nfslockdans(struct proc *p, struct lockd_ans *ansp);
-int	nfslockdfd(struct proc *p, int fd);
-int	nfslockdwait(struct proc *p);
+int	nfs_dolock(struct vnop_advlock_args *ap);
+int	nfslockdans(proc_t p, struct lockd_ans *ansp);
+int	nfslockdfd(proc_t p, int fd);
+int	nfslockdwait(proc_t p);
+
+extern vnode_t nfslockdvnode;
+extern int nfslockdwaiting;
 #endif
 #endif /* __APPLE_API_PRIVATE */
