@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -48,7 +46,7 @@
 
 #include <sys/appleapiopts.h>
 
-#ifdef __APPLE_API_OBSOLETE
+#ifdef BSD_KERNEL_PRIVATE
 /* Size of a page in an object file. */
 #define	__LDPGSZ	4096
 
@@ -99,13 +97,7 @@ struct exec {
 	unsigned int	a_drsize;	/* data relocation size */
 };
 
-/*
- * Address of ps_strings structure (in user space).
- */
-#define	PS_STRINGS \
-	((struct ps_strings *)(USRSTACK - sizeof(struct ps_strings)))
-
-#endif /* __APPLE_API_OBSOLETE */
+#endif /* BSD_KERNEL_PRIVATE */
 
 #endif /* _BSD_PPC_EXEC_H_ */
 
