@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000-2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -66,7 +64,7 @@
 #define _SYS_ACCT_H_
 
 #include <sys/appleapiopts.h>
-
+#include <sys/cdefs.h>
 /*
  * Accounting structures; these use a comp_t type which is a 3 bits base 8
  * exponent, 13 bit fraction ``floating point'' number.  Units are 1/AHZ
@@ -103,7 +101,11 @@ struct acct {
 #ifdef KERNEL
 #ifdef __APPLE_API_PRIVATE
 extern struct vnode	*acctp;
-int	acct_process __P((struct proc *p));
+
+__BEGIN_DECLS
+int	acct_process(struct proc *p);
+__END_DECLS
+
 #endif /* __APPLE_API_PRIVATE */
 #endif /* KERNEL */
 
