@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -38,15 +39,15 @@ struct aio_workq_entry
 {
 	TAILQ_ENTRY( aio_workq_entry )	aio_workq_link;
 	struct proc						*procp;		/* user proc that queued this request */
-	struct aiocb					*uaiocbp;  	/* pointer passed in from user land */
-	struct aiocb					*fsyncp;	/* not NULL means this request must complete */
+	user_addr_t						uaiocbp;  	/* pointer passed in from user land */
+	user_addr_t						fsyncp;		/* not NULL means this request must complete */
 												/* before an aio_fsync call can proceed. */
 	vm_map_t						aio_map;	/* user land map we have a reference to */
-	ssize_t							returnval;	/* return value from read / write request */	
+	user_ssize_t					returnval;	/* return value from read / write request */	
 	int								errorval;	/* error value from read / write request */
 	int								flags;		
 	long							group_tag;	/* identifier used to group IO requests */
-	struct aiocb					aiocb;		/* copy of aiocb from user land */
+	struct user_aiocb				aiocb;		/* copy of aiocb from user land */
 };
 typedef struct aio_workq_entry aio_workq_entry;
 
@@ -66,11 +67,27 @@ typedef struct aio_workq_entry aio_workq_entry;
 									/* 	waiting for one or more active IO requests to */
 									/*	complete */
 
+/*
+ * Prototypes
+ */
 
-__private_extern__ void		_aio_close( struct proc *p, int fd );
-__private_extern__ void		_aio_exit( struct proc *p );
-__private_extern__ void		_aio_exec( struct proc *p );
-__private_extern__ void		_aio_create_worker_threads( int num );
+__private_extern__ void		
+_aio_close(struct proc *p, int fd);
+
+__private_extern__ void		
+_aio_exit(struct proc *p);
+
+__private_extern__ void		
+_aio_exec(struct proc *p);
+
+__private_extern__ void		
+_aio_create_worker_threads(int num);
+
+__private_extern__ void		
+aio_init(void);
+
+task_t						
+get_aiotask(void);
 
 #endif /* KERNEL */
 
