@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -25,6 +26,7 @@
 
 #include <sys/ioctl.h>
 #include <sys/types.h>
+#include <sys/cdefs.h>
 
 /*
  * Definitions
@@ -54,6 +56,10 @@
  * DKIOCGETMAXSEGMENTBYTECOUNTWRITE get maximum segment byte count for writes
  */
 
+#if __DARWIN_ALIGN_POWER
+#pragma options align=power
+#endif
+
 typedef struct
 {
     char path[128];
@@ -67,6 +73,7 @@ typedef struct
     u_int8_t               reserved0096[4];        /* reserved, clear to zero */
 } dk_format_capacity_t;
 
+/* LP64todo: not 64-bit clean */
 typedef struct
 {
     dk_format_capacity_t * capacities;
@@ -75,6 +82,10 @@ typedef struct
     u_int8_t               reserved0064[8];        /* reserved, clear to zero */
 } dk_format_capacities_t;
 
+#if __DARWIN_ALIGN_POWER
+#pragma options align=reset
+#endif
+
 #define DKIOCEJECT                       _IO('d', 21)
 #define DKIOCSYNCHRONIZECACHE            _IO('d', 22)
 
@@ -98,10 +109,11 @@ typedef struct
 #define DKIOCGETMAXSEGMENTBYTECOUNTWRITE _IOR('d', 69, u_int64_t)
 
 #ifdef KERNEL
-#define DKIOCGETISVIRTUAL                _IOR('d', 72, u_int32_t)
 #define DKIOCGETBLOCKCOUNT32             _IOR('d', 25, u_int32_t)
 #define DKIOCSETBLOCKSIZE                _IOW('d', 24, u_int32_t)
 #define DKIOCGETBSDUNIT                  _IOR('d', 27, u_int32_t)
+#define DKIOCISVIRTUAL                   _IOR('d', 72, u_int32_t)
+#define DKIOCGETBASE                     _IOR('d', 73, u_int64_t)
 #endif /* KERNEL */
 
 #endif	/* _SYS_DISK_H_ */
