@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -61,6 +59,7 @@
 #define _SYS_DISKLABEL_H_
 
 #include <sys/appleapiopts.h>
+#include <sys/types.h>		/* for daddr_t */
 
 #ifdef __APPLE_API_OBSOLETE
 
@@ -360,7 +359,7 @@ struct partinfo {
 #include <sys/cdefs.h>
 
 __BEGIN_DECLS
-struct disklabel *getdiskbyname __P((const char *));
+struct disklabel *getdiskbyname(const char *);
 __END_DECLS
 
 #endif
