@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -66,17 +64,11 @@
 #ifndef	_SYS_DKSTAT_H_
 #define	_SYS_DKSTAT_H_
 
-#include <sys/appleapiopts.h>
-
-#ifdef	__APPLE_API_PRIVATE
-
-#ifdef KERNEL
+#ifdef KERNEL_PRIVATE
 extern long tk_cancc;
 extern long tk_nin;
 extern long tk_nout;
 extern long tk_rawcc;
 #endif
 
-#endif	/* __APPLE_API_PRIVATE */
-
 #endif	/* _SYS_DKSTAT_H_ */
