@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -27,9 +25,12 @@
 #ifndef _SYS_EV_H_
 #define _SYS_EV_H_
 
+#if !defined(__LP64__)
+
 #include <sys/appleapiopts.h>
 
 #include <sys/queue.h>
+#include <sys/cdefs.h>
 
 struct eventreq {
   int      er_type;
@@ -62,8 +63,7 @@ typedef struct eventreq *er_t;
 #define EV_TIMEOUT 0x20000
 #define EV_DMASK   0xffffff00
 
-#ifdef KERNEL
-#ifdef __APPLE_API_PRIVATE
+#ifdef BSD_KERNEL_PRIVATE
 
 struct eventqelt {
   TAILQ_ENTRY(eventqelt)  ee_slist;
@@ -71,12 +71,13 @@ struct eventqelt {
   struct eventreq  ee_req;
   struct proc *    ee_proc;
   u_int            ee_flags;
-#define EV_QUEUED 1
+#define EV_QUEUED	0x01
   u_int            ee_eventmask;
-  struct socket   *ee_sp;
 };
 
-#endif /* __APPLE_API_PRIVATE */
-#endif /* KERNEL */
+int waitevent_close(struct proc *p, struct fileproc *);
+#endif /* BSD_KERNEL_PRIVATE */
+
+#endif /* !defined(__LP64__) */
 
 #endif /* _SYS_EV_H_ */
