@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000-2002 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -62,6 +60,7 @@
 #define	_SYS_PTRACE_H_
 
 #include <sys/appleapiopts.h>
+#include <sys/cdefs.h>
 
 #define	PT_TRACE_ME	0	/* child declares it's being traced */
 #define	PT_READ_I	1	/* read word in child's I space */
@@ -83,21 +82,21 @@
 #define	PT_DENY_ATTACH	31
 
 #define	PT_FIRSTMACH	32	/* for machine-specific requests */
-#include <machine/ptrace.h>	/* machine-specific requests, if any */
+
+__BEGIN_DECLS
 
 #ifdef KERNEL
 #ifdef __APPLE_API_PRIVATE
-void	proc_reparent __P((struct proc *child, struct proc *newparent));
+
+void	proc_reparent(struct proc *child, struct proc *newparent);
 
 #endif /* __APPLE_API_PRIVATE */
 #else /* !KERNEL */
 
-#include <sys/cdefs.h>
-
-__BEGIN_DECLS
-int	ptrace __P((int _request, pid_t _pid, caddr_t _addr, int _data));
-__END_DECLS
+int	ptrace(int _request, pid_t _pid, caddr_t _addr, int _data);
 
 #endif /* !KERNEL */
 
+__END_DECLS
+
 #endif	/* !_SYS_PTRACE_H_ */
