@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -29,6 +27,7 @@
 
 
 #include <sys/appleapiopts.h>
+#include <sys/cdefs.h>
 
 /* Kernel Events Protocol */ 
 #define SYSPROTO_EVENT 		1	/* kernel events protocol */
@@ -48,15 +47,17 @@ struct sockaddr_sys
 
 
 #ifdef KERNEL
-#ifdef __APPLE_API_PRIVATE
+#ifdef KERNEL_PRIVATE
 
 extern struct domain systemdomain;
 
 /* built in system domain protocols init function */
-int kern_event_init();
-int kern_control_init();
+__BEGIN_DECLS
+int kern_event_init(void);
+int kern_control_init(void);
+__END_DECLS
 
-#endif /* __APPLE_API_PRIVATE */
+#endif /* KERNEL_PRIVATE */
 #endif /* KERNEL */
 
 #endif /* _SYSTEM_DOMAIN_H_ */
