@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -64,17 +65,30 @@
 #define _SYS_TIMEB_H_
 
 #include <sys/appleapiopts.h>
+#include <sys/cdefs.h>
+#include <sys/_types.h>
 
-#ifdef __APPLE_API_OBSOLETE
+/* [XSI] The time_t type shall be defined as described in <sys/types.h> */
+#ifndef	_TIME_T
+#define	_TIME_T
+typedef	__darwin_time_t	time_t;
+#endif
 
-/* The ftime(2) system call structure -- deprecated. */
+/*
+ * [XSI] Structure whose address is passed as the first parameter to ftime()
+ */
 struct timeb {
-	time_t	time;			/* seconds since the Epoch */
-	unsigned short millitm;		/* + milliseconds since the Epoch */
-	short	timezone;		/* minutes west of CUT */
-	short	dstflag;		/* DST == non-zero */
+	time_t		time;		/* [XSI] Seconds since the Epoch */
+	unsigned short	millitm;	/* [XSI] Milliseconds since the Epoch */
+	short		timezone;	/* [XSI] Minutes west of CUT */
+	short		dstflag;	/* [XSI] non-zero if DST in effect */
 };
 
-#endif /* __APPLE_API_OBSOLETE */
+#ifndef KERNEL
+__BEGIN_DECLS
+/* [XSI] Legacy interface */
+int	ftime(struct timeb *);
+__END_DECLS
+#endif /* !KERNEL */
 
 #endif /* !_SYS_TIMEB_H_ */
