@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -136,7 +134,7 @@ extern struct	proc *traceproc;
 extern int	tracewhich, tracebuf[TRCSIZ];
 extern u_int	tracex;
 extern char	traceflags[TR_NFLAGS];
-#define	pack(v,b)	(((v)->v_mount->mnt_stat.f_fsid.val[0])<<16)|(b)
+#define	pack(v,b)	(((v)->v_mount->mnt_vfsstat.f_fsid.val[0])<<16)|(b)
 #define	trace(a,b,c) {							\
 	if (traceflags[a])						\
 		trace1(a,b,c);						\
