@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -61,20 +62,20 @@
 #ifndef	_SYS_UTSNAME_H
 #define	_SYS_UTSNAME_H
 
+#include <sys/cdefs.h>
+
 #define	_SYS_NAMELEN	256
 
 struct	utsname {
-	char	sysname[_SYS_NAMELEN];	/* Name of OS */
-	char	nodename[_SYS_NAMELEN];	/* Name of this network node */
-	char	release[_SYS_NAMELEN];	/* Release level */
-	char	version[_SYS_NAMELEN];	/* Version level */
-	char	machine[_SYS_NAMELEN];	/* Hardware type */
+	char	sysname[_SYS_NAMELEN];	/* [XSI] Name of OS */
+	char	nodename[_SYS_NAMELEN];	/* [XSI] Name of this network node */
+	char	release[_SYS_NAMELEN];	/* [XSI] Release level */
+	char	version[_SYS_NAMELEN];	/* [XSI] Version level */
+	char	machine[_SYS_NAMELEN];	/* [XSI] Hardware type */
 };
 
-#include <sys/cdefs.h>
-
 __BEGIN_DECLS
-int uname __P((struct utsname *));
+int uname(struct utsname *);
 __END_DECLS
 
 #endif	/* !_SYS_UTSNAME_H */
