@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -36,7 +37,7 @@
 #include <sys/time.h>
 #include <sys/attr.h>
 
-#ifndef _POSIX_SOURCE
+#ifndef _POSIX_C_SOURCE
 
 struct vstat {
 	fsid_t			vst_volid;		/* volume identifier */
@@ -49,7 +50,7 @@ struct vstat {
 	gid_t	  		vst_gid;		/* group ID of the file's group */
 	dev_t			vst_dev;		/* inode's device */
 	dev_t	  		vst_rdev;		/* device type */
-#ifndef _POSIX_SOURCE
+#ifndef _POSIX_C_SOURCE
 	struct	timespec vst_atimespec;	/* time of last access */
 	struct	timespec vst_mtimespec;	/* time of last data modification */
 	struct	timespec vst_ctimespec;	/* time of last file status change */
@@ -67,7 +68,7 @@ struct vstat {
 	u_int32_t 		vst_flags;		/* user defined flags for file */
 };
 
-#endif /* ! _POSIX_SOURCE */
+#endif /* ! _POSIX_C_SOURCE */
 #endif /* __APPLE_API_OBSOLETE */
 
 #endif /* !_SYS_VSTAT_H_ */
