@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -34,13 +35,12 @@
 #include <kern/queue.h>
 
 #ifdef	KERNEL
-
 #include <mach/boolean.h>
+#include <mach/memory_object_types.h>
+#include <mach/vm_types.h>
 #include <vm/vm_pager.h>
 
-vm_pager_t	vnode_pager_setup();
-boolean_t	vnode_has_page();
-boolean_t	vnode_pager_active();
+vm_pager_t	vnode_pager_setup(struct vnode *, memory_object_t);
 
 /*
  *  Vstructs are the internal (to us) description of a unit of backing store.
@@ -89,6 +89,7 @@ struct bs_map {
 	struct vnode    *vp;   
 	void     	*bs;
 };
+extern struct bs_map  bs_port_table[];
 
 
 
@@ -123,11 +124,15 @@ typedef struct vstruct {
 #define	VNODE_PAGER_NULL	((vnode_pager_t) 0)
 
 
+pager_return_t	vnode_pagein(struct vnode *, upl_t,
+			     upl_offset_t, vm_object_offset_t,
+			     upl_size_t, int, int *);
+pager_return_t	vnode_pageout(struct vnode *, upl_t,
+			      upl_offset_t, vm_object_offset_t,
+			      upl_size_t, int, int *);
 
-pager_return_t	pager_vnode_pagein();
-pager_return_t	pager_vnode_pageout();
-pager_return_t	vnode_pagein();
-pager_return_t	vnode_pageout();
+extern vm_object_offset_t vnode_pager_get_filesize(
+	struct vnode *vp);
 
 #endif	/* KERNEL */
 
