@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -32,19 +30,16 @@
 #define super IOService
 OSDefineMetaClassAndStructors(AppleIntelClock, IOService);
 
-extern "C" {
-extern void	hardclock(void);
-};
-
 bool
 AppleIntelClock::start(IOService *provider)
 {
 	if (!super::start(provider))
 		return false;
 
-	provider->registerInterrupt(kIRQ_Clock, 0, (IOInterruptAction) hardclock);
-	provider->enableInterrupt(kIRQ_Clock);
-
+	/*
+	 * The clock is already provided by the kernel, so all we need 
+	 * here is publish its availability for any IOKit client to use.
+	 */
 	publishResource("IORTC", this);
 	return true;
 }
