@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -43,7 +41,7 @@ OSDefineMetaClassAndStructors(AppleCPU, IOCPU);
 bool AppleCPU::start(IOService *provider)
 {
   kern_return_t       result;
-  ml_processor_info_t processor_info;
+  ml_processor_info_t this_processor_info;
   
   if (!super::start(provider)) return false;
   
@@ -55,16 +53,18 @@ bool AppleCPU::start(IOService *provider)
   
   cpuIC->registerCPUInterruptController();
   
-  processor_info.cpu_id           = (cpu_id_t)this;
-  processor_info.boot_cpu         = true;
-  processor_info.start_paddr      = 0;
-  processor_info.supports_nap     = false;
-  processor_info.l2cr_value       = 0;
-  processor_info.time_base_enable = 0;
+  this_processor_info.cpu_id           = (cpu_id_t)this;
+  this_processor_info.boot_cpu         = true;
+  this_processor_info.start_paddr      = 0;
+  this_processor_info.supports_nap     = false;
+  this_processor_info.l2cr_value       = 0;
+  this_processor_info.time_base_enable = 0;
   
   // Register this CPU with mach.
-  result = ml_processor_register(&processor_info, &machProcessor,
-				 &ipi_handler);
+  result = ml_processor_register(
+				&this_processor_info,
+				&machProcessor,
+				&ipi_handler);
   if (result == KERN_FAILURE) return false;
   
   setCPUState(kIOCPUStateUninitalized);
