@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -76,6 +74,9 @@ public:
   static  void           initCPUs(void);
   
   virtual bool           start(IOService *provider);
+  virtual OSObject       *getProperty(const OSSymbol *aKey) const;
+  virtual bool           setProperty(const OSSymbol *aKey, OSObject *anObject);
+  virtual bool           serializeProperties(OSSerialize *serialize) const;
   virtual IOReturn       setProperties(OSObject *properties);
   virtual void           initCPU(bool boot) = 0;
   virtual void           quiesceCPU(void) = 0;
