@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -53,8 +51,6 @@
 #include <kern/queue.h>
 #include <libkern/c++/OSObject.h>
 
-class IOCommandPool;
-
 /*!
  * @class IOCommand
  * @abstract
@@ -67,12 +63,11 @@ class IOCommandPool;
  
 class IOCommand : public OSObject
 {
-    OSDeclareAbstractStructors(IOCommand)
-    
-protected:
-    virtual bool init(void);
+    OSDeclareDefaultStructors(IOCommand)
     
 public:
+    virtual bool init(void);
+    
 /*! @var fCommandChain
     This variable is used by the current 'owner' to queue the command.  During the life cycle of a command it moves through a series of queues.  This is the queue pointer for it.  Only valid while 'ownership' is clear.  For instance a IOCommandPool uses this pointer to maintain its list of free commands.  May be manipulated using the kern/queue.h macros */
     queue_chain_t fCommandChain;	/* used to queue commands */
