@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -59,7 +57,7 @@ extern "C" {
 enum {
     kIOServiceMatching		= 100,
     kIOBSDNameMatching		= 101,
-    kIOOFPathMatching		= 102,
+    kIOOFPathMatching		= 102
 };
 
 // IOCatalogueSendData
@@ -77,7 +75,7 @@ enum {
     kIOCatalogRemoveDrivers,
     kIOCatalogRemoveDriversNoMatch,
     kIOCatalogStartMatching,
-    kIOCatalogRemoveKernelLinker,
+    kIOCatalogRemoveKernelLinker
 };
 
 // IOCatalogueGetData
@@ -86,7 +84,10 @@ enum {
     @constant kIOCatalogGetContents  Returns a snapshot of the database to the caller.
 */
 enum {
-    kIOCatalogGetContents	= 1,
+    kIOCatalogGetContents	  = 1,
+    kIOCatalogGetModuleDemandList = 2,
+    kIOCatalogGetCacheMissList	  = 3,
+    kIOCatalogGetROMMkextList     = 4
 };
 
 // IOCatalogueReset
@@ -95,7 +96,7 @@ enum {
     @constant kIOCatalogResetDefault  Removes all entries from IOCatalogue except those used for booting the system.
 */
 enum {
-    kIOCatalogResetDefault	= 1,
+    kIOCatalogResetDefault	= 1
 };
 
 // IOCatalogueTerminate
@@ -108,7 +109,7 @@ enum {
 enum {
     kIOCatalogModuleUnload      = 1,
     kIOCatalogModuleTerminate,
-    kIOCatalogServiceTerminate,
+    kIOCatalogServiceTerminate
 };
 
 enum {
