@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -28,8 +29,15 @@
 
 typedef UInt32 IOMessage;
 
-#define iokit_common_msg(message)     (UInt32)(sys_iokit|sub_iokit_common|message)
-#define iokit_family_msg(sub,message) (UInt32)(sys_iokit|sub|message)
+#define iokit_common_msg(message)          (UInt32)(sys_iokit|sub_iokit_common|message)
+#define iokit_family_msg(sub,message)      (UInt32)(sys_iokit|sub|message)
+
+/*! @defined iokit_vendor_specific_msg
+    @discussion iokit_vendor_specific_msg passes messages in the sub_iokit_vendor_specific
+    subsystem. It can be used to be generate messages that are used for private 
+    communication between vendor specific code with the  IOService::message() etc. APIs.
+*/
+#define iokit_vendor_specific_msg(message) (UInt32)(sys_iokit|sub_iokit_vendor_specific|message)
 
 #define kIOMessageServiceIsTerminated      iokit_common_msg(0x010)
 #define kIOMessageServiceIsSuspended       iokit_common_msg(0x020)
