@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -65,9 +63,13 @@ protected:
     AbsoluteTime abstime;
 
 /*! @struct ExpansionData
-    @discussion This structure will be used to expand the capablilties of the IOWorkLoop in the future.
+    @discussion This structure is private to the IOTimerEventSource implementation.
     */    
-    struct ExpansionData { };
+    struct ExpansionData
+    {
+        SInt32	     calloutGeneration;
+        IOWorkLoop * workLoop;
+    };
 
 /*! @var reserved
     Reserved for future use.  (Internal use only)  */
@@ -92,6 +94,8 @@ protected:
     @abstract Have to implement it is mandatory in $link IOEventSource, but IOTimerEventSources don't actually use this work-loop mechanism. */
     virtual bool checkForWork();
 
+    virtual void setWorkLoop(IOWorkLoop *workLoop);
+
 public:
 
 /*! @typedef Action
@@ -206,6 +210,9 @@ public:
     @discussion Clear down any oustanding calls.  By the time this function completes it is guaranteed that the action will not be called again. */
    virtual void cancelTimeout();
 
+private:
+    static void timeoutAndRelease(void *self, void *wl);
+
 private:
     OSMetaClassDeclareReservedUnused(IOTimerEventSource, 0);
     OSMetaClassDeclareReservedUnused(IOTimerEventSource, 1);
