@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -49,16 +50,6 @@
 
 #include <architecture/i386/asm_help.h>
 
-// 'Till we're building in kernel
-.macro DISABLE_PREEMPTION
-#ifdef KERNEL
-#endif
-.endmacro
-.macro ENABLE_PREEMPTION
-#ifdef KERNEL
-#endif
-.endmacro
-
 /* 
  * void
  * ev_lock(p)
@@ -95,7 +86,6 @@ LEAF(_ev_unlock, 0)
 LEAF(_IOSpinUnlock, 0)
 	movl		4(%esp), %ecx
 	movl		$0, (%ecx)
-	ENABLE_PREEMPTION()
 	ret
 END(_ev_unlock)
 
@@ -111,7 +101,6 @@ END(_ev_unlock)
 
 LEAF(_ev_try_lock, 0)
 LEAF(_IOTrySpinLock, 0)
-        DISABLE_PREEMPTION()
         movl            4(%esp), %ecx 
 	xorl		%eax, %eax
         lock
@@ -120,7 +109,6 @@ LEAF(_IOTrySpinLock, 0)
 	movl	$1, %eax		/* yes */
 	ret
 1:
-	ENABLE_PREEMPTION()
 	xorl	%eax, %eax		/* no */
 END(_ev_try_lock)
 
