@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -22,7 +20,7 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
-#warning include <mach.h> is going away, please don't use it anymore.
+#warning include <mach.h> is going away, please do not use it anymore.
 
 #include <sys/appleapiopts.h>
 
