@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -40,6 +38,8 @@
 
 __BEGIN_DECLS
 
+#ifdef	KERNEL_PRIVATE
+
 OS_INLINE
 uint64_t
 __OSAbsoluteTime(
@@ -56,7 +56,31 @@ __OSAbsoluteTimePtr(
 	return ((uint64_t *)abstime);
 }
 
+#define AbsoluteTime_to_scalar(x)	(*(uint64_t *)(x))
+
+/* t1 < = > t2 */
+#define CMP_ABSOLUTETIME(t1, t2)				\
+	(AbsoluteTime_to_scalar(t1) >				\
+		AbsoluteTime_to_scalar(t2)? (int)+1 :	\
+	 (AbsoluteTime_to_scalar(t1) <				\
+		AbsoluteTime_to_scalar(t2)? (int)-1 : 0))
+
+/* t1 += t2 */
+#define ADD_ABSOLUTETIME(t1, t2)				\
+	(AbsoluteTime_to_scalar(t1) +=				\
+				AbsoluteTime_to_scalar(t2))
+
+/* t1 -= t2 */
+#define SUB_ABSOLUTETIME(t1, t2)				\
+	(AbsoluteTime_to_scalar(t1) -=				\
+				AbsoluteTime_to_scalar(t2))
+
+#define ADD_ABSOLUTETIME_TICKS(t1, ticks)		\
+	(AbsoluteTime_to_scalar(t1) +=				\
+						(int32_t)(ticks))
+
+#endif	/* KERNEL_PRIVATE */
+
 __END_DECLS
 
 #endif /* _OS_OSBASE_H */
-
