@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -104,17 +105,17 @@ OSHostByteOrder(void) {
 OS_INLINE
 uint16_t
 OSReadBigInt16(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
-    return *(volatile uint16_t *)((int8_t *)base + offset);
+    return *(volatile uint16_t *)((uintptr_t)base + offset);
 }
 
 OS_INLINE
 uint32_t
 OSReadBigInt32(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -124,7 +125,7 @@ OSReadBigInt32(
 OS_INLINE
 uint64_t
 OSReadBigInt64(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -334,7 +335,7 @@ OSSwapBigToHostInt64(
 OS_INLINE
 uint16_t
 OSReadBigInt16(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -344,7 +345,7 @@ OSReadBigInt16(
 OS_INLINE
 uint32_t
 OSReadBigInt32(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -354,7 +355,7 @@ OSReadBigInt32(
 OS_INLINE
 uint64_t
 OSReadBigInt64(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -401,7 +402,7 @@ OSWriteBigInt64(
 OS_INLINE
 uint16_t
 OSReadLittleInt16(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -411,7 +412,7 @@ OSReadLittleInt16(
 OS_INLINE
 uint32_t
 OSReadLittleInt32(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
@@ -421,7 +422,7 @@ OSReadLittleInt32(
 OS_INLINE
 uint64_t
 OSReadLittleInt64(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                     offset
 )
 {
