@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -59,14 +57,29 @@ private:
     virtual bool initWithCStringNoCopy(const char *cString);
 
 protected:
+    /*!
+        @function taggedRelease
+        @abstract Overriden super class release method so we can synchronise with the symbol pool.
+        @discussion When we release an symbol we need to synchronise the destruction of the object with any potential searches that may be occuring through the family factor methods.  See OSObject::taggedRelease
+    */
+    virtual void taggedRelease(const void *tag, const int when) const;
+
     /*!
         @function free
-        @abstract A member function to release all resources created or used by the OSString object.
-        @discussion This function should not be called directly, use release() instead.
+        @abstract Overriden super class release method so we can synchronise with the symbol pool.
+        @discussion When we release an symbol we need to synchronise the destruction of the object with any potential searches that may be occuring through the family factor methods.  See OSObject::free
     */
     virtual void free();
 
 public:
+    /*!
+        @function taggedRelease
+        @abstract Release a tag.
+        @discussion The C++ language has forced me to override this method even though I have implemented it as { super::taggedRelease(tag) }. It seems that C++ is confused about the appearance of the protected taggedRelease with 2 args and refuses to only inherit one function.  See OSObject::taggedRelease
+    */
+    virtual void taggedRelease(const void *tag) const;
+
+
     /*!
         @function withString
         @abstract A static constructor function to create an OSSymbol instance from an OSString object or returns an existing OSSymbol object based on the OSString object given.
