@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -41,7 +42,7 @@
 OS_INLINE
 uint16_t
 OSReadSwapInt16(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                          offset
 )
 {
@@ -56,7 +57,7 @@ OSReadSwapInt16(
 OS_INLINE
 uint32_t
 OSReadSwapInt32(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                          offset
 )
 {
@@ -71,17 +72,17 @@ OSReadSwapInt32(
 OS_INLINE
 uint64_t
 OSReadSwapInt64(
-    volatile void               * base,
+    const volatile void               * base,
     uintptr_t                          offset
 )
 {
-    uint64_t * inp;
+    const volatile uint64_t * inp;
     union ullc {
         uint64_t     ull;
         uint32_t     ul[2];
     } outv;
 
-    inp = (uint64_t *)base;
+    inp = (const volatile uint64_t *)base;
     outv.ul[0] = OSReadSwapInt32(inp, offset + 4);
     outv.ul[1] = OSReadSwapInt32(inp, offset);
     return outv.ull;
@@ -125,14 +126,14 @@ OSWriteSwapInt64(
     uint64_t                        data
 )
 {
-    uint64_t * outp;
-    union ullc {
+    volatile uint64_t * outp;
+    volatile union ullc {
         uint64_t     ull;
         uint32_t     ul[2];
     } *inp;
 
-    outp = (uint64_t *)base;
-    inp  = (union ullc *)&data;
+    outp = (volatile uint64_t *)base;
+    inp  = (volatile union ullc *)&data;
     OSWriteSwapInt32(outp, offset, inp->ul[1]);
     OSWriteSwapInt32(outp, offset + 4, inp->ul[0]);
 }
