@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -59,9 +57,11 @@
 /*
  * Symbol table routines for a.out format files.
  */
+#include <mach/mach_types.h>
 #include <mach/boolean.h>
 #include <mach/std_types.h>
 #include <machine/db_machdep.h>		/* data types */
+#include <vm/pmap.h>
 #include <string.h>			/* For strcpy(), strcmp() */
 #include <ddb/db_aout.h>
 #include <ddb/db_output.h>		/* For db_printf() */
@@ -663,7 +663,7 @@ aout_db_search_symbol(
 	db_strategy_t	strategy,
 	db_expr_t	*diffp)		/* in/out */
 {
-	register unsigned long	diff = *diffp;
+	db_expr_t diff = *diffp;
 	register struct nlist	*symp = 0;
 	struct nlist		*sp, *ep, *cp;
 	boolean_t		first_pass = FALSE;
