@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -201,7 +199,7 @@ void db_set_breakpoint(
 	task_t		task,
 	db_addr_t	addr,
 	int		count,
-	thread_act_t	thr_act,
+	thread_t	thr_act,
 	boolean_t	task_bpt);
 
 db_breakpoint_t db_find_breakpoint(
