@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -66,10 +67,10 @@
  * space.
  */
 #define db_current_task()						\
-		((current_act())? current_act()->task: TASK_NULL)
+		((current_thread())? current_thread()->task: TASK_NULL)
 #define db_current_space()						\
-		((current_act())?\
-			current_act()->task: TASK_NULL)
+		((current_thread())?\
+			current_thread()->task: TASK_NULL)
 #define db_target_space(thr_act, user_space)				\
 		((!(user_space) || ((thr_act)))?\
 			TASK_NULL:					\
@@ -79,24 +80,24 @@
 		((task) == TASK_NULL || (task) == db_current_space())
 
 extern task_t		db_default_task;	/* default target task */
-extern thread_act_t	db_default_act;		/* default target thr_act */
+extern thread_t	db_default_act;		/* default target thr_act */
 
 
 /* Prototypes for functions exported by this module.
  */
 
-int db_lookup_act(thread_act_t target_act);
+int db_lookup_act(thread_t target_act);
 
 int db_lookup_task(task_t target_task);
 
 int db_lookup_task_act(
 	task_t		task,
-	thread_act_t		target_act);
+	thread_t		target_act);
 
-boolean_t db_check_act_address_valid(thread_act_t thr_act);
+boolean_t db_check_act_address_valid(thread_t thr_act);
 
 boolean_t db_get_next_act(
-	thread_act_t		*actp,
+	thread_t		*actp,
 	int		position);
 
 void db_init_default_act(void);
