@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -159,7 +160,7 @@ db_cmp_variable_name(
 		  	      || (vp->high >= 0 && ap->suffix[0] > vp->high))))
 	    return(FALSE);
 	strcpy(ap->modif, (*np)? np+1: "");
-	ap->thr_act = (db_option(ap->modif, 't')?db_default_act: THR_ACT_NULL);
+	ap->thr_act = (db_option(ap->modif, 't')?db_default_act: THREAD_NULL);
 	ap->level = level;
 	ap->hidden_level = -1;
 	return(TRUE);
@@ -249,7 +250,7 @@ db_read_write_variable(
 	    ap = &aux_param;
 	    ap->modif = "";
 	    ap->level = 0;
-	    ap->thr_act = THR_ACT_NULL;
+	    ap->thr_act = THREAD_NULL;
 	}
 	if (rw_flag == DB_VAR_SET && vp->precious)
 		db_read_write_variable(vp, &old_value, DB_VAR_GET, ap);
@@ -262,7 +263,7 @@ db_read_write_variable(
 	    (*func)(vp, valuep, rw_flag, ap);
 	if (rw_flag == DB_VAR_SET && vp->precious)
 		db_printf("\t$%s:%s<%#x>\t%#8lln\t=\t%#8lln\n", vp->name,
-			  ap->modif, ap->thr_act, old_value, *valuep);
+			  ap->modif, ap->thr_act, (unsigned long long)old_value, (unsigned long long)*valuep);
 }
 
 void
@@ -431,7 +432,7 @@ db_show_one_variable(void)
 
 	    strcpy(aux_param.modif, *p ? p + 1 : "");
 	    aux_param.thr_act = (db_option(aux_param.modif, 't') ?
-			db_default_act : THR_ACT_NULL);
+			db_default_act : THREAD_NULL);
 	}
 
 	if (cur->hidden_level)
@@ -504,14 +505,14 @@ db_show_one_variable(void)
 		aux_param.suffix[0] = i;
 		(*cur->fcn)(cur, (db_expr_t *)0, DB_VAR_SHOW, &aux_param);
 	    } else {
-		db_printf("%#lln", *(cur->valuep + i));
+		db_printf("%#lln", (unsigned long long)*(cur->valuep + i));
 	        db_find_xtrn_task_sym_and_offset(*(cur->valuep + i), &name,
 						 &offset, TASK_NULL);
 		if (name != (char *)0 && offset <= db_maxoff &&
 		    offset != *(cur->valuep + i)) {
 		    db_printf("\t%s", name);
 		    if (offset != 0)
-			db_printf("+%#r", offset);
+			db_printf("+%#llr", (unsigned long long)offset);
 		}
 	    }
 	    db_putchar('\n');
@@ -594,7 +595,7 @@ db_show_variable(void)
 
 	aux_param.modif = "";
 	aux_param.level = 1;
-	aux_param.thr_act = THR_ACT_NULL;
+	aux_param.thr_act = THREAD_NULL;
 
 	for (cur = db_vars; cur < db_evars; cur++) {
 	    i = cur->low;
@@ -650,14 +651,14 @@ db_show_variable(void)
 		    aux_param.suffix[0] = i;
 		    (*cur->fcn)(cur, (db_expr_t *)0, DB_VAR_SHOW, &aux_param);
 		} else {
-		    db_printf("%#lln", *(cur->valuep + i));
+		    db_printf("%#lln", (unsigned long long)*(cur->valuep + i));
 		    db_find_xtrn_task_sym_and_offset(*(cur->valuep + i), &name,
 						     &offset, TASK_NULL);
 		    if (name != (char *)0 && offset <= db_maxoff &&
 			offset != *(cur->valuep + i)) {
 			db_printf("\t%s", name);
 			if (offset != 0)
-			    db_printf("+%#r", offset);
+			    db_printf("+%#llr", (unsigned long long)offset);
 		    }
 		}
 		db_putchar('\n');
