@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -172,7 +170,7 @@ struct db_var_aux_param {
 	short		level;			/* number of levels */
 	short		hidden_level;		/* hidden level */
 	short		suffix[DB_VAR_LEVEL];	/* suffix */
-	thread_act_t	thr_act;		/* target thr_act */
+	thread_t	thr_act;		/* target thr_act */
 };
 
 typedef struct db_var_aux_param	*db_var_aux_param_t;
