@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -51,6 +52,8 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <kern/spl.h>
 #include <kern/processor.h>
 #include <kern/misc_protos.h>
+#include <i386/cpu_data.h>
+#include <i386/cpu_number.h>
 #include <i386/pio.h>
 #include <i386/AT386/rtc.h>
 #include <i386/AT386/bbclock_entries.h>
@@ -81,13 +84,12 @@ bbc_config(void)
 	int		BbcFlag;
 	struct rtc_st	rtclk;
 
-#if	NCPUS > 1 && AT386
 	mp_disable_preemption();
 	if (cpu_number() != master_cpu) {
 		mp_enable_preemption();
 		return(1);
 	}
-#endif
+
 	/*
 	 * Setup device.
 	 */
@@ -104,9 +106,7 @@ bbc_config(void)
 		printf("battery clock configured\n");
 	else
 		printf("WARNING: Battery Clock Failure!\n");
-#if	NCPUS > 1 && AT386
 	mp_enable_preemption();
-#endif
 	return (BbcFlag);
 }
 
@@ -124,7 +124,6 @@ bbc_gettime(
 	spl_t		s;
 	thread_t	thread;
 
-#if 	NCPUS > 1 && AT386
 	if ((thread = current_thread()) != THREAD_NULL) {
 		thread_bind(thread, master_processor);
 		mp_disable_preemption();
@@ -135,7 +134,7 @@ bbc_gettime(
 			mp_enable_preemption();
 		}
 	}
-#endif
+
 	s = LOCK_BBC();
 	rtcget(&rtclk);
 	sec = hexdectodec(rtclk.rtc_sec);
@@ -159,10 +158,8 @@ bbc_gettime(
 	cur_time->tv_nsec = 0;
 	UNLOCK_BBC(s);
 
-#if	NCPUS > 1 && AT386
 	if (thread != THREAD_NULL)
 		thread_bind(thread, PROCESSOR_NULL);
-#endif
 	return (KERN_SUCCESS);
 }
 
@@ -179,7 +176,6 @@ bbc_settime(
 	spl_t		s;
 	thread_t	thread;
 
-#if	NCPUS > 1 && AT386
 	if ((thread = current_thread()) != THREAD_NULL) {
 		thread_bind(thread, master_processor);
 		mp_disable_preemption();
@@ -190,7 +186,7 @@ bbc_settime(
 			mp_enable_preemption();
 		}
 	}
-#endif
+
 	s = LOCK_BBC();
 	rtcget(&rtclk);
 	diff = 0;
@@ -214,39 +210,9 @@ bbc_settime(
 	rtcput(&rtclk);
 	UNLOCK_BBC(s);
 
-#if	NCPUS > 1 && AT386
 	if (thread != THREAD_NULL)
 		thread_bind(current_thread(), PROCESSOR_NULL);
-#endif
-	return (KERN_SUCCESS);
-}
 
-/*
- * Get clock device attributes.
- */
-kern_return_t
-bbc_getattr(
-	clock_flavor_t		flavor,
-	clock_attr_t		attr,		/* OUT */
-	mach_msg_type_number_t	*count)		/* IN/OUT */
-{
-	if (*count != 1)
-		return (KERN_FAILURE);
-	switch (flavor) {
-
-	case CLOCK_GET_TIME_RES:	/* >0 res */
-		*(clock_res_t *) attr = NSEC_PER_SEC;
-		break;
-
-	case CLOCK_ALARM_CURRES:	/* =0 no alarm */
-	case CLOCK_ALARM_MINRES:
-	case CLOCK_ALARM_MAXRES:
-		*(clock_res_t *) attr = 0;
-		break;
-
-	default:
-		return (KERN_INVALID_VALUE);
-	}
 	return (KERN_SUCCESS);
 }
 
@@ -258,7 +224,7 @@ rtcget(
 	struct rtc_st	* regs)
 {
 	outb(RTC_ADDR, RTC_D); 
-	if (inb(RTC_DATA) & RTC_VRT == 0)
+	if ((inb(RTC_DATA) & RTC_VRT) == 0)
 		return (-1);
 	outb(RTC_ADDR, RTC_A);	
 	while (inb(RTC_DATA) & RTC_UIP)		/* busy wait */
