@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -61,8 +59,6 @@
 #include <types.h>
 #include <kern/clock.h>
 
-#include <i386/rtclock_entries.h>
-
 /*
  * Clock device subsystem configuration. The clock_list[]
  * table contains the clock structures for all clocks in
@@ -77,9 +73,9 @@ extern	struct clock_ops	sysclk_ops, calend_ops;
 struct	clock	clock_list[] = {
 
 	/* SYSTEM_CLOCK */
-	{ &sysclk_ops,		0,	0,	0 },
+	{ &sysclk_ops, 0, 0, {0} },
 
 	/* CALENDAR_CLOCK */
-	{ &calend_ops,		0,	0,	0 },
+	{ &calend_ops, 0, 0, {0} }
 };
 int	clock_count = sizeof(clock_list) / sizeof(clock_list[0]);
