@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -107,13 +105,3 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define	SPL_CMP_GE(a, b)	((unsigned)(a) >= (unsigned)(b))
 #define	SPL_CMP_LE(a, b)	((unsigned)(a) <= (unsigned)(b))
 
-#ifdef	MACH_KERNEL
-#ifndef	ASSEMBLER
-
-#include <i386/AT386/misc_protos.h>
-
-extern i386_intr_t		ivect[];
-extern int		iunit[];
-extern unsigned char	intpri[];
-#endif	/* ASSEMBLER */
-#endif	/* MACH_KERNEL */
