@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -55,6 +56,7 @@
  * same LDT.
  */
 #include <i386/seg.h>
+#include <i386/pmap.h>
 #include <mach/i386/vm_types.h>
 #include <mach/i386/vm_param.h>
 
@@ -73,18 +75,31 @@ struct fake_descriptor	ldt[LDTSZ] = {
           ACC_P|ACC_PL_U|ACC_CALL_GATE
         },				/* call gate for mach rpc */
 /*017*/	{ 0,
-	  (VM_MAX_ADDRESS-VM_MIN_ADDRESS-1)>>12,
-	  SZ_32|SZ_G,
+	  0xfffff,
+ 	  SZ_32|SZ_G,
 	  ACC_P|ACC_PL_U|ACC_CODE_R
 	},				/* user code segment */
 /*01F*/	{ 0,
-	  (VM_MAX_ADDRESS-VM_MIN_ADDRESS-1)>>12,
+	  0xfffff,
 	  SZ_32|SZ_G,
 	  ACC_P|ACC_PL_U|ACC_DATA_W
 	},				/* user data segment */
 /*027*/	{ 0,
-	  (VM_MAX_ADDRESS-VM_MIN_ADDRESS-1)>>12,
+	  0xfffff,
 	  SZ_32|SZ_G,
 	  ACC_P|ACC_PL_U|ACC_DATA_W
 	},				/* user cthread segment */
+// Storage space for user ldt entries we will make room for 10 entries initially
+// as we will probably never need many more than that
+/*02F*/	{ 0, 0, 0, 0},
+/*037*/	{ 0, 0, 0, 0},
+/*03F*/	{ 0, 0, 0, 0},
+/*047*/	{ 0, 0, 0, 0},
+/*04F*/	{ 0, 0, 0, 0},
+/*057*/	{ 0, 0, 0, 0},
+/*05F*/	{ 0, 0, 0, 0},
+/*067*/	{ 0, 0, 0, 0},
+/*06F*/	{ 0, 0, 0, 0},
+/*077*/	{ 0, 0, 0, 0},
+
 };
