@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -31,8 +32,11 @@
 #include <kern/kern_types.h>
 #include <pexpert/pexpert.h>
 
+#include <sys/cdefs.h>
 #include <sys/appleapiopts.h>
 
+__BEGIN_DECLS
+
 /* Interrupt handling */
 
 /* Initialize Interrupts */
@@ -70,6 +74,7 @@ struct ml_processor_info {
 
 typedef struct ml_processor_info ml_processor_info_t;
 
+
 /* Register a processor */
 kern_return_t ml_processor_register(
         cpu_id_t cpu_id,
@@ -178,7 +183,7 @@ struct ml_cpu_info {
 typedef struct ml_cpu_info ml_cpu_info_t;
 
 /* Get processor info */
-void ml_cpu_get_info(ml_cpu_info_t *cpu_info);
+void ml_cpu_get_info(ml_cpu_info_t *ml_cpu_info);
 
 #endif /* __APPLE_API_UNSTABLE */
 
@@ -202,14 +207,6 @@ void bzero_phys(
 	addr64_t phys_address,
 	uint32_t length);
 
-#ifdef  MACH_KERNEL_PRIVATE 
-
-void machine_idle(void);
-
-void machine_signal_idle(
-        processor_t processor);
-#endif /* MACH_KERNEL_PRIVATE */
-
 void ml_thread_policy(
 	thread_t thread,
 	unsigned policy_id,
@@ -228,6 +225,15 @@ void ml_init_max_cpus(
 int ml_get_max_cpus(
 	void);
 
+extern void	ml_cpu_up(void);
+extern void	ml_cpu_down(void);
+
+extern int	set_be_bit(void);
+extern int	clr_be_bit(void);
+extern int	be_tracing(void);
+
 #endif /* __APPLE_API_PRIVATE */
 
+__END_DECLS
+
 #endif /* _I386_MACHINE_ROUTINES_H_ */
