@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -83,6 +81,6 @@ extern boolean_t selector_check(
 			int		sel,
 			int		type);
 extern void	user_ldt_free(
-			user_ldt_t	ldt);
+			user_ldt_t	uldt);
 
 #endif	/* _I386_USER_LDT_H_ */
