@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -119,7 +120,8 @@ ipc_entry_tree_collision(
 	ipc_splay_tree_bounds(&space->is_tree, name, &lower, &upper);
 
 	index = MACH_PORT_INDEX(name);
-	return (((lower != ~0) && (MACH_PORT_INDEX(lower) == index)) ||
+	return (((lower != (mach_port_name_t)~0) && 
+		 (MACH_PORT_INDEX(lower) == index)) ||
 		((upper != 0) && (MACH_PORT_INDEX(upper) == index)));
 }
 
@@ -535,7 +537,6 @@ ipc_entry_dealloc(
 			ipc_tree_entry_t tentry;
 			mach_port_name_t tname;
 			boolean_t pick;
-			ipc_entry_bits_t bits;
 			ipc_object_t obj;
 
 			/* must move an entry from tree to table */
@@ -634,8 +635,8 @@ ipc_entry_dealloc(
 
 kern_return_t
 ipc_entry_grow_table(
-	ipc_space_t	space,
-	int		target_size)
+	ipc_space_t		space,
+	ipc_table_elems_t	target_size)
 {
 	ipc_entry_num_t osize, size, nsize, psize;
 
@@ -654,10 +655,7 @@ ipc_entry_grow_table(
 			 *	We just wait for them to finish.
 			 */
 
-			assert_wait((event_t) space, THREAD_UNINT);
-			is_write_unlock(space);
-			thread_block((void (*)(void)) 0);
-			is_write_lock(space);
+			is_write_sleep(space);
 			return KERN_SUCCESS;
 		}
 
