@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -214,8 +212,7 @@ typedef struct ipc_hash_global_bucket {
 
 #define	IHGB_NULL	((ipc_hash_global_bucket_t) 0)
 
-#define	ihgb_lock_init(ihgb)	mutex_init(&(ihgb)->ihgb_lock_data, \
-					   ETAP_IPC_IHGB)
+#define	ihgb_lock_init(ihgb)	mutex_init(&(ihgb)->ihgb_lock_data, 0)
 #define	ihgb_lock(ihgb)		mutex_lock(&(ihgb)->ihgb_lock_data)
 #define	ihgb_unlock(ihgb)	mutex_unlock(&(ihgb)->ihgb_lock_data)
 
@@ -287,17 +284,14 @@ ipc_hash_global_lookup(
 
 void
 ipc_hash_global_insert(
-	ipc_space_t			space,
-	ipc_object_t			obj,
-	mach_port_name_t		name,
-	ipc_tree_entry_t		entry)
+	ipc_space_t				space,
+	ipc_object_t				obj,
+	__assert_only mach_port_name_t	name,
+	ipc_tree_entry_t			entry)
 {
 	ipc_hash_global_bucket_t bucket;
 
-
 	assert(!is_fast_space(space));
-
-
 	assert(entry->ite_name == name);
 	assert(space != IS_NULL);
 	assert(entry->ite_space == space);
@@ -328,16 +322,15 @@ ipc_hash_global_insert(
 
 void
 ipc_hash_global_delete(
-	ipc_space_t			space,
-	ipc_object_t			obj,
-	mach_port_name_t		name,
-	ipc_tree_entry_t		entry)
+	ipc_space_t				space,
+	ipc_object_t				obj,
+	__assert_only mach_port_name_t	name,
+	ipc_tree_entry_t			entry)
 {
 	ipc_hash_global_bucket_t bucket;
 	ipc_tree_entry_t this, *last;
 
 	assert(!is_fast_space(space));
-
 	assert(entry->ite_name == name);
 	assert(space != IS_NULL);
 	assert(entry->ite_space == space);
@@ -460,10 +453,10 @@ ipc_hash_local_lookup(
 
 void
 ipc_hash_local_insert(
-	ipc_space_t		space,
-	ipc_object_t		obj,
-	mach_port_index_t	index,
-	ipc_entry_t		entry)
+	ipc_space_t			space,
+	ipc_object_t			obj,
+	mach_port_index_t		index,
+	__assert_only ipc_entry_t	entry)
 {
 	ipc_entry_t table;
 	ipc_entry_num_t size;
@@ -504,10 +497,10 @@ ipc_hash_local_insert(
 
 void
 ipc_hash_local_delete(
-	ipc_space_t		space,
-	ipc_object_t		obj,
-	mach_port_index_t	index,
-	ipc_entry_t		entry)
+	ipc_space_t			space,
+	ipc_object_t			obj,
+	mach_port_index_t		index,
+	__assert_only ipc_entry_t	entry)
 {
 	ipc_entry_t table;
 	ipc_entry_num_t size;
