@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -63,10 +61,11 @@
 #ifndef	_IPC_IPC_HASH_H_
 #define _IPC_IPC_HASH_H_
 
+#include <mach/port.h>
+#include <mach/mach_types.h>
 #include <mach/boolean.h>
 #include <mach/kern_return.h>
-#include <mach_debug/hash_info.h>
-
+#include <ipc/ipc_types.h>
 /*
  * Exported interfaces
  */
@@ -126,6 +125,8 @@ extern void ipc_hash_init(void);
 
 #if	MACH_IPC_DEBUG
 
+#include <mach_debug/hash_info.h>
+
 extern natural_t ipc_hash_info(
 	hash_info_bucket_t	*info,
 	mach_msg_type_number_t count);
