@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -68,6 +69,8 @@
 #include <ipc/ipc_port.h>
 #include <ipc/ipc_print.h>
 
+#include <kern/kern_types.h>
+#include <kern/spl.h>
 /*
  *	Routine:	ipc_pset_alloc
  *	Purpose:
@@ -235,8 +238,6 @@ kern_return_t
 ipc_pset_remove_from_all(
 	ipc_port_t	port)
 {
-	ipc_pset_t pset;
-
 	assert(ip_active(port));
 	
 	if (port->ip_pset_count == 0)
@@ -319,13 +320,10 @@ ipc_list_count(
  *	Purpose:
  *		Pretty-print a port set for kdb.
  */
-
 void
 ipc_pset_print(
 	ipc_pset_t	pset)
 {
-	extern int db_indent;
-
 	printf("pset 0x%x\n", pset);
 
 	db_indent += 2;
