@@ -1,77 +1,28 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
  * @OSF_COPYRIGHT@
  */
-/*
- * HISTORY
- * 
- * Revision 1.1.1.1  1998/09/22 21:05:28  wsanchez
- * Import of Mac OS X kernel (~semeria)
- *
- * Revision 1.1.1.1  1998/03/07 02:26:16  wsanchez
- * Import of OSF Mach kernel (~mburg)
- *
- * Revision 1.1.6.1  1994/09/23  02:11:47  ezf
- * 	change marker to not FREE
- * 	[1994/09/22  21:30:41  ezf]
- *
- * Revision 1.1.2.3  1993/07/22  16:17:25  rod
- * 	Add ANSI prototypes.  CR #9523.
- * 	[1993/07/22  13:33:20  rod]
- * 
- * Revision 1.1.2.2  1993/06/02  23:33:40  jeffc
- * 	Added to OSF/1 R1.3 from NMK15.0.
- * 	[1993/06/02  21:11:07  jeffc]
- * 
- * Revision 1.1  1992/09/30  02:08:11  robert
- * 	Initial revision
- * 
- * $EndLog$
- */
-/* CMU_HIST */
-/*
- * Revision 2.5  91/10/09  16:10:41  af
- * 	 Revision 2.4.2.1  91/09/16  10:16:00  rpd
- * 	 	Added MACH_PORT_SMALLEST, MACH_PORT_LARGEST definitions to reduce lint.
- * 	 	[91/09/02            rpd]
- * 
- * Revision 2.4.2.1  91/09/16  10:16:00  rpd
- * 	Added MACH_PORT_SMALLEST, MACH_PORT_LARGEST definitions to reduce lint.
- * 	[91/09/02            rpd]
- * 
- * Revision 2.4  91/05/14  16:37:08  mrt
- * 	Correcting copyright
- * 
- * Revision 2.3  91/02/05  17:23:52  mrt
- * 	Changed to new Mach copyright
- * 	[91/02/01  15:51:43  mrt]
- * 
- * Revision 2.2  90/06/02  14:51:49  rpd
- * 	Created for new IPC.
- * 	[90/03/26  21:03:46  rpd]
- * 
- */
-/* CMU_ENDHIST */
 /* 
  * Mach Operating System
  * Copyright (c) 1991,1990,1989 Carnegie Mellon University
@@ -478,9 +429,9 @@ ipc_splay_tree_insert(
 
 void
 ipc_splay_tree_delete(
-	ipc_splay_tree_t	splay,
-	mach_port_name_t	name,
-	ipc_tree_entry_t	entry)
+	ipc_splay_tree_t			splay,
+	mach_port_name_t			name,
+	__assert_only ipc_tree_entry_t	entry)
 {
 	ipc_tree_entry_t root, saved;
 
