@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -56,6 +57,7 @@ typedef boolean_t
     unsigned short *
 );
 
+extern
 boolean_t
 kdp_packet(
     unsigned char *,
@@ -63,9 +65,11 @@ kdp_packet(
     unsigned short *
 );
 
+extern
 boolean_t
-kdp_remove_all_breakpoints ();
+kdp_remove_all_breakpoints (void);
 
+extern
 void
 kdp_exception(
     unsigned char *,
@@ -76,42 +80,44 @@ kdp_exception(
     unsigned int
 );
 
+extern
 boolean_t
 kdp_exception_ack(
     unsigned char *,
     int
 );
 
+extern
 void
 kdp_panic(
     const char		*msg
 );
 
-void
-kdp_reset(
-    void
-);
-
+extern
 void
 kdp_reboot(
     void
 );
 
+extern
 void
 kdp_us_spin(
     int usec
 );
 
+extern
 int
 kdp_intr_disbl(
     void
 );
 
+extern
 void
 kdp_intr_enbl(
     int s
 );
 
+extern
 kdp_error_t
 kdp_machine_read_regs(
     unsigned int cpu,
@@ -120,6 +126,7 @@ kdp_machine_read_regs(
     int *size
 );
 
+extern
 kdp_error_t
 kdp_machine_write_regs(
     unsigned int cpu,
@@ -128,13 +135,19 @@ kdp_machine_write_regs(
     int *size
 );
 
+extern
 void
 kdp_machine_hostinfo(
     kdp_hostinfo_t *hostinfo
 );
 
+extern
 void
 kdp_sync_cache(
     void
 );
 
+unsigned int
+kdp_ml_get_breakinsn(
+    void
+);
