@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -82,13 +83,13 @@ struct ip {
 		u_long ip_w;
 		struct {
 			unsigned int
-#if _BIG_ENDIAN == __LITTLE_ENDIAN__
+#ifdef __LITTLE_ENDIAN__
         		ip_xhl:4,	/* header length */   
                 	ip_xv:4,		/* version */
         		ip_xtos:8,	/* type of service */
         		ip_xlen:16;	/* total length */
 #endif
-#if _BIG_ENDIAN == __BIG_ENDIAN__
+#ifdef __BIG_ENDIAN__
         		ip_xv:4,                 /* version */
                 	ip_xhl:4,                /* header length */
         		ip_xtos:8,               /* type of service */
@@ -125,8 +126,9 @@ typedef struct ether_header ether_header_t;
 #define ETHERTYPE_IP       0x0800  /* IP protocol */
 
 #define ntohs(x)           OSSwapBigToHostInt16(x)
+#define ntohl(x)           OSSwapBigToHostInt32(x)
 #define htons(x)           OSSwapHostToBigInt16(x)
-
+#define htonl(x)           OSSwapHostToBigInt32(x)
 /*
  * Ethernet Address Resolution Protocol.
  *
