@@ -1,36 +1,39 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 #include <mach/mach_types.h>
-#include <kern/queue.h>
-#include <kern/ast.h>
+
+#include <kern/kern_types.h>
+#include <kern/processor.h>
 #include <kern/thread.h>
-#include <kern/thread_act.h>
 #include <kern/task.h>
 #include <kern/spl.h>
 #include <kern/lock.h>
-#include <vm/vm_map.h>
-#include <vm/pmap.h>
+#include <kern/ast.h>
 #include <ipc/ipc_port.h>
 #include <ipc/ipc_object.h>
+#include <vm/vm_map.h>
+#include <vm/pmap.h>
+#include <vm/vm_protos.h> /* last */
 
 #undef thread_should_halt
 #undef ipc_port_release
@@ -41,27 +44,16 @@ task_t	bsd_init_task = TASK_NULL;
 char	init_task_failure_data[1024];
 extern unsigned int not_in_kdp; /* Skip acquiring locks if we're in kdp */
  
-thread_act_t get_firstthread(task_t);
-vm_map_t  get_task_map(task_t);
-ipc_space_t  get_task_ipcspace(task_t);
-boolean_t is_kerneltask(task_t);
-boolean_t is_thread_idle(thread_t);
-vm_offset_t get_map_min( vm_map_t);
-vm_offset_t get_map_max( vm_map_t);
+thread_t get_firstthread(task_t);
 int get_task_userstop(task_t);
-int get_thread_userstop(thread_act_t);
+int get_thread_userstop(thread_t);
 boolean_t thread_should_abort(thread_t);
 boolean_t current_thread_aborted(void);
-void task_act_iterate_wth_args(task_t, void(*)(thread_act_t, void *), void *);
+void task_act_iterate_wth_args(task_t, void(*)(thread_t, void *), void *);
 void ipc_port_release(ipc_port_t);
 boolean_t is_thread_active(thread_t);
-kern_return_t get_thread_waitresult(thread_t);
-vm_size_t get_vmmap_size(vm_map_t);
-int get_vmmap_entries(vm_map_t);
-int  get_task_numacts(task_t);
-thread_act_t get_firstthread(task_t task);
-kern_return_t get_signalact(task_t , thread_act_t *, int);
-void astbsd_on(void);
+kern_return_t get_signalact(task_t , thread_t *, int);
+int get_vmsubmap_entries(vm_map_t, vm_object_offset_t, vm_object_offset_t);
 
 /*
  *
@@ -82,7 +74,7 @@ void set_bsdtask_info(task_t t,void * v)
 /*
  *
  */
-void *get_bsdthread_info(thread_act_t th)
+void *get_bsdthread_info(thread_t th)
 {
 	return(th->uthread);
 }
@@ -93,121 +85,143 @@ void *get_bsdthread_info(thread_act_t th)
  * can't go away, so we make sure it is still active after
  * retrieving the first thread for extra safety.
  */
-thread_act_t get_firstthread(task_t task)
+thread_t get_firstthread(task_t task)
 {
-	thread_act_t	thr_act;
+	thread_t	thread = (thread_t)queue_first(&task->threads);
+
+	if (queue_end(&task->threads, (queue_entry_t)thread))
+		thread = THREAD_NULL;
 
-	thr_act = (thread_act_t)queue_first(&task->threads);
-	if (queue_end(&task->threads, (queue_entry_t)thr_act))
-		thr_act = THR_ACT_NULL;
 	if (!task->active)
-		return(THR_ACT_NULL);
-	return(thr_act);
+		return (THREAD_NULL);
+
+	return (thread);
 }
 
-kern_return_t get_signalact(task_t task,thread_act_t * thact, int setast)
+kern_return_t
+get_signalact(
+	task_t		task,
+	thread_t	*result_out,
+	int			setast)
 {
-
-        thread_act_t inc;
-        thread_act_t ninc;
-        thread_act_t thr_act;
-	thread_t	th;
+	kern_return_t	result = KERN_SUCCESS;
+	thread_t		inc, thread = THREAD_NULL;
 
 	task_lock(task);
+
 	if (!task->active) {
 		task_unlock(task);
-		return(KERN_FAILURE);
+
+		return (KERN_FAILURE);
 	}
 
-        thr_act = THR_ACT_NULL;
-        for (inc  = (thread_act_t)queue_first(&task->threads);
-			 !queue_end(&task->threads, (queue_entry_t)inc);
-             inc  = ninc) {
-                th = act_lock_thread(inc);
-                if ((inc->active)  && 
-                    ((th->state & (TH_ABORT|TH_ABORT_SAFELY)) != TH_ABORT)) {
-                    thr_act = inc;
-                   break;
+	for (inc  = (thread_t)queue_first(&task->threads);
+			!queue_end(&task->threads, (queue_entry_t)inc); ) {
+                thread_mtx_lock(inc);
+                if (inc->active  && 
+	                    (inc->state & (TH_ABORT|TH_ABORT_SAFELY)) != TH_ABORT) {
+                    thread = inc;
+					break;
                 }
-                act_unlock_thread(inc);
-                ninc = (thread_act_t)queue_next(&inc->task_threads);
-        }
-out:
-        if (thact) 
-                *thact = thr_act;
-        if (thr_act) {
-                if (setast)
-                    act_set_astbsd(thr_act);
+                thread_mtx_unlock(inc);
+
+				inc = (thread_t)queue_next(&inc->task_threads);
+	}
+
+	if (result_out) 
+		*result_out = thread;
+
+	if (thread) {
+		if (setast)
+			act_set_astbsd(thread);
+
+		thread_mtx_unlock(thread);
+	}
+	else
+		result = KERN_FAILURE;
 
-                act_unlock_thread(thr_act);
-        }
 	task_unlock(task);
 
-        if (thr_act) 
-            return(KERN_SUCCESS);
-        else 
-            return(KERN_FAILURE);
+	return (result);
 }
 
 
-kern_return_t check_actforsig(task_t task, thread_act_t thact, int setast)
+kern_return_t
+check_actforsig(
+	task_t			task,
+	thread_t		thread,
+	int				setast)
 {
-
-        thread_act_t inc;
-        thread_act_t ninc;
-        thread_act_t thr_act;
-		thread_t	th;
-		int found=0;
+	kern_return_t	result = KERN_FAILURE;
+	thread_t		inc;
 
 	task_lock(task);
+
 	if (!task->active) {
 		task_unlock(task);
-		return(KERN_FAILURE);
+
+		return (KERN_FAILURE);
 	}
 
-        thr_act = THR_ACT_NULL;
-        for (inc  = (thread_act_t)queue_first(&task->threads);
-			 !queue_end(&task->threads, (queue_entry_t)inc);
-             inc  = ninc) {
+	for (inc  = (thread_t)queue_first(&task->threads);
+			!queue_end(&task->threads, (queue_entry_t)inc); ) {
+		if (inc == thread) {
+			thread_mtx_lock(inc);
 
-				if (inc != thact) {
-                	ninc = (thread_act_t)queue_next(&inc->task_threads);
-						continue;
-				}
-                th = act_lock_thread(inc);
-                if ((inc->active)  && 
-                    ((th->state & (TH_ABORT|TH_ABORT_SAFELY)) != TH_ABORT)) {
-					found = 1;
-                    thr_act = inc;
-                   break;
-                }
-                act_unlock_thread(inc);
-                /* ninc = (thread_act_t)queue_next(&inc->thr_acts); */
+			if (inc->active  && 
+				(inc->state & (TH_ABORT|TH_ABORT_SAFELY)) != TH_ABORT) {
+				result = KERN_SUCCESS;
 				break;
-        }
-out:
-		if (found) {
-            if (setast)
-				act_set_astbsd(thr_act);
+			}
 
-           act_unlock_thread(thr_act);
-        }
-		task_unlock(task);
+			thread_mtx_unlock(inc);
+			break;
+		}
 
-        if (found) 
-            return(KERN_SUCCESS);
-        else 
-            return(KERN_FAILURE);
+		inc = (thread_t)queue_next(&inc->task_threads);
+	}
+
+	if (result == KERN_SUCCESS) {
+		if (setast)
+			act_set_astbsd(thread);
+
+		thread_mtx_unlock(thread);
+	}
+
+	task_unlock(task);
+
+	return (result);
 }
 
 /*
- *
+ * This is only safe to call from a thread executing in
+ * in the task's context or if the task is locked  Otherwise,
+ * the map could be switched for the task (and freed) before
+ * we to return it here.
  */
 vm_map_t  get_task_map(task_t t)
 {
 	return(t->map);
 }
 
+vm_map_t  get_task_map_reference(task_t t)
+{
+	vm_map_t m;
+
+	if (t == NULL)
+		return VM_MAP_NULL;
+
+	task_lock(t);
+	if (!t->active) {
+		task_unlock(t);
+		return VM_MAP_NULL;
+	}
+	m = t->map;
+	vm_map_reference_swap(m);
+	task_unlock(t);
+	return m;
+}
+
 /*
  *
  */
@@ -237,25 +251,19 @@ int is_64signalregset(void)
 vm_map_t
 swap_task_map(task_t task,vm_map_t map)
 {
-	thread_act_t act = current_act();
+	thread_t thread = current_thread();
 	vm_map_t old_map;
 
-	if (task != act->task)
+	if (task != thread->task)
 		panic("swap_task_map");
 
 	task_lock(task);
 	old_map = task->map;
-	act->map = task->map = map;
+	thread->map = task->map = map;
 	task_unlock(task);
 	return old_map;
 }
 
-vm_map_t
-swap_act_map(thread_act_t thr_act,vm_map_t map)
-{
-	panic("swap_act_map");
-}
-
 /*
  *
  */
@@ -274,7 +282,7 @@ pmap_t  get_map_pmap(vm_map_t map)
 /*
  *
  */
-task_t	get_threadtask(thread_act_t th)
+task_t	get_threadtask(thread_t th)
 {
 	return(th->task);
 }
@@ -319,7 +327,7 @@ getact_thread(
 /*
  *
  */
-vm_offset_t
+vm_map_offset_t
 get_map_min(
 	vm_map_t	map)
 {
@@ -329,13 +337,13 @@ get_map_min(
 /*
  *
  */
-vm_offset_t
+vm_map_offset_t
 get_map_max(
 	vm_map_t	map)
 {
 	return(vm_map_max(map));
 }
-vm_size_t
+vm_map_size_t
 get_vmmap_size(
 	vm_map_t	map)
 {
@@ -421,7 +429,7 @@ get_task_userstop(
  */
 int
 get_thread_userstop(
-	thread_act_t th)
+	thread_t th)
 {
 	return(th->user_stop_count);
 }
@@ -433,8 +441,7 @@ boolean_t
 thread_should_abort(
 	thread_t th)
 {
-	return(!th->top_act || 
-	       (th->state & (TH_ABORT|TH_ABORT_SAFELY)) == TH_ABORT);
+	return ((th->state & (TH_ABORT|TH_ABORT_SAFELY)) == TH_ABORT);
 }
 
 /*
@@ -452,9 +459,8 @@ current_thread_aborted (
 	thread_t th = current_thread();
 	spl_t s;
 
-	if (!th->top_act || 
-		((th->state & (TH_ABORT|TH_ABORT_SAFELY)) == TH_ABORT &&
-		 th->interrupt_level != THREAD_UNINT))
+	if ((th->state & (TH_ABORT|TH_ABORT_SAFELY)) == TH_ABORT &&
+			(th->options & TH_OPT_INTMASK) != THREAD_UNINT)
 		return (TRUE);
 	if (th->state & TH_ABORT_SAFELY) {
 		s = splsched();
@@ -472,19 +478,20 @@ current_thread_aborted (
  */
 void
 task_act_iterate_wth_args(
-	task_t task,
-	void (*func_callback)(thread_act_t, void *),
-	void *func_arg)
+	task_t			task,
+	void			(*func_callback)(thread_t, void *),
+	void			*func_arg)
 {
-        thread_act_t inc, ninc;
+	thread_t	inc;
 
 	task_lock(task);
-        for (inc  = (thread_act_t)queue_first(&task->threads);
-			 !queue_end(&task->threads, (queue_entry_t)inc);
-             inc  = ninc) {
-                ninc = (thread_act_t)queue_next(&inc->task_threads);
-                (void) (*func_callback)(inc, func_arg);
-        }
+
+	for (inc  = (thread_t)queue_first(&task->threads);
+			!queue_end(&task->threads, (queue_entry_t)inc); ) {
+		(void) (*func_callback)(inc, func_arg);
+		inc = (thread_t)queue_next(&inc->task_threads);
+	}
+
 	task_unlock(task);
 }
 
@@ -502,13 +509,6 @@ is_thread_active(
 	return(th->active);
 }
 
-kern_return_t
-get_thread_waitresult(
-	thread_t th)
-{
-	return(th->wait_result);
-}
-
 void
 astbsd_on(void)
 {
