@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -23,11 +24,18 @@
  * @OSF_COPYRIGHT@
  */
 
-#ifndef	_IPC_MIG_H_
-#define	_IPC_MIG_H_
+#ifndef	_KERN_IPC_MIG_H_
+#define	_KERN_IPC_MIG_H_
 
 #include <mach/mig.h>
+#include <mach/mach_types.h>
 #include <mach/message.h>
+#include <kern/kern_types.h>
+
+#include <sys/cdefs.h>
+
+#ifdef	XNU_KERNEL_PRIVATE
+
 #include <sys/kdebug.h>
 
 /*
@@ -113,6 +121,10 @@
 			      (unsigned int)(0),				      \
 			      (unsigned int)(0))
 
+#endif	/* XNU_KERNEL_PRIVATE */
+
+__BEGIN_DECLS
+
 /* Send a message from the kernel */
 extern mach_msg_return_t mach_msg_send_from_kernel(
 	mach_msg_header_t	*msg,
@@ -124,11 +136,15 @@ extern mach_msg_return_t mach_msg_rpc_from_kernel(
 	mach_msg_size_t		send_size,
 	mach_msg_size_t		rcv_size);
 
+__END_DECLS
+
+#ifdef	MACH_KERNEL_PRIVATE
+
 extern void mach_msg_receive_continue(void);
 
-#include <sys/appleapiopts.h>
+/* Initialize kernel server dispatch table */
+extern void		mig_init(void);
 
-#ifdef __APPLE_API_EVOLVING
 /*
  * Kernel implementation of the MIG object base class
  *
@@ -138,7 +154,7 @@ extern void mach_msg_receive_continue(void);
  */
 
 typedef struct mig_object {
-			IMIGObjectVtbl		*pVtbl; /* our interface def */
+			const IMIGObjectVtbl	*pVtbl; /* our interface def */
 			mach_port_t		port;	 /* our port pointer  */
 } mig_object_data_t;
 
@@ -150,10 +166,34 @@ typedef struct mig_object {
  * chain and deliver the appropriate notification.
  */
 typedef struct mig_notify_object {
-			IMIGNotifyObjectVtbl	*pVtbl; /* our interface def */
+			const IMIGNotifyObjectVtbl *pVtbl; /* our interface def */
 			mach_port_t		port;	 /* our port pointer  */
 } mig_notify_object_data_t;
 
-#endif  /* __APPLE_API_EVOLVING */
+extern kern_return_t mig_object_init(
+			mig_object_t		mig_object,
+			const IMIGObject	*interface);
 
-#endif	/* _IPC_MIG_H_ */
+extern void mig_object_destroy(
+			mig_object_t		mig_object);
+
+extern void mig_object_reference(
+			mig_object_t		mig_object);
+
+extern void mig_object_deallocate(
+			mig_object_t		mig_object);
+
+extern ipc_port_t convert_mig_object_to_port(
+			mig_object_t		mig_object);
+
+extern mig_object_t convert_port_to_mig_object(
+			ipc_port_t		port,
+			const MIGIID		*iid);
+
+boolean_t mig_object_no_senders(
+			ipc_port_t		port,
+			mach_port_mscount_t	mscount);
+
+#endif  /* MACH_KERNEL_PRIVATE */
+
+#endif	/* _KERN_IPC_MIG_H_ */
