@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -48,43 +49,53 @@
  * the rights to redistribute these changes.
  */
 
+#ifdef	KERNEL_PRIVATE
+
 #ifndef	_KERN_KALLOC_H_
 #define _KERN_KALLOC_H_
 
 #include <mach/machine/vm_types.h>
+#include <sys/cdefs.h>
+
+__BEGIN_DECLS
+
+extern void *kalloc(vm_size_t	size);
+
+extern void *kalloc_noblock(vm_size_t	size);
+
+extern void *kget(vm_size_t	size);
+
+extern void kfree(void		*data,
+		  vm_size_t	size);
+
+__END_DECLS
+
+#ifdef	MACH_KERNEL_PRIVATE
+
+#include <kern/lock.h>
 
 #define KALLOC_MINSIZE		16
 
-extern vm_offset_t	kalloc(
-						vm_size_t	size);
-
-extern vm_offset_t	kalloc_noblock(
-				       vm_size_t	size);
-
-extern vm_offset_t	kget(
-						vm_size_t	size);
-
-extern void			kfree(
-						vm_offset_t	data,
-						vm_size_t	size);
-
-#include <sys/appleapiopts.h>
-
-#ifdef	__APPLE_API_PRIVATE
-
-#ifdef MACH_KERNEL_PRIVATE
-#include <kern/lock.h>
-
 extern void		kalloc_init(void);
 
-extern void		krealloc(
-					vm_offset_t	*addrp,
-					vm_size_t	old_size,
-					vm_size_t	new_size,
-					simple_lock_t	lock);
+extern void		krealloc(void		**addrp,
+				 vm_size_t	old_size,
+				 vm_size_t	new_size,
+				 simple_lock_t	lock);
 
-#endif /* MACH_KERNEL_PRIVATE */
+extern void		kalloc_fake_zone_info(
+				int		*count,
+				vm_size_t	*cur_size,
+				vm_size_t	*max_size,
+				vm_size_t	*elem_size,
+				vm_size_t	*alloc_size,
+				int		*collectable,
+				int		*exhaustable);
 
-#endif	/* __APPLE_APPI_PRIVATE */
+extern vm_size_t kalloc_max_prerounded;
+
+#endif	/* MACH_KERNEL_PRIVATE */
 
 #endif	/* _KERN_KALLOC_H_ */
+
+#endif	/* KERNEL_PRIVATE */
