@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -29,6 +30,7 @@
  */
 
 #include <mach/mach_types.h>
+#include <mach/mach_traps.h>
 #include <mach/mach_port_server.h>
 
 #include <mach/mk_timer.h>
@@ -49,7 +51,8 @@ static void	mk_timer_expire(
 				void			*p1);
 
 mach_port_name_t
-mk_timer_create(void)
+mk_timer_create_trap(
+	__unused struct mk_timer_create_trap_args *args)
 {
 	mk_timer_t			timer;
 	ipc_space_t			myspace = current_space();
@@ -67,12 +70,12 @@ mk_timer_create(void)
 		result = ipc_port_translate_receive(myspace, name, &port);
 
 	if (result != KERN_SUCCESS) {
-		zfree(mk_timer_zone, (vm_offset_t)timer);
+		zfree(mk_timer_zone, timer);
 
 		return (MACH_PORT_NULL);
 	}
 
-	simple_lock_init(&timer->lock, ETAP_MISC_TIMER);
+	simple_lock_init(&timer->lock, 0);
 	call_entry_setup(&timer->call_entry, mk_timer_expire, timer);
 	timer->is_armed = timer->is_dead = FALSE;
 	timer->active = 0;
@@ -111,7 +114,7 @@ mk_timer_port_destroy(
 		timer->is_dead = TRUE;
 		if (timer->active == 0) {
 			simple_unlock(&timer->lock);
-			zfree(mk_timer_zone, (vm_offset_t)timer);
+			zfree(mk_timer_zone, timer);
 
 			ipc_port_release_send(port);
 			return;
@@ -134,7 +137,7 @@ mk_timer_init(void)
 static void
 mk_timer_expire(
 	void			*p0,
-	void			*p1)
+	__unused void		*p1)
 {
 	mk_timer_t			timer = p0;
 	ipc_port_t			port;
@@ -171,7 +174,7 @@ mk_timer_expire(
 
 	if (--timer->active == 0 && timer->is_dead) {
 		simple_unlock(&timer->lock);
-		zfree(mk_timer_zone, (vm_offset_t)timer);
+		zfree(mk_timer_zone, timer);
 
 		ipc_port_release_send(port);
 		return;
@@ -181,9 +184,10 @@ mk_timer_expire(
 }
 
 kern_return_t
-mk_timer_destroy(
-	mach_port_name_t	name)
+mk_timer_destroy_trap(
+	struct mk_timer_destroy_trap_args *args)
 {
+	mach_port_name_t	name = args->name;
 	ipc_space_t			myspace = current_space();
 	ipc_port_t			port;
 	kern_return_t		result;
@@ -205,18 +209,16 @@ mk_timer_destroy(
 }
 
 kern_return_t
-mk_timer_arm(
-	mach_port_name_t	name,
-	uint64_t			expire_time)
+mk_timer_arm_trap(
+	struct mk_timer_arm_trap_args *args)
 {
-	uint64_t			time_of_arming;
+	mach_port_name_t	name = args->name;
+	uint64_t			expire_time = args->expire_time;
 	mk_timer_t			timer;
 	ipc_space_t			myspace = current_space();
 	ipc_port_t			port;
 	kern_return_t		result;
 
-	clock_get_uptime(&time_of_arming);
-
 	result = ipc_port_translate_receive(myspace, name, &port);
 	if (result != KERN_SUCCESS)
 		return (result);
@@ -229,7 +231,6 @@ mk_timer_arm(
 		ip_unlock(port);
 
 		if (!timer->is_dead) {
-			timer->time_of_arming = time_of_arming;
 			timer->is_armed = TRUE;
 
 			if (!thread_call_enter_delayed(&timer->call_entry, expire_time))
@@ -247,10 +248,11 @@ mk_timer_arm(
 }
 
 kern_return_t
-mk_timer_cancel(
-	mach_port_name_t	name,
-	uint64_t			*result_time)
+mk_timer_cancel_trap(
+	struct mk_timer_cancel_trap_args *args)
 {
+	mach_port_name_t	name = args->name;
+	mach_vm_address_t	result_time_addr = args->result_time; 
 	uint64_t			armed_time = 0;
 	mk_timer_t			timer;
 	ipc_space_t			myspace = current_space();
@@ -283,8 +285,8 @@ mk_timer_cancel(
 	}
 
 	if (result == KERN_SUCCESS)
-		if (	result_time	!= NULL										&&
-				copyout((void *)&armed_time, (void *)result_time,
+		if (	result_time_addr != 0										&&
+				copyout((void *)&armed_time, result_time_addr,
 								sizeof (armed_time)) != 0					)
 			result = KERN_FAILURE;
 
