@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -56,16 +54,22 @@
 #ifndef	_KERN_SYSCALL_SUBR_H_
 #define _KERN_SYSCALL_SUBR_H_
 
-#include <mach/boolean.h>
-#include <mach/kern_return.h>
-#include <kern/kern_types.h>
-#include <mach/port.h>
+#include <mach/mach_traps.h>
 
-/* Attempt to context switch */
-extern boolean_t	swtch(void);
+extern void		thread_depress_abstime(
+					uint64_t		interval);
 
-/* Attempt to context switch */
-extern boolean_t	swtch_pri(
-					int				pri);
+extern void		thread_depress_ms(
+					mach_msg_timeout_t	interval);
+
+extern kern_return_t	thread_depress_abort_internal(
+							thread_t				thread);
+
+extern void		thread_depress_expire(
+					void			*thread,
+					void			*p1);
+
+extern void		thread_poll_yield(
+					thread_t		self);
 
 #endif	/* _KERN_SYSCALL_SUBR_H_ */
