@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -60,15 +61,21 @@
  *	its kernel stack.  Some architectures may need
  *	to save more state in the pcb for these traps.
  */
+typedef	void	mach_munge_t(const void *, void *);
 
 typedef struct {
 	int			mach_trap_arg_count;
 	int			(*mach_trap_function)(void);
-	boolean_t	mach_trap_stack;
+#if defined(__i386__)
+	boolean_t		mach_trap_stack;
+#else 
+	mach_munge_t		*mach_trap_arg_munge32; /* system call arguments for 32-bit */
+	mach_munge_t		*mach_trap_arg_munge64; /* system call arguments for 64-bit */
+#endif
 #if	!MACH_ASSERT
 	int			mach_trap_unused;
 #else
-	char*		mach_trap_name;
+	const char*		mach_trap_name;
 #endif /* !MACH_ASSERT */
 } mach_trap_t;
 
@@ -76,15 +83,25 @@ typedef struct {
 
 
 extern mach_trap_t		mach_trap_table[];
-extern int				mach_trap_count;
-extern kern_return_t	kern_invalid(void);
+extern int			mach_trap_count;
 
+#if defined(__i386__)
 #if	!MACH_ASSERT
-#define	MACH_TRAP(name, arg_count)			\
+#define	MACH_TRAP(name, arg_count, munge32, munge64)	\
 		{ (arg_count), (int (*)(void)) (name), FALSE, 0 }
 #else
-#define MACH_TRAP(name, arg_count)			\
+#define MACH_TRAP(name, arg_count, munge32, munge64)		\
 		{ (arg_count), (int (*)(void)) (name), FALSE, #name }
 #endif /* !MACH_ASSERT */
+#else  /* !defined(__i386__) */
+#if	!MACH_ASSERT
+#define	MACH_TRAP(name, arg_count, munge32, munge64)	\
+		{ (arg_count), (int (*)(void)) (name), (munge32), (munge64), 0 }
+#else
+#define MACH_TRAP(name, arg_count, munge32, munge64)		\
+  		{ (arg_count), (int (*)(void)) (name), (munge32), (munge64), #name }
+#endif /* !MACH_ASSERT */
+
+#endif /* !defined(__i386__) */
 
 #endif	/* _KERN_SYSCALL_SW_H_ */
