@@ -1,22 +1,23 @@
 /*
- * Copyright (c) 1993-1995, 1999-2000 Apple Computer, Inc.
+ * Copyright (c) 1993-1995, 1999-2004 Apple Computer, Inc.
  * All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -37,21 +38,8 @@
 #include <kern/timer_call.h>
 #include <kern/call_entry.h>
 
-#ifdef i386
-/*
- * Until we arrange for per-cpu timers, use the master cpus queues only.
- * Fortunately, the timer_call_lock synchronizes access to all queues.
- */
-#undef	cpu_number()
-#define cpu_number() 0
-#endif /* i386 */
-  
 decl_simple_lock_data(static,timer_call_lock)
 
-static
-queue_head_t
-	timer_call_queues[NCPUS];
-
 static struct {
 	int		delayed_num,
 			delayed_hiwat;
@@ -68,16 +56,12 @@ void
 timer_call_initialize(void)
 {
 	spl_t				s;
-	int					i;
 
-	simple_lock_init(&timer_call_lock, ETAP_MISC_TIMER);
+	simple_lock_init(&timer_call_lock, 0);
 
 	s = splclock();
 	simple_lock(&timer_call_lock);
 
-	for (i = 0; i < NCPUS; i++)
-		queue_init(&timer_call_queues[i]);
-
 	clock_set_timer_func((clock_timer_func_t)timer_call_interrupt);
 
 	simple_unlock(&timer_call_lock);
@@ -159,7 +143,7 @@ timer_call_enter(
 	call->param1	= 0;
 	call->deadline	= deadline;
 
-	queue = &timer_call_queues[cpu_number()];
+	queue = &PROCESSOR_DATA(current_processor(), timer_call_queue);
 
 	_delayed_call_enqueue(queue, call);
 
@@ -193,7 +177,7 @@ timer_call_enter1(
 	call->param1	= param1;
 	call->deadline	= deadline;
 
-	queue = &timer_call_queues[cpu_number()];
+	queue = &PROCESSOR_DATA(current_processor(), timer_call_queue);
 
 	_delayed_call_enqueue(queue, call);
 
@@ -263,8 +247,8 @@ timer_call_shutdown(
 
 	assert(processor != current_processor());
 
-	queue = &timer_call_queues[processor->slot_num];
-	myqueue = &timer_call_queues[cpu_number()];
+	queue = &PROCESSOR_DATA(processor, timer_call_queue);
+	myqueue = &PROCESSOR_DATA(current_processor(), timer_call_queue);
 
 	simple_lock(&timer_call_lock);
 
@@ -292,10 +276,12 @@ timer_call_interrupt(
 	uint64_t				timestamp)
 {
 	timer_call_t		call;
-	queue_t				queue = &timer_call_queues[cpu_number()];
+	queue_t				queue;
 
 	simple_lock(&timer_call_lock);
 
+	queue = &PROCESSOR_DATA(current_processor(), timer_call_queue);
+
 	call = TC(queue_first(queue));
 
 	while (!queue_end(queue, qe(call))) {
