@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -30,15 +31,14 @@
  */
 
 /*
- * N.B. This interface has been deprecated and the contents
- * of this file should be considered obsolete.
+ *	All interfaces defined here are obsolete.
  */
 
 #ifndef	_MACH_CLOCK_TYPES_H_
 #define	_MACH_CLOCK_TYPES_H_
 
+#include <stdint.h>
 #include <mach/time_value.h>
-#include <sys/appleapiopts.h>
 
 /*
  * Type definitions.
@@ -59,19 +59,13 @@ struct mach_timespec {
 };
 typedef struct mach_timespec	mach_timespec_t;
 
-#ifdef	__APPLE_API_UNSTABLE
-
 /*
  * Reserved clock id values for default clocks.
  */
-#define SYSTEM_CLOCK		0	/* advances monotonically and
-								 * uniformly; set to zero at boot */
-#define CALENDAR_CLOCK		1	/* 'wall' clock; effectively
-								 * synchronized to UTC */
+#define SYSTEM_CLOCK		0
+#define CALENDAR_CLOCK		1
 
-#define REALTIME_CLOCK		0	/* obsolete; use SYSTEM or CALENDAR
-								 * clock depending on particular
-								 * requirements */
+#define REALTIME_CLOCK		0
 
 /*
  * Attribute names.
@@ -124,6 +118,4 @@ typedef struct mach_timespec	mach_timespec_t;
 
 #define BAD_ALRMTYPE(t)		(((t) &~ TIME_RELATIVE) != 0)
 
-#endif	/* __APPLE_API_UNSTABLE */
-
 #endif /* _MACH_CLOCK_TYPES_H_ */
