@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -22,11 +20,11 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+
 #ifndef _MACH_MACHINE_RPC_H_
 #define _MACH_MACHINE_RPC_H_
 
-
-#if defined (__ppc__)
+#if defined (__ppc__) || defined (__ppc64__)
 #include "mach/ppc/rpc.h"
 #elif defined (__i386__)
 #include "mach/i386/rpc.h"
@@ -34,5 +32,4 @@
 #error architecture not supported
 #endif
 
-
-#endif /* _MACH_MACHINE_RPC_H_ */
+#endif	/* _MACH_MACHINE_RPC_H_ */
