@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -22,11 +20,13 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
+
+#ifdef	PRIVATE
+
 #ifndef _MACH_MACHINE_SYSCALL_SW_H_
 #define _MACH_MACHINE_SYSCALL_SW_H_
 
-
-#if defined (__ppc__)
+#if defined (__ppc__) || defined (__ppc64__)
 #include "mach/ppc/syscall_sw.h"
 #elif defined (__i386__)
 #include "mach/i386/syscall_sw.h"
@@ -34,5 +34,6 @@
 #error architecture not supported
 #endif
 
+#endif	/* _MACH_MACHINE_SYSCALL_SW_H_ */
 
-#endif /* _MACH_MACHINE_SYSCALL_SW_H_ */
+#endif	/* PRIVATE */
