@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -62,6 +63,8 @@
 #include <mach/message.h>
 #include <mach/kern_return.h>
 
+#include <sys/cdefs.h>
+
 /*
  *	These error codes should be specified as system 4, subsytem 2.
  *	But alas backwards compatibility makes that impossible.
@@ -87,11 +90,16 @@
  *	mig_reply_error_t format message.  Clients must accept
  *	these in addition to the expected reply message format.
  */
+#pragma pack(4)
 typedef struct {
 	mach_msg_header_t	Head;
 	NDR_record_t		NDR;
 	kern_return_t		RetCode;
 } mig_reply_error_t;
+#pragma pack()
+
+
+__BEGIN_DECLS
 
 #define __NDR_convert__mig_reply_error_t__defined
 #if mig_internal
@@ -108,5 +116,6 @@ __NDR_convert__mig_reply_error_t(mig_reply_error_t *x)
 #endif /* __NDR_convert__int_rep__kern_return_t__defined */
 }
 
-#endif	/* _MACH_MIG_ERRORS_H_ */
+__END_DECLS
 
+#endif	/* _MACH_MIG_ERRORS_H_ */
