@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -43,10 +41,6 @@
 #include <mach/machine/rpc.h>
 #include <mach/thread_status.h>
 
-#include <sys/appleapiopts.h>
-
-#ifdef __APPLE_API_OBSOLETE
-
 /*
  * These are the types for RPC-specific variants of the MIG routine
  * descriptor and subsystem data types.
@@ -133,6 +127,4 @@ typedef struct rpc_subsystem  *rpc_subsystem_t;
 
 #define RPC_SUBSYSTEM_NULL	((rpc_subsystem_t) 0)
 
-#endif  /* __APPLE_API_OBSOLETE */
-
 #endif	/* _MACH_RPC_H_ */
