@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -51,10 +49,8 @@
  * the rights to redistribute these changes.
  */
 
-#ifndef	TIME_VALUE_H_
-#define	TIME_VALUE_H_
-
-#include <sys/appleapiopts.h>
+#ifndef	_MACH_TIME_VALUE_H_
+#define	_MACH_TIME_VALUE_H_
 
 #include <mach/machine/vm_types.h>
 
@@ -66,9 +62,8 @@ struct time_value {
 	integer_t seconds;
 	integer_t microseconds;
 };
-typedef	struct time_value	time_value_t;
 
-#ifdef	__APPLE_API_UNSTABLE
+typedef	struct time_value	time_value_t;
 
 /*
  *	Macros to manipulate time values.  Assume that time values
@@ -93,25 +88,4 @@ typedef	struct time_value	time_value_t;
 	}							\
 }
 
-#endif	/* __APPLE_API_UNSTABLE */
-
-#ifdef	__APPLE_API_OBSOLETE
-
-/*
- *	Time value available through the mapped-time interface.
- *	Read this mapped value with
- *		do {
- *			secs = mtime->seconds;
- *			usecs = mtime->microseconds;
- *		} while (secs != mtime->check_seconds);
- */
-
-typedef struct mapped_time_value {
-	integer_t seconds;
-	integer_t microseconds;
-	integer_t check_seconds;
-} mapped_time_value_t;
-
-#endif	/* __APPLE_API_OBSOLETE */
-
-#endif	/* TIME_VALUE_H_ */
+#endif	/* _MACH_TIME_VALUE_H_ */
