@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -25,62 +23,6 @@
 /*
  * @OSF_COPYRIGHT@
  */
-/*
- * HISTORY
- * 
- * Revision 1.1.1.1  1998/09/22 21:05:45  wsanchez
- * Import of Mac OS X kernel (~semeria)
- *
- * Revision 1.1.1.1  1998/03/07 02:26:17  wsanchez
- * Import of OSF Mach kernel (~mburg)
- *
- * Revision 1.2.12.2  1995/01/06  19:52:51  devrcs
- * 	mk6 CR668 - 1.3b26 merge
- * 	64bit cleanup
- * 	[1994/10/14  03:43:40  dwm]
- *
- * Revision 1.2.12.1  1994/09/23  02:46:19  ezf
- * 	change marker to not FREE
- * 	[1994/09/22  21:44:33  ezf]
- * 
- * Revision 1.2.2.2  1993/06/09  02:45:03  gm
- * 	Added to OSF/1 R1.3 from NMK15.0.
- * 	[1993/06/02  21:19:19  jeffc]
- * 
- * Revision 1.2  1993/04/19  16:41:52  devrcs
- * 	ansi C conformance changes
- * 	[1993/02/02  18:57:07  david]
- * 
- * Revision 1.1  1992/09/30  02:32:41  robert
- * 	Initial revision
- * 
- * $EndLog$
- */
-/* CMU_HIST */
-/*
- * Revision 2.5  91/05/14  17:04:15  mrt
- * 	Correcting copyright
- * 
- * Revision 2.4  91/02/05  17:38:17  mrt
- * 	Changed to new Mach copyright
- * 	[91/02/01  17:29:40  mrt]
- * 
- * Revision 2.3  90/06/19  23:00:29  rpd
- * 	Added zi_ prefix to zone_info field names.
- * 	Added zi_collectable field to zone_info.
- * 	Added zn_ prefix to zone_name field names.
- * 	[90/06/05            rpd]
- * 
- * Revision 2.2  90/06/02  15:00:54  rpd
- * 	Created.
- * 	[90/03/26  23:53:57  rpd]
- * 
- * Revision 2.2  89/05/06  12:36:08  rpd
- * 	Created.
- * 	[89/05/06  12:35:19  rpd]
- * 
- */ 
-/* CMU_ENDHIST */
 /* 
  * Mach Operating System
  * Copyright (c) 1991,1990,1989 Carnegie Mellon University
@@ -131,10 +73,10 @@ typedef zone_name_t *zone_name_array_t;
 
 typedef struct zone_info {
 	integer_t	zi_count;	/* Number of elements used now */
-	vm_size_t	zi_cur_size;	/* current memory utilization */
-	vm_size_t	zi_max_size;	/* how large can this zone grow */
-	vm_size_t	zi_elem_size;	/* size of an element */
-	vm_size_t	zi_alloc_size;	/* size used for more memory */
+	natural_t	zi_cur_size;	/* current memory utilization */
+	natural_t	zi_max_size;	/* how large can this zone grow */
+	natural_t	zi_elem_size;	/* size of an element */
+	natural_t	zi_alloc_size;	/* size used for more memory */
 	integer_t	zi_pageable;	/* zone pageable? */
 	integer_t	zi_sleepable;	/* sleep if empty? */
 	integer_t	zi_exhaustible;	/* merely return if empty? */
