@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -31,19 +32,26 @@
 /* Special check bits for the compage_descriptor "special" field. */
  
 #define	kCommPageDCBA		0x0001			// this routine uses DCBA, map to NOP if not appropriate
-#define	kCommPageSYNC		0x0002			// this routine uses SYNC, map to NOP if UP
-#define	kCommPageMTCRF		0x0004			// set bit 11 in MTCRF if only 1 cr specified
+#define	kCommPageSYNC		0x0002			// this routine uses SYNC, LWSYNC, or EIEIO, map to NOP if UP
+#define kCommPageISYNC		0x0004			// this routine uses ISYNC, map to NOP if UP
+#define	kCommPageMTCRF		0x0008			// set bit 11 in MTCRF if only 1 cr specified
+
+#define kPort32to64			0x1000			// written for 32-bit, must port to 64-bit
+#define kCommPage64			0x2000			// this routine is useable in 64-bit mode
+#define kCommPage32			0x4000			// this routine is useable in 32-bit mode
+#define kCommPageBoth		(kCommPage32+kCommPage64)
 
 
 #ifdef	__ASSEMBLER__
 
 #define	COMMPAGE_DESCRIPTOR(label,address,must,cant,special)	\
+    .globl  EXT(label)  @\
 LEXT(label)	@\
     .short	label-.	@\
     .short	.-label-2	@\
     .short	address	@\
     .short	special	@\
-    .long	must	@\
+    .long	must    @\
     .long	cant
     
 
@@ -64,7 +72,8 @@ typedef	struct	commpage_descriptor	{
 } commpage_descriptor;
 
 
-extern	char	*commPagePtr;				// virt address of commpage in kernel map
+extern	char	*commPagePtr32;				// virt address of 32-bit commpage in kernel map
+extern	char	*commPagePtr64;				// virt address of 64-bit commpage in kernel map
 
 
 extern	void	commpage_set_timestamp(uint64_t tbr,uint32_t secs,uint32_t usecs,uint32_t ticks_per_sec);
