@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2004-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -40,13 +41,14 @@
 
 #include <kern/host_notify.h>
 
-#include <machine/mach_param.h>	/* HZ */
 #include <machine/commpage.h>
 #include <machine/machine_routines.h>
 #include <ppc/exception.h>
 #include <ppc/proc_reg.h>
+#include <ppc/pms.h>
+#include <ppc/rtclock.h>
 
-#include <pexpert/pexpert.h>
+#include <IOKit/IOPlatformExpert.h>
 
 #include <sys/kdebug.h>
 
@@ -54,8 +56,6 @@ int		sysclk_config(void);
 
 int		sysclk_init(void);
 
-void treqs(uint32_t dec);
-
 kern_return_t	sysclk_gettime(
 	mach_timespec_t			*cur_time);
 
@@ -76,8 +76,6 @@ struct clock_ops sysclk_ops = {
 
 int		calend_config(void);
 
-int		calend_init(void);
-
 kern_return_t	calend_gettime(
 	mach_timespec_t			*cur_time);
 
@@ -87,7 +85,7 @@ kern_return_t	calend_getattr(
 	mach_msg_type_number_t	*count);
 
 struct clock_ops calend_ops = {
-	calend_config,			calend_init,
+	calend_config,			0,
 	calend_gettime,			0,
 	calend_getattr,			0,
 	0,
@@ -105,11 +103,27 @@ static struct rtclock_calend {
 	int32_t				adjdelta;
 }					rtclock_calend;
 
-static boolean_t		rtclock_initialized;
+static uint32_t		rtclock_boottime;
 
-static uint64_t			rtclock_tick_deadline[NCPUS];
+#define TIME_ADD(rsecs, secs, rfrac, frac, unit)	\
+MACRO_BEGIN											\
+	if (((rfrac) += (frac)) >= (unit)) {			\
+		(rfrac) -= (unit);							\
+		(rsecs) += 1;								\
+	}												\
+	(rsecs) += (secs);								\
+MACRO_END
 
-#define NSEC_PER_HZ		(NSEC_PER_SEC / HZ)
+#define TIME_SUB(rsecs, secs, rfrac, frac, unit)	\
+MACRO_BEGIN											\
+	if ((int32_t)((rfrac) -= (frac)) < 0) {			\
+		(rfrac) += (unit);							\
+		(rsecs) -= 1;								\
+	}												\
+	(rsecs) -= (secs);								\
+MACRO_END
+
+#define NSEC_PER_HZ		(NSEC_PER_SEC / 100)
 static uint32_t		 	rtclock_tick_interval;
 
 static uint32_t			rtclock_sec_divisor;
@@ -118,25 +132,14 @@ static mach_timebase_info_data_t	rtclock_timebase_const;
 
 static boolean_t		rtclock_timebase_initialized;
 
-static struct rtclock_timer {
-	uint64_t			deadline;
-	uint32_t
-	/*boolean_t*/		is_set:1,
-						has_expired:1,
-						:0;
-}					rtclock_timer[NCPUS];
-
 static clock_timer_func_t	rtclock_timer_expire;
 
 static timer_call_data_t	rtclock_alarm_timer;
 
-static void		timespec_to_absolutetime(
-					mach_timespec_t		*ts,
-					uint64_t			*result);
-
-static int		deadline_to_decrementer(
-					uint64_t		deadline,
-					uint64_t		now);
+static void		nanotime_to_absolutetime(
+					uint32_t		secs,
+					uint32_t		nanosecs,
+					uint64_t		*result);
 
 static void		rtclock_alarm_expire(
 					timer_call_param_t		p0,
@@ -144,11 +147,6 @@ static void		rtclock_alarm_expire(
 
 /* global data declarations */
 
-#define DECREMENTER_MAX		0x7FFFFFFFUL
-#define DECREMENTER_MIN		0xAUL
-
-natural_t		rtclock_decrementer_min;
-
 decl_simple_lock_data(static,rtclock_lock)
 
 /*
@@ -197,7 +195,7 @@ timebase_callback(
 	}
 	else {
 		UNLOCK_RTC(s);
-		printf("rtclock timebase_callback: late old %d / %d new %d / %d",
+		printf("rtclock timebase_callback: late old %d / %d new %d / %d\n",
 					rtclock_timebase_const.numer, rtclock_timebase_const.denom,
 							numer, denom);
 		return;
@@ -213,12 +211,9 @@ timebase_callback(
 int
 sysclk_config(void)
 {
-	if (cpu_number() != master_cpu)
-		return(1);
-
 	timer_call_setup(&rtclock_alarm_timer, rtclock_alarm_expire, NULL);
 
-	simple_lock_init(&rtclock_lock, ETAP_MISC_RT_CLOCK);
+	simple_lock_init(&rtclock_lock, 0);
 
 	PE_register_timebase_callback(timebase_callback);
 
@@ -231,31 +226,16 @@ sysclk_config(void)
 int
 sysclk_init(void)
 {
-	uint64_t		abstime;
-	int				decr, mycpu = cpu_number();
+	uint64_t				abstime;
+	struct per_proc_info	*pp;
 
-	if (mycpu != master_cpu) {
-		if (rtclock_initialized == FALSE) {
-			panic("sysclk_init on cpu %d, rtc not initialized\n", mycpu);
-		}
-		/* Set decrementer and hence our next tick due */
-		abstime = mach_absolute_time();
-		rtclock_tick_deadline[mycpu] = abstime;
-		rtclock_tick_deadline[mycpu] += rtclock_tick_interval;
-		decr = deadline_to_decrementer(rtclock_tick_deadline[mycpu], abstime);
-		treqs(decr);
+	pp = getPerProc();
 
-		return(1);
-	}
-
-	/* Set decrementer and our next tick due */
 	abstime = mach_absolute_time();
-	rtclock_tick_deadline[mycpu] = abstime;
-	rtclock_tick_deadline[mycpu] += rtclock_tick_interval;
-	decr = deadline_to_decrementer(rtclock_tick_deadline[mycpu], abstime);
-	treqs(decr);
-
-	rtclock_initialized = TRUE;
+	pp->rtclock_tick_deadline = abstime + rtclock_tick_interval;	/* Get the time we need to pop */
+	pp->rtcPop = pp->rtclock_tick_deadline;	/* Set the rtc pop time the same for now */
+	
+	(void)setTimerReq();			/* Start the timers going */
 
 	return (1);
 }
@@ -348,7 +328,7 @@ sysclk_setalarm(
 {
 	uint64_t	abstime;
 
-	timespec_to_absolutetime(deadline, &abstime);
+	nanotime_to_absolutetime(deadline->tv_sec, deadline->tv_nsec, &abstime);
 	timer_call_enter(&rtclock_alarm_timer, abstime);
 }
 
@@ -361,18 +341,6 @@ calend_config(void)
 	return (1);
 }
 
-/*
- * Initialize the calendar clock.
- */
-int
-calend_init(void)
-{
-	if (cpu_number() != master_cpu)
-		return(1);
-
-	return (1);
-}
-
 /*
  * Get the current clock time.
  */
@@ -446,12 +414,7 @@ clock_get_calendar_microtime(
 		now -= (t64 * divisor);
 		*microsecs = (now * USEC_PER_SEC) / divisor;
 
-		if ((*microsecs += microepoch) >= USEC_PER_SEC) {
-			*microsecs -= USEC_PER_SEC;
-			epoch += 1;
-		}
-
-		*secs += epoch;
+		TIME_ADD(*secs, epoch, *microsecs, microepoch, USEC_PER_SEC);
 	}
 	else {
 		uint32_t	delta, t32;
@@ -469,12 +432,7 @@ clock_get_calendar_microtime(
 			t32 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
 
 			if (t32 > delta)
-				*microsecs += (t32 - delta);
-
-			if (*microsecs >= USEC_PER_SEC) {
-				*microsecs -= USEC_PER_SEC;
-				*secs += 1;
-			}
+				TIME_ADD(*secs, 0, *microsecs, (t32 - delta), USEC_PER_SEC);
 		}
 
 		simple_unlock(&rtclock_lock);
@@ -512,12 +470,7 @@ clock_gettimeofday(
 		t64 = now - (secs_64 * rtclock_sec_divisor);
 		microsecs = usec_64 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
 
-		if ((microsecs += microepoch) >= USEC_PER_SEC) {
-			microsecs -= USEC_PER_SEC;
-			epoch += 1;
-		}
-        
-		secs += epoch;
+		TIME_ADD(secs, epoch, microsecs, microepoch, USEC_PER_SEC);
         
         /* adjust "now" to be absolute time at _start_ of usecond */
         now -= t64 - ((usec_64 * rtclock_sec_divisor) / USEC_PER_SEC);
@@ -540,12 +493,7 @@ clock_gettimeofday(
 			t32 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
 
 			if (t32 > delta)
-				microsecs += (t32 - delta);
-
-			if (microsecs >= USEC_PER_SEC) {
-				microsecs -= USEC_PER_SEC;
-				secs += 1;
-			}
+				TIME_ADD(secs, 0, microsecs, (t32 - delta), USEC_PER_SEC);
 		}
 
         /* no need to disable timestamp, it is already off */
@@ -583,12 +531,7 @@ clock_get_calendar_nanotime(
 		now -= (t64 * divisor);
 		*nanosecs = ((now * USEC_PER_SEC) / divisor) * NSEC_PER_USEC;
 
-		if ((*nanosecs += nanoepoch) >= NSEC_PER_SEC) {
-			*nanosecs -= NSEC_PER_SEC;
-			epoch += 1;
-		}
-
-		*secs += epoch;
+		TIME_ADD(*secs, epoch, *nanosecs, nanoepoch, NSEC_PER_SEC);
 	}
 	else {
 		uint32_t	delta, t32;
@@ -606,12 +549,7 @@ clock_get_calendar_nanotime(
 			t32 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
 
 			if (t32 > delta)
-				*nanosecs += ((t32 - delta) * NSEC_PER_USEC);
-
-			if (*nanosecs >= NSEC_PER_SEC) {
-				*nanosecs -= NSEC_PER_SEC;
-				*secs += 1;
-			}
+				TIME_ADD(*secs, 0, *nanosecs, ((t32 - delta) * NSEC_PER_USEC), NSEC_PER_SEC);
 		}
 
 		simple_unlock(&rtclock_lock);
@@ -632,25 +570,78 @@ clock_set_calendar_microtime(
 	newsecs = (microsecs < 500*USEC_PER_SEC)?
 						secs: secs + 1;
 
-	LOCK_RTC(s);
+	s = splclock();
+	simple_lock(&rtclock_lock);
+
     commpage_set_timestamp(0,0,0,0);
 
-	clock_get_system_microtime(&sys, &microsys);
-	if ((int32_t)(microsecs -= microsys) < 0) {
-		microsecs += USEC_PER_SEC;
-		secs -= 1;
+	/*
+	 *	Cancel any adjustment in progress.
+	 */
+	if (rtclock_calend.adjdelta < 0) {
+		uint64_t	now, t64;
+		uint32_t	delta, t32;
+
+		delta = -rtclock_calend.adjdelta;
+
+		sys = rtclock_calend.epoch;
+		microsys = rtclock_calend.microepoch;
+
+		now = mach_absolute_time();
+
+		if (now > rtclock_calend.epoch1)
+			t64 = now - rtclock_calend.epoch1;
+		else
+			t64 = 0;
+
+		t32 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
+
+		if (t32 > delta)
+			TIME_ADD(sys, 0, microsys, (t32 - delta), USEC_PER_SEC);
+
+		rtclock_calend.epoch = sys;
+		rtclock_calend.microepoch = microsys;
+
+		sys = t64 = now / rtclock_sec_divisor;
+		now -= (t64 * rtclock_sec_divisor);
+		microsys = (now * USEC_PER_SEC) / rtclock_sec_divisor;
+
+		TIME_SUB(rtclock_calend.epoch, sys, rtclock_calend.microepoch, microsys, USEC_PER_SEC);
 	}
 
-	secs -= sys;
-
-	rtclock_calend.epoch = secs;
-	rtclock_calend.microepoch = microsecs;
 	rtclock_calend.epoch1 = 0;
 	rtclock_calend.adjdelta = rtclock_calend.adjtotal = 0;
-	UNLOCK_RTC(s);
 
+	/*
+	 *	Calculate the new calendar epoch based on
+	 *	the new value and the system clock.
+	 */
+	clock_get_system_microtime(&sys, &microsys);
+	TIME_SUB(secs, sys, microsecs, microsys, USEC_PER_SEC);
+
+	/*
+	 *	Adjust the boottime based on the delta.
+	 */
+	rtclock_boottime += secs - rtclock_calend.epoch;
+
+	/*
+	 *	Set the new calendar epoch.
+	 */
+	rtclock_calend.epoch = secs;
+	rtclock_calend.microepoch = microsecs;
+
+	simple_unlock(&rtclock_lock);
+
+	/*
+	 *	Set the new value for the platform clock.
+	 */
 	PESetGMTTimeOfDay(newsecs);
 
+	splx(s);
+
+	/*
+	 *	Send host notifications.
+	 */
 	host_notify_calendar_change();
 }
 
@@ -693,12 +684,7 @@ clock_set_calendar_adjtime(
 		t32 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
 
 		if (t32 > delta)
-			microsys += (t32 - delta);
-
-		if (microsys >= USEC_PER_SEC) {
-			microsys -= USEC_PER_SEC;
-			sys += 1;
-		}
+			TIME_ADD(sys, 0, microsys, (t32 - delta), USEC_PER_SEC);
 
 		rtclock_calend.epoch = sys;
 		rtclock_calend.microepoch = microsys;
@@ -707,12 +693,7 @@ clock_set_calendar_adjtime(
 		now -= (t64 * rtclock_sec_divisor);
 		microsys = (now * USEC_PER_SEC) / rtclock_sec_divisor;
 
-		if ((int32_t)(rtclock_calend.microepoch -= microsys) < 0) {
-			rtclock_calend.microepoch += USEC_PER_SEC;
-			sys	+= 1;
-		}
-
-		rtclock_calend.epoch -= sys;
+		TIME_SUB(rtclock_calend.epoch, sys, rtclock_calend.microepoch, microsys, USEC_PER_SEC);
 	}
 
 	if (total != 0) {
@@ -742,12 +723,7 @@ clock_set_calendar_adjtime(
 			now -= (t64 * rtclock_sec_divisor);
 			microsys = (now * USEC_PER_SEC) / rtclock_sec_divisor;
 
-			if ((rtclock_calend.microepoch += microsys) >= USEC_PER_SEC) {
-				rtclock_calend.microepoch -= USEC_PER_SEC;
-				sys	+= 1;
-			}
-
-			rtclock_calend.epoch += sys;
+			TIME_ADD(rtclock_calend.epoch, sys, rtclock_calend.microepoch, microsys, USEC_PER_SEC);
 		}
 
 		rtclock_calend.adjtotal = total;
@@ -775,7 +751,7 @@ clock_set_calendar_adjtime(
 uint32_t
 clock_adjust_calendar(void)
 {
-	uint32_t		micronew, interval = 0;
+	uint32_t		interval = 0;
 	int32_t			delta;
 	spl_t			s;
 
@@ -785,13 +761,7 @@ clock_adjust_calendar(void)
 	delta = rtclock_calend.adjdelta;
 
 	if (delta > 0) {
-		micronew = rtclock_calend.microepoch + delta;
-		if (micronew >= USEC_PER_SEC) {
-			micronew -= USEC_PER_SEC;
-			rtclock_calend.epoch += 1;
-		}
-
-		rtclock_calend.microepoch = micronew;
+		TIME_ADD(rtclock_calend.epoch, 0, rtclock_calend.microepoch, delta, USEC_PER_SEC);
 
 		rtclock_calend.adjtotal -= delta;
 		if (delta > rtclock_calend.adjtotal)
@@ -813,13 +783,7 @@ clock_adjust_calendar(void)
 
 		t32 = (t64 * USEC_PER_SEC) / rtclock_sec_divisor;
 
-		micronew = rtclock_calend.microepoch + t32 + delta;
-		if (micronew >= USEC_PER_SEC) {
-			micronew -= USEC_PER_SEC;
-			rtclock_calend.epoch += 1;
-		}
-
-		rtclock_calend.microepoch = micronew;
+		TIME_ADD(rtclock_calend.epoch, 0, rtclock_calend.microepoch, (t32 + delta), USEC_PER_SEC);
 
 		rtclock_calend.adjtotal -= delta;
 		if (delta < rtclock_calend.adjtotal)
@@ -832,12 +796,7 @@ clock_adjust_calendar(void)
 			now -= (t64 * rtclock_sec_divisor);
 			microsys = (now * USEC_PER_SEC) / rtclock_sec_divisor;
 
-			if ((int32_t)(rtclock_calend.microepoch -= microsys) < 0) {
-				rtclock_calend.microepoch += USEC_PER_SEC;
-				sys += 1;
-			}
-
-			rtclock_calend.epoch -= sys;
+			TIME_SUB(rtclock_calend.epoch, sys, rtclock_calend.microepoch, microsys, USEC_PER_SEC);
 
 			rtclock_calend.epoch1 = 0;
 		}
@@ -851,6 +810,13 @@ clock_adjust_calendar(void)
 	return (interval);
 }
 
+/*
+ *	clock_initialize_calendar:
+ *
+ *	Set the calendar and related clocks
+ *	from the platform clock at boot or
+ *	wake event.
+ */
 void
 clock_initialize_calendar(void)
 {
@@ -861,23 +827,51 @@ clock_initialize_calendar(void)
 	LOCK_RTC(s);
     commpage_set_timestamp(0,0,0,0);
 
-	clock_get_system_microtime(&sys, &microsys);
-	if ((int32_t)(microsecs -= microsys) < 0) {
-		microsecs += USEC_PER_SEC;
-		secs -= 1;
+	if ((int32_t)secs >= (int32_t)rtclock_boottime) {
+		/*
+		 *	Initialize the boot time based on the platform clock.
+		 */
+		if (rtclock_boottime == 0)
+			rtclock_boottime = secs;
+
+		/*
+		 *	Calculate the new calendar epoch based
+		 *	on the platform clock and the system
+		 *	clock.
+		 */
+		clock_get_system_microtime(&sys, &microsys);
+		TIME_SUB(secs, sys, microsecs, microsys, USEC_PER_SEC);
+
+		/*
+		 *	Set the new calendar epoch.
+		 */
+		rtclock_calend.epoch = secs;
+		rtclock_calend.microepoch = microsecs;
+
+		/*
+		 *	 Cancel any adjustment in progress.
+		 */
+		rtclock_calend.epoch1 = 0;
+		rtclock_calend.adjdelta = rtclock_calend.adjtotal = 0;
 	}
 
-	secs -= sys;
-
-	rtclock_calend.epoch = secs;
-	rtclock_calend.microepoch = microsecs;
-	rtclock_calend.epoch1 = 0;
-	rtclock_calend.adjdelta = rtclock_calend.adjtotal = 0;
 	UNLOCK_RTC(s);
 
+	/*
+	 *	Send host notifications.
+	 */
 	host_notify_calendar_change();
 }
 
+void
+clock_get_boottime_nanotime(
+	uint32_t			*secs,
+	uint32_t			*nanosecs)
+{
+	*secs = rtclock_boottime;
+	*nanosecs = 0;
+}
+
 void
 clock_timebase_info(
 	mach_timebase_info_t	info)
@@ -894,30 +888,25 @@ void
 clock_set_timer_deadline(
 	uint64_t				deadline)
 {
+	int						decr;
 	uint64_t				abstime;
-	int						decr, mycpu;
-	struct rtclock_timer	*mytimer;
+	rtclock_timer_t			*mytimer;
+	struct per_proc_info	*pp;
 	spl_t					s;
 
 	s = splclock();
-	mycpu = cpu_number();
-	mytimer = &rtclock_timer[mycpu];
+	pp = getPerProc();
+	mytimer = &pp->rtclock_timer;
 	mytimer->deadline = deadline;
-	mytimer->is_set = TRUE;
-	if (!mytimer->has_expired) {
-		abstime = mach_absolute_time();
-		if (	mytimer->deadline < rtclock_tick_deadline[mycpu]		) {
-			decr = deadline_to_decrementer(mytimer->deadline, abstime);
-			if (	rtclock_decrementer_min != 0				&&
-					rtclock_decrementer_min < (natural_t)decr		)
-				decr = rtclock_decrementer_min;
 
-			treqs(decr);
+	if (!mytimer->has_expired && (deadline < pp->rtclock_tick_deadline)) {		/* Has the timer already expired or is less that set? */
+		pp->rtcPop = deadline;			/* Yes, set the new rtc pop time */
+		decr = setTimerReq();			/* Start the timers going */
 
-			KERNEL_DEBUG_CONSTANT(MACHDBG_CODE(DBG_MACH_EXCP_DECI, 1)
-										| DBG_FUNC_NONE, decr, 2, 0, 0, 0);
-		}
+		KERNEL_DEBUG_CONSTANT(MACHDBG_CODE(DBG_MACH_EXCP_DECI, 1)
+									| DBG_FUNC_NONE, decr, 2, 0, 0, 0);
 	}
+
 	splx(s);
 }
 
@@ -933,77 +922,73 @@ clock_set_timer_func(
 	UNLOCK_RTC(s);
 }
 
-/*
- * Reset the clock device. This causes the realtime clock
- * device to reload its mode and count value (frequency).
- */
-void
-rtclock_reset(void)
-{
-	return;
-}
-
 /*
  * Real-time clock device interrupt.
  */
 void
-rtclock_intr(
-	int						device,
-	struct savearea			*ssp,
-	spl_t					old_spl)
-{
+rtclock_intr(struct savearea *ssp) {
+	
 	uint64_t				abstime;
-	int						decr1, decr2, mycpu = cpu_number();
-	struct rtclock_timer	*mytimer = &rtclock_timer[mycpu];
+	int						decr;
+	rtclock_timer_t			*mytimer;
+	struct per_proc_info	*pp;
 
-	/*
-	 * We may receive interrupts too early, we must reject them.
-	 */
-	if (rtclock_initialized == FALSE) {
-		treqs(DECREMENTER_MAX);		/* Max the decrementer if not init */
-		return;
-	}
-
-	decr1 = decr2 = DECREMENTER_MAX;
+	pp = getPerProc();
+	mytimer = &pp->rtclock_timer;
 
 	abstime = mach_absolute_time();
-	if (	rtclock_tick_deadline[mycpu] <= abstime		) {
+	if (pp->rtclock_tick_deadline <= abstime) {	/* Have we passed the pop time? */
 		clock_deadline_for_periodic_event(rtclock_tick_interval, abstime,
-										  		&rtclock_tick_deadline[mycpu]);
+										  		&pp->rtclock_tick_deadline);
 		hertz_tick(USER_MODE(ssp->save_srr1), ssp->save_srr0);
+		abstime = mach_absolute_time();			/* Refresh the current time since we went away */
 	}
 
-	abstime = mach_absolute_time();
-	if (	mytimer->is_set					&&
-			mytimer->deadline <= abstime		) {
-		mytimer->has_expired = TRUE; mytimer->is_set = FALSE;
-		(*rtclock_timer_expire)(abstime);
+	if (mytimer->deadline <= abstime) {			/* Have we expired the deadline? */
+		mytimer->has_expired = TRUE;			/* Remember that we popped */
+		mytimer->deadline = EndOfAllTime;		/* Set timer request to the end of all time in case we have no more events */
+		(*rtclock_timer_expire)(abstime);		/* Process pop */
 		mytimer->has_expired = FALSE;
 	}
 
-	abstime = mach_absolute_time();
-	decr1 = deadline_to_decrementer(rtclock_tick_deadline[mycpu], abstime);
-
-	if (mytimer->is_set)
-		decr2 = deadline_to_decrementer(mytimer->deadline, abstime);
-
-	if (decr1 > decr2)
-		decr1 = decr2;
-
-	if (	rtclock_decrementer_min != 0					&&
-			rtclock_decrementer_min < (natural_t)decr1		)
-		decr1 = rtclock_decrementer_min;
-
-	treqs(decr1);
+	pp->rtcPop = (pp->rtclock_tick_deadline < mytimer->deadline) ?	/* Get shortest pop */
+		pp->rtclock_tick_deadline :				/* It was the periodic timer */
+		mytimer->deadline;						/* Actually, an event request */
+	
+	decr = setTimerReq();						/* Request the timer pop */
 
 	KERNEL_DEBUG_CONSTANT(MACHDBG_CODE(DBG_MACH_EXCP_DECI, 1)
-						  | DBG_FUNC_NONE, decr1, 3, 0, 0, 0);
+						  | DBG_FUNC_NONE, decr, 3, 0, 0, 0);
+}
+
+/*
+ *	Request an interruption at a specific time 
+ *
+ *	Sets the decrementer to pop at the right time based on the timebase.
+ *	The value is chosen by comparing the rtc request with the power management.
+ *	request.  We may add other values at a future time.
+ *
+ */
+ 
+int setTimerReq(void) {
+
+	struct per_proc_info *pp;
+	int decr;
+	uint64_t nexttime;
+	
+	pp = getPerProc();							/* Get per_proc */
+
+	nexttime = pp->rtcPop;						/* Assume main timer */
+
+	decr = setPop((pp->pms.pmsPop < nexttime) ? pp->pms.pmsPop : nexttime);	/* Schedule timer pop */
+
+	return decr;								/* Pass back what we actually set */
 }
 
 static void
 rtclock_alarm_expire(
-	timer_call_param_t		p0,
-	timer_call_param_t		p1)
+	__unused void	*p0,
+	__unused void	*p1)
 {
 	mach_timespec_t		timestamp;
 
@@ -1012,31 +997,30 @@ rtclock_alarm_expire(
 	clock_alarm_intr(SYSTEM_CLOCK, &timestamp);
 }
 
-static int
-deadline_to_decrementer(
-	uint64_t		deadline,
-	uint64_t		now)
-{
-	uint64_t		delt;
-
-	if (deadline <= now)
-		return DECREMENTER_MIN;
-	else {
-		delt = deadline - now;
-		return (delt >= (DECREMENTER_MAX + 1))? DECREMENTER_MAX:
-				((delt >= (DECREMENTER_MIN + 1))? (delt - 1): DECREMENTER_MIN);
-	}
-}
-
 static void
-timespec_to_absolutetime(
-	mach_timespec_t		*ts,
+nanotime_to_absolutetime(
+	uint32_t			secs,
+	uint32_t			nanosecs,
 	uint64_t			*result)
 {
+	uint32_t	divisor = rtclock_sec_divisor;
+
+	*result = ((uint64_t)secs * divisor) +
+				((uint64_t)nanosecs * divisor) / NSEC_PER_SEC;
+}
+
+void
+absolutetime_to_microtime(
+	uint64_t			abstime,
+	uint32_t			*secs,
+	uint32_t			*microsecs)
+{
+	uint64_t	t64;
 	uint32_t	divisor;
 
-	*result = ((uint64_t)ts->tv_sec * (divisor = rtclock_sec_divisor)) +
-				((uint64_t)ts->tv_nsec * divisor) / NSEC_PER_SEC;
+	*secs = t64 = abstime / (divisor = rtclock_sec_divisor);
+	abstime -= (t64 * divisor);
+	*microsecs = (abstime * USEC_PER_SEC) / divisor;
 }
 
 void
@@ -1107,25 +1091,8 @@ nanoseconds_to_absolutetime(
 	*result += (nanosecs * divisor) / NSEC_PER_SEC;
 }
 
-/*
- * Spin-loop delay primitives.
- */
 void
-delay_for_interval(
-	uint32_t		interval,
-	uint32_t		scale_factor)
-{
-	uint64_t		now, end;
-
-	clock_interval_to_deadline(interval, scale_factor, &end);
-
-	do {
-		now = mach_absolute_time();
-	} while (now < end);
-}
-
-void
-clock_delay_until(
+machine_delay_until(
 	uint64_t		deadline)
 {
 	uint64_t		now;
@@ -1135,30 +1102,3 @@ clock_delay_until(
 	} while (now < deadline);
 }
 
-void
-delay(
-	int		usec)
-{
-	delay_for_interval((usec < 0)? -usec: usec, NSEC_PER_USEC);
-}
-
-/*
- *	Request a decrementer pop
- *
- */
-
-void treqs(uint32_t dec) {
-
-
-	struct per_proc_info *pp;
-	uint64_t nowtime, newtime;
-	
-	nowtime = mach_absolute_time();						/* What time is it? */
-	pp = getPerProc();									/* Get our processor block */
-	newtime = nowtime + (uint64_t)dec;					/* Get requested pop time */
-	pp->rtcPop = newtime;								/* Copy it */
-	
-	mtdec((uint32_t)(newtime - nowtime));				/* Set decrementer */
-	return;
-
-}
