@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -89,6 +90,9 @@ extern
 mach_port_t		convert_mo_control_to_port(
 				memory_object_control_t	control);
 
+extern void memory_object_control_disable(
+	memory_object_control_t	control);
+
 extern
 memory_object_control_t convert_port_to_mo_control(
 				mach_port_t		port);
@@ -101,9 +105,30 @@ extern
 memory_object_t		convert_port_to_memory_object(
 				mach_port_t		port);
 
+extern upl_t convert_port_to_upl(
+				ipc_port_t	port);
+
+extern ipc_port_t convert_upl_to_port( upl_t );
+
+__private_extern__ void upl_no_senders(ipc_port_t, mach_port_mscount_t);
+
 extern kern_return_t	memory_object_free_from_cache(
 				host_t		host,
 				int		*pager_id,
 				int		*count);
 
+extern kern_return_t	memory_object_iopl_request(
+	ipc_port_t		port,
+	memory_object_offset_t	offset,
+	vm_size_t		*upl_size,
+	upl_t			*upl_ptr,
+	upl_page_info_array_t	user_page_list,
+	unsigned int		*page_list_count,
+	int			*flags);
+	
+
+extern kern_return_t	memory_object_pages_resident(
+	memory_object_control_t		control,
+	boolean_t			*		has_pages_resident);
+
 #endif	/* _VM_MEMORY_OBJECT_H_ */
