@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -39,9 +37,4 @@ extern unsigned int	vm_page_info(
 					hash_info_bucket_t	*info,
 					unsigned int		count);
 
-extern int		pmap_list_resident_pages(
-					pmap_t		pmap,
-					vm_offset_t	*listp,
-					int		space);
-
 #endif	/* VM_VM_DEBUG_H */
