@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -58,16 +59,17 @@
 #ifndef	_VM_VM_FAULT_H_
 #define _VM_VM_FAULT_H_
 
+#include <mach/mach_types.h>
 #include <mach/kern_return.h>
 #include <mach/boolean.h>
 #include <mach/vm_prot.h>
 #include <mach/vm_param.h>
 #include <mach/vm_behavior.h>
-#include <vm/vm_page.h>
-#include <vm/vm_object.h>
-#include <vm/vm_map.h>
+
+#ifdef	KERNEL_PRIVATE
 
 typedef	kern_return_t	vm_fault_return_t;
+
 #define VM_FAULT_SUCCESS		0
 #define VM_FAULT_RETRY			1
 #define VM_FAULT_INTERRUPTED		2
@@ -75,6 +77,25 @@ typedef	kern_return_t	vm_fault_return_t;
 #define VM_FAULT_FICTITIOUS_SHORTAGE 	4
 #define VM_FAULT_MEMORY_ERROR		5
 
+/*
+ *	Page fault handling based on vm_map (or entries therein)
+ */
+
+extern kern_return_t vm_fault(
+		vm_map_t	map,
+		vm_map_offset_t	vaddr,
+		vm_prot_t	fault_type,
+		boolean_t	change_wiring,
+		int             interruptible,
+		pmap_t		pmap,
+		vm_map_offset_t	pmap_addr);
+
+#ifdef	MACH_KERNEL_PRIVATE
+
+#include <vm/vm_page.h>
+#include <vm/vm_object.h>
+#include <vm/vm_map.h>
+
 extern void vm_fault_init(void);
 
 /*
@@ -88,8 +109,8 @@ extern vm_fault_return_t vm_fault_page(
 		vm_prot_t	fault_type,	/* What access is requested */
 		boolean_t	must_be_resident,/* Must page be resident? */
 		int		interruptible,/* how may fault be interrupted */
-		vm_object_offset_t lo_offset,	/* Map entry start */
-		vm_object_offset_t hi_offset,	/* Map entry end */
+		vm_map_offset_t lo_offset,	/* Map entry start */
+		vm_map_offset_t hi_offset,	/* Map entry end */
 		vm_behavior_t	behavior,	/* Expected paging behavior */
 		/* Modifies in place: */
 		vm_prot_t	*protection,	/* Protection for mapping */
@@ -104,45 +125,37 @@ extern vm_fault_return_t vm_fault_page(
 		boolean_t	no_zero_fill,	/* don't fill absent pages */
 		boolean_t	data_supply,	/* treat as data_supply */
 		vm_map_t	map,
-		vm_offset_t	vaddr);
+		vm_map_offset_t	vaddr);
 
 extern void vm_fault_cleanup(
 		vm_object_t	object,
 		vm_page_t	top_page);
-/*
- *	Page fault handling based on vm_map (or entries therein)
- */
-
-extern kern_return_t vm_fault(
-		vm_map_t	map,
-		vm_offset_t	vaddr,
-		vm_prot_t	fault_type,
-		boolean_t	change_wiring,
-		int             interruptible,
-		pmap_t		pmap,
-		vm_offset_t	pmap_addr);
 
 extern kern_return_t vm_fault_wire(
 		vm_map_t	map,
 		vm_map_entry_t	entry,
 		pmap_t		pmap,
-		vm_offset_t	pmap_addr);
+		vm_map_offset_t	pmap_addr);
 
 extern void vm_fault_unwire(
 		vm_map_t	map,
 		vm_map_entry_t	entry,
 		boolean_t	deallocate,
 		pmap_t		pmap,
-		vm_offset_t	pmap_addr);
+		vm_map_offset_t	pmap_addr);
 
 extern kern_return_t	vm_fault_copy(
 		vm_object_t		src_object,
 		vm_object_offset_t	src_offset,
-		vm_size_t		*src_size,	/* INOUT */
+		vm_map_size_t		*copy_size,	/* INOUT */
 		vm_object_t		dst_object,
 		vm_object_offset_t	dst_offset,
 		vm_map_t		dst_map,
 		vm_map_version_t	 *dst_version,
 		int			interruptible);
 
+#endif	/* MACH_KERNEL_PRIVATE */
+
+#endif	/* KERNEL_PRIVATE */
+
 #endif	/* _VM_VM_FAULT_H_ */
