@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -41,7 +42,7 @@ extern int	vm_follow_object(
 			vm_object_t	object);
 
 extern void	vm_object_print(
-			vm_object_t	object,
+			db_addr_t	object,
 			boolean_t	have_addr,
 			int		arg_count,
 			char		*modif);
@@ -49,7 +50,7 @@ extern void	vm_object_print(
 #include <vm/vm_page.h>
 
 extern void	vm_page_print(
-			vm_page_t	p);
+			db_addr_t	p);
 
 #include <mach_pagemap.h>
 #if	MACH_PAGEMAP
@@ -61,7 +62,7 @@ extern void vm_external_print(
 
 extern void	db_vm(void);
 
-extern vm_size_t db_vm_map_total_size(
+extern vm_map_size_t db_vm_map_total_size(
 			db_addr_t	map);
 
 #endif	/* VM_PRINT_H */
