@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -25,14 +23,19 @@
 
 extern dt_data gMemoryMapNode;
 extern dt_data gDriversProp;
+extern dt_data gRootpathProp;
+extern dt_data gCompatibleProp;
+
+#define kDefaultPlatformName "ACPI"
 
 dt_init fakePPCDeviceTree[] =
 {
     NODE( 2, 1 ),
         PROP( "name", "device-tree"),
-        PROP( "compatible", "X86PC"),
-        NODE( 1, 1 ),
+        STRINGPROP( "compatible", gCompatibleProp ),
+        NODE( 2, 1 ),
             PROP( "name", "chosen" ),
+            STRINGPROP( "rootpath", gRootpathProp ),
             DATANODE( gMemoryMapNode ),
                 PROP( "name", "memory-map" ),
                 DATAPROP( gDriversProp ),
