@@ -3,29 +3,33 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
 #include <pexpert/pexpert.h>
 #include <pexpert/protos.h>
 #include <machine/machine_routines.h>
-#include <i386/machine_cpu.h>
 #include <i386/mp.h>
 #include <sys/kdebug.h>
 
+
+void PE_incoming_interrupt(int, void *);
+
+
 struct i386_interrupt_handler {
 	IOInterruptHandler	handler;
 	void			*nub;
@@ -37,10 +41,6 @@ typedef struct i386_interrupt_handler i386_interrupt_handler_t;
 
 i386_interrupt_handler_t	PE_interrupt_handler;
 
-void PE_platform_interrupt_initialize(void)
-{
-}
-
 
 
 void
@@ -49,17 +49,11 @@ PE_incoming_interrupt(int interrupt, void *state)
 	i386_interrupt_handler_t	*vector;
 
 	KERNEL_DEBUG_CONSTANT(MACHDBG_CODE(DBG_MACH_EXCP_INTR, 0) | DBG_FUNC_START,
-			      0, ((unsigned int *)state)[5], 0, 0, 0);
+			      0, ((unsigned int *)state)[7], 0, 0, 0);
 
 	vector = &PE_interrupt_handler;
 
-	switch (interrupt) {
-	case APIC_ERROR_INTERRUPT:
-	case SPURIOUS_INTERRUPT:
-	case INTERPROCESS_INTERRUPT:
-		lapic_interrupt(interrupt, state);
-		break;
-	default:
+	if (!lapic_interrupt(interrupt, state)) {
 		vector->handler(vector->target, state, vector->nub, interrupt);
 	}
 
@@ -67,7 +61,8 @@ PE_incoming_interrupt(int interrupt, void *state)
 	   0, 0, 0, 0, 0);
 }
 
-void PE_install_interrupt_handler(void *nub, int source,
+void PE_install_interrupt_handler(void *nub,
+				  __unused int source,
 				  void *target,
 				  IOInterruptHandler handler,
 				  void *refCon)
