@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -27,7 +25,7 @@
 
 #include <mach/ppc/thread_status.h> /* for struct ppc_saved_state */
 
-extern void	(PE_incoming_interrupt)(int type, struct ppc_saved_state *ssp,
+extern void	(PE_incoming_interrupt)(int type, ppc_saved_state_t *ssp,
 					unsigned int dsisr, unsigned int dar);
 
 #endif /* POWERMAC_INTERRUPTS_H_ */
