@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -23,7 +24,6 @@
  * file: pe_init.c
  *    PPC platform expert initialization.
  */
-#include <mach/boot_info.h>
 #include <mach/time_value.h>
 #include <pexpert/protos.h>
 #include <pexpert/pexpert.h>
@@ -33,6 +33,7 @@
 #include <kern/debug.h>
 #include <kern/sched_prim.h>
 
+
 /* extern references */
 void pe_identify_machine(void);
 
@@ -149,7 +150,13 @@ void PE_init_iokit(void)
 
 void PE_init_platform(boolean_t vm_initialized, void *_args)
 {
-        boot_args *args = (boot_args *)_args;
+	DTEntry dsouth, dnorth, root, dcpu;
+	char *model;
+	int msize, size;
+	uint32_t *south, *north, *pdata, *ddata;
+	int i;
+	
+	boot_args *args = (boot_args *)_args;
 
 	if (PE_state.initialized == FALSE)
 	{
@@ -180,6 +187,7 @@ void PE_init_platform(boolean_t vm_initialized, void *_args)
 	else
 	{
 	    pe_init_debug();
+	
 	}
 }
 
